#<profile4xi profile = "12" />
# =================================================================================================
#<program4xi program = "gpu2kernel4xi.py">
#<abstract4xi abstract = "GPU kernel management"/>
#<refer4xi reference = "kernel4xi.py"/>
#<verify4xi verification = "10" target = "gpu2kernel4xi5test.py">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting; [Rel 7.21]"/>
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# Public API  
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'GPU2Kernel4Xi',                       # ACK 5.7.1  p157    

            'add2kernel4gpu',                      # ACK               

            'execute2kernel4gpu',                  # ACK 5.8    p169                                             
            'execute2task4gpu',                    # ACK 5.8    p172   
            'execute2nativekernel4gpu',            # ACK 5.8    p174     
            
            'create2kernel4gpu',                   # ACK 5.7.1  p157   
            'create2kernelsinprogram4gpu',         # ACK 5.7.1  p158  
                       
            'delete2kernel4gpu',                   # ACK

            'fetch2kernel4gpu',                    # ACK 
            
            'get2kernelinfo4gpu',                  # ACK 5.7.3  p162  
            'get2kernelworkgroupinfo4gpu',         # ACK 5.7.3  p164  
            'get2kernelarginfo4gpu',               # ACK 5.7.3  p166                                  
            
            'release2kernel4gpu',                  # ACK 5.7.1  p159                                         
            'retain2kernel4gpu',                   # ACK 5.7.1  p159   
 
            'set2kernel4gpu',                      # ACK 
            'set2kernelcallback4gpu',              # ACK 
            'set2kernelarg4gpu',                   # ACK 5.7.2  p160 
                                          
            'trace2kernel4gpu'                     # ACK 

          ] 
#"
#/>


# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os
from platform import processor
import sys
import threading                
import _thread   
from decimal import *
#"
#/>


# -------------------------------------------------------------------------------------------------
# Scope4Xi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
import version4xi
from version4xi import VersionKB4Xi

# -------------------------------------------------------------------------------------------------
# Buffer                           
# -------------------------------------------------------------------------------------------------
from gpu2buffer4xi import get2bufferinfo4gpu

# -------------------------------------------------------------------------------------------------
# Command queue module                           
# -------------------------------------------------------------------------------------------------
from gpu2commandqueue4xi import GPU2CommandQueue4Xi
from gpu2commandqueue4xi import create2commandqueue4gpu 
from gpu2commandqueue4xi import flush2commandqueue4gpu
from gpu2commandqueue4xi import finish2commandqueue4gpu
from gpu2commandqueue4xi import get2commandqueueinfo4gpu
from gpu2commandqueue4xi import release2commandqueue4gpu
from gpu2commandqueue4xi import retain2commandqueue4gpu
from gpu2commandqueue4xi import set2commandqueue4gpu
from gpu2commandqueue4xi import trace2commandqueue4gpu

# -------------------------------------------------------------------------------------------------
# Context module                           
# -------------------------------------------------------------------------------------------------
from gpu2context4xi import GPU2Context4Xi 
from gpu2context4xi import create2context4gpu
from gpu2context4xi import get2context4gpu
from gpu2context4xi import release2context4gpu
from gpu2context4xi import retain2context4gpu
from gpu2context4xi import set2context4gpu


# -------------------------------------------------------------------------------------------------
# Device module                        
# -------------------------------------------------------------------------------------------------
from gpu2device4xi import GPU4Xi
from gpu2device4xi import init2gpu4xi 
from gpu2device4xi import set2gpu4xi
from gpu2device4xi import get2gpu4xi

# -------------------------------------------------------------------------------------------------
# Event module                           
# -------------------------------------------------------------------------------------------------
from gpu2event4xi import GPU2Event4Xi
from gpu2event4xi import create2event4gpu
from gpu2event4xi import get2eventinfo4gpu
from gpu2event4xi import release2event4gpu
from gpu2event4xi import retain2event4gpu
from gpu2event4xi import set2eventcallback4gpu 
from gpu2event4xi import set2event4gpu
from gpu2event4xi import trace2event4gpu
from gpu2event4xi import wait2event4gpu


# -------------------------------------------------------------------------------------------------
# Platform module                           
# -------------------------------------------------------------------------------------------------
from platformids4xi  import get2platformids4xi
from platforminfo4xi import PlatformInfo4Xi                                                                        
from platforminfo4xi import init2platforminfo4xi
from platforminfo4xi import set2platforminfo4xi
from platforminfo4xi import get2platforminfo4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Program module                           
# -------------------------------------------------------------------------------------------------
from gpu2program4xi import GPU2Program4Xi
from gpu2program4xi import add2programwithsource4gpu
from gpu2program4xi import add2programwithbinary4gpu
from gpu2program4xi import build2program4gpu
from gpu2program4xi import compile2program4gpu
from gpu2program4xi import create2programwithsource4gpu
from gpu2program4xi import create2programwithbinary4gpu
from gpu2program4xi import delete2programwithsource4gpu
from gpu2program4xi import delete2programwithbinary4gpu
from gpu2program4xi import get2programinfo4gpu
# from gpu2program4xi import get2programbuildinfo4gpu
from gpu2program4xi import link2program4gpu
from gpu2program4xi import release2program4gpu
from gpu2program4xi import retain2program4gpu
from gpu2program4xi import set2programcallback4gpu 
from gpu2program4xi import set2programwithsource4gpu
from gpu2program4xi import set2programwithbinary4gpu
from gpu2program4xi import trace2program4gpu
# from gpu2program4xi import unload2compiler4gpu

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from arena4xi import Arena4Xi
from cpu_corecount4xi import cpu_corecount4xi
from dummyprocess4xi import DummyProcess4Xi
from print4xi import print4xi
from pipe4xi import Pipe4Xi
from process4xi import Process4Xi
from queue4xi import Queue4Xi  
from time4xi import time4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi


#"
#/>

# -------------------------------------------------------------------------------------------------
# Global constants                         
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global gpu2kernel4xi                                                    # Kernel of GPU
global gpu2build4xi                                                     # Build of GPU
global gpu2compile4xi                                                   # Compile of GPU
global gpu2trace4xi                                                     # TRACE

global fetch2code4gpu                                                   # Fetched code

global import2use4gpu                                                   # Imports of target function for Process4Xi

global kb2fetch4gpu                                                     # Knowledgebase for fetching
global kb2kernel4gpu                                                    # Knowledgebase for kernel   

global folder2lib4gpu                                                   # Folder of GPU library

global profile2fetch                                                       # Profile knowledgebase for fetching
global keys2profile4gpu

global target2function4gpu                                              # Target function for Process4Xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "
temp2gpu = 'gpu'
key2dir = temp2gpu[0]                                                                                               # MORE COMPLICATED WAY DUE TO AUTOMATIC SOFTWARE GENERATION
folder2lib4gpu = VersionKB4Xi[key2dir] 

profile2fetch = {}                                                                                                  # PROFILES FOR FETCHING
profile2fetch ['.py']  = 'ACK'                                                                                    
profile2fetch ['.pyc'] = 'ACK'                                                                                    
profile2fetch ['.cpp'] = 'ACK'                                                                                   
profile2fetch ['.exe'] = 'ACK'       
keys2profile4gpu = profile2fetch.keys()

kb2fetch4gpu = {}                                                                                                   # PROGRAMS TO BE FETCHED - ALWAYS DEFINE BOTH SOURCE AND BINARY PAIRS                               
kb2fetch4gpu[1] = folder2lib4gpu + 'gpu2sampler4xi5test.py'                                                         # Python code
kb2fetch4gpu[2] = folder2lib4gpu + '__pycache__//gpu2sampler4xi.cpython-33.pyc'   
kb2fetch4gpu[3] = folder2lib4gpu + 'gpu2dummy4xi5test.cpp'                                                          # C++ code
kb2fetch4gpu[4] = folder2lib4gpu + "bin//gpu2dummy4xi5test.exe"                                                     # C++ binary

# kb2fetch4gpu[1]  = folder2lib4gpu + 'gpu2sampler4xi5test.py'                                                      
# kb2fetch4gpu[2]  = folder2lib4gpu + '__pycache__//gpu2sampler4xi.cpython-33.pyc'                                  
# kb2fetch4gpu[3]  = folder2lib4gpu + 'gpu2image4xi5test.py'
# kb2fetch4gpu[4]  = folder2lib4gpu + '__pycache__//gpu2image4xi.cpython-33.pyc'
# kb2fetch4gpu[5]  = folder2lib4gpu + 'gpu3image4xi5test.py'
# kb2fetch4gpu[6]  = folder2lib4gpu + '__pycache__//gpu3image4xi.cpython-33.pyc'
# kb2fetch4gpu[7]  = folder2lib4gpu + 'gpu2commandqueue4xi5test.py'
# kb2fetch4gpu[8]  = folder2lib4gpu + '__pycache__//gpu2commandqueue4xi.cpython-33.pyc'
# kb2fetch4gpu[9]  = folder2lib4gpu + 'gpu2event4xi5test.py'
# kb2fetch4gpu[10] = folder2lib4gpu + '__pycache__//gpu2event4xi.cpython-33.pyc'
# kb2fetch4gpu[10]  = folder2lib4gpu + 'GPU2kernel4xi5test.py'
# kb2fetch4gpu[11] = folder2lib4gpu + '__pycache__//gpu2program.cpython-33.pyc'

fetch2code4gpu = {}                                                                                                 # reception arena in memory for code

kb2kernel4gpu = {}
kb2kernel4gpu ['.py'] = {}
kb2kernel4gpu ['.py'] ['TAG']            = "<function4xi function ="                                                
#</function4xi>                                                                                                     # added for parsing!
kb2kernel4gpu ['.py'] ['VALUE']          = "gpu2sampler4xi5test" 
kb2kernel4gpu ['.py'] ['ATTRIBUTE']      = 'usage = "' 

kb2kernel4gpu ['.cpp'] = {}
kb2kernel4gpu ['.cpp'] ['TAG']            = "<function4xi function ="                                               
#</function4xi>                                                                                                     # added for parsing!
kb2kernel4gpu ['.cpp'] ['VALUE']          = "gpu2dummy4xi5test" 
kb2kernel4gpu ['.cpp'] ['ATTRIBUTE']      = 'usage = "kernel'                                                       

#"/>

# -------------------------------------------------------------------------------------------------
# Command queue module of GPU imports for target function                           
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2commandqueue4gpu" usage = " importing target function for Process4Xi">
def importcreate2commandqueue4gpu():
   global target2function4gpu
   from gpu2commandqueue4xi import create2commandqueue4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importflush2commandqueue4gpu" usage = " importing target function for Process4Xi">
def importflush2commandqueue4gpu():
   global target2function4gpu
   from gpu2commandqueue4xi import flush2commandqueue4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importfinish2commandqueue4gpu" usage = " importing target function for Process4Xi">
def importfinish2commandqueue4gpu():
   global target2function4gpu
   from gpu2commandqueue4xi import finish2commandqueue4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importget2commandqueueinfo4gpu" usage = " importing target function for Process4Xi">
def importget2commandqueueinfo4gpu():
   global target2function4gpu
   from gpu2commandqueue4xi import get2commandqueueinfo4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importrelease2commandqueue4gpu" usage = " importing target function for Process4Xi">
def importrelease2commandqueue4gpu():
   global target2function4gpu
   from gpu2commandqueue4xi import release2commandqueue4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importretain2commandqueue4gpu" usage = " importing target function for Process4Xi">
def importretain2commandqueue4gpu():
   global target2function4gpu
   from gpu2commandqueue4xi import retain2commandqueue4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importset2commandqueue4gpu" usage = " importing target function for Process4Xi">
def importset2commandqueue4gpu():
   global target2function4gpu
   from gpu2commandqueue4xi import set2commandqueue4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importtrace2commandqueue4gpu" usage = " importing target function for Process4Xi">
def importtrace2commandqueue4gpu():
   global target2function4gpu
   from gpu2commandqueue4xi import trace2commandqueue4gpu as target2function4gpu
#</function4xi>
 

# -------------------------------------------------------------------------------------------------
# Context module of GPU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2context4gpu" usage = " importing target function for Process4Xi">
def importcreate2context4gpu():
   global target2function4gpu
   from gpu2context4xi import create2context4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importget2context4gpu" usage = " importing target function for Process4Xi">
def importget2context4gpu():
   global target2function4gpu
   from gpu2context4xi import get2context4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importrelease2context4gpu" usage = " importing target function for Process4Xi">
def importrelease2context4gpu():
   global target2function4gpu
   from gpu2context4xi import release2context4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importretain2context4gpu" usage = " importing target function for Process4Xi">
def importretain2context4gpu():
   global target2function4gpu
   from gpu2context4xi import retain2context4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importset2context4gpu" usage = " importing target function for Process4Xi">
def importset2context4gpu():
   global target2function4gpu
   from gpu2context4xi import set2context4gpu as target2function4gpu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Device module of GPU imports for target function                           
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importinit2gpu4xi" usage = " importing target function for Process4Xi">
def importinit2gpu4xi():
   global target2function4gpu
   from gpu2device4xi import init2gpu4xi as target2function4gpu 
#</function4xi>
 
#<function4xi function = "importset2gpu4xi" usage = " importing target function for Process4Xi">
def importset2gpu4xi():
   global target2function4gpu
   from gpu2device4xi import set2gpu4xi   as target2function4gpu
#</function4xi>
 
#<function4xi function = "importget2gpu4xi" usage = " importing target function for Process4Xi">
def importget2gpu4xi():
   global target2function4gpu
   from gpu2device4xi import get2gpu4xi   as target2function4gpu
#</function4xi>
 

# -------------------------------------------------------------------------------------------------
# Event module of GPU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2event4gpu" usage = " importing target function for Process4Xi">
def importcreate2event4gpu():
   global target2function4gpu
   from gpu2event4xi import create2event4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importget2eventinfo4gpu" usage = " importing target function for Process4Xi">
def importget2eventinfo4gpu():
   global target2function4gpu
   from gpu2event4xi import get2eventinfo4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importrelease2event4gpu" usage = " importing target function for Process4Xi">
def importrelease2event4gpu():
   global target2function4gpu
   from gpu2event4xi import release2event4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importretain2event4gpu" usage = " importing target function for Process4Xi">
def importretain2event4gpu():
   global target2function4gpu
   from gpu2event4xi import retain2event4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importset2eventcallback4gpu" usage = " importing target function for Process4Xi">
def importset2eventcallback4gpu():
   global target2function4gpu
   from gpu2event4xi import set2eventcallback4gpu as target2function4gpu 
#</function4xi>
 
#<function4xi function = "importset2event4gpu" usage = " importing target function for Process4Xi">
def importset2event4gpu():
   global target2function4gpu
   from gpu2event4xi import set2event4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importtrace2event4gpu" usage = " importing target function for Process4Xi">
def importtrace2event4gpu():
   global target2function4gpu
   from gpu2event4xi import trace2event4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importwait2event4gpu" usage = " importing target function for Process4Xi">
def importwait2event4gpu():
   global target2function4gpu
   from gpu2event4xi import wait2event4gpu as target2function4gpu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# 2D Image module of GPU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2image4gpu" usage = " importing target function for Process4Xi">
def importcreate2image4gpu():
   global target2function4gpu
   from gpu2image4xi import create2image4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importget2imageinfo4gpu" usage = " importing target function for Process4Xi">
def importget2imageinfo4gpu():
   global target2function4gpu
   from gpu2image4xi import get2imageinfo4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importget2supportedimageformats4gpu" usage = " importing target function for Process4Xi">
def importget2supportedimageformats4gpu():
   global target2function4gpu
   from gpu2image4xi import get2supportedimageformats4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre2copyimage4gpu" usage = " importing target function for Process4Xi">
def importmanoeuvre2copyimage4gpu():
   global target2function4gpu
   from gpu2image4xi import manoeuvre2copyimage4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre2copyimagetobuffer4gpu" usage = " importing target function for Process4Xi">
def importmanoeuvre2copyimagetobuffer4gpu():
   global target2function4gpu
   from gpu2image4xi import manoeuvre2copyimagetobuffer4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre2copybuffertoimage4gpu" usage = " importing target function for Process4Xi">
def importmanoeuvre2copybuffertoimage4gpu():
   global target2function4gpu
   from gpu2image4xi import manoeuvre2copybuffertoimage4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre2readimage4gpu" usage = " importing target function for Process4Xi">
def importmanoeuvre2readimage4gpu():
   global target2function4gpu
   from gpu2image4xi import manoeuvre2readimage4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre2writeimage4gpu" usage = " importing target function for Process4Xi">
def importmanoeuvre2writeimage4gpu():
   global target2function4gpu
   from gpu2image4xi import manoeuvre2writeimage4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre2mapimage4gpu" usage = " importing target function for Process4Xi">
def importmanoeuvre2mapimage4gpu():
   global target2function4gpu
   from gpu2image4xi import manoeuvre2mapimage4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importcopy2image4gpu" usage = " importing target function for Process4Xi">
def importcopy2image4gpu():
   global target2function4gpu
   from gpu2image4xi import copy2image4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importcopy2imagetobuffer4gpu" usage = " importing target function for Process4Xi">
def importcopy2imagetobuffer4gpu():
   global target2function4gpu
   from gpu2image4xi import copy2imagetobuffer4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importcopy2buffertoimage4gpu" usage = " importing target function for Process4Xi">
def importcopy2buffertoimage4gpu():
   global target2function4gpu
   from gpu2image4xi import copy2buffertoimage4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importread2image4gpu" usage = " importing target function for Process4Xi">
def importread2image4gpu():
   global target2function4gpu
   from gpu2image4xi import read2image4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importmap2image4gpu" usage = " importing target function for Process4Xi">
def importmap2image4gpu():
   global target2function4gpu
   from gpu2image4xi import map2image4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importrelease2image4gpu" usage = " importing target function for Process4Xi">
def importrelease2image4gpu():
   global target2function4gpu
   from gpu2image4xi import release2image4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importretain2image4gpu" usage = " importing target function for Process4Xi">
def importretain2image4gpu():
   global target2function4gpu
   from gpu2image4xi import retain2image4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importset2imagecallback4gpu" usage = " importing target function for Process4Xi">
def importset2imagecallback4gpu():
   global target2function4gpu
   from gpu2image4xi import set2imagecallback4gpu as target2function4gpu 
#</function4xi>
 
#<function4xi function = "importset2image4gpu" usage = " importing target function for Process4Xi">
def importset2image4gpu():
   global target2function4gpu
   from gpu2image4xi import set2image4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importtrace2image4gpu" usage = " importing target function for Process4Xi">
def importtrace2image4gpu():
   global target2function4gpu
   from gpu2image4xi import trace2image4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importwait2image4gpu" usage = " importing target function for Process4Xi">
def importwait2image4gpu():
   global target2function4gpu
   from gpu2image4xi import wait2image4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importwrite2image4gpu" usage = " importing target function for Process4Xi">
def importwrite2image4gpu():
   global target2function4gpu
   from gpu2image4xi import write2image4gpu as target2function4gpu
#</function4xi>
 

# -------------------------------------------------------------------------------------------------
# 3D Image module of GPU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate3image4gpu" usage = " importing target function for Process4Xi">
def importcreate3image4gpu():
   global target2function4gpu
   from gpu3image4xi import create3image4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importget3imageinfo4gpu" usage = " importing target function for Process4Xi">
def importget3imageinfo4gpu():
   global target2function4gpu
   from gpu3image4xi import get3imageinfo4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importget3supportedimageformats4gpu" usage = " importing target function for Process4Xi">
def importget3supportedimageformats4gpu():
   global target2function4gpu
   from gpu3image4xi import get3supportedimageformats4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre3copyimage4gpu" usage = " importing target function for Process4Xi">
def importmanoeuvre3copyimage4gpu():
   global target2function4gpu
   from gpu3image4xi import manoeuvre3copyimage4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre3copyimagetobuffer4gpu" usage = " importing target function for Process4Xi">
def importmanoeuvre3copyimagetobuffer4gpu():
   global target2function4gpu
   from gpu3image4xi import manoeuvre3copyimagetobuffer4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre3copybuffertoimage4gpu" usage = " importing target function for Process4Xi">
def importmanoeuvre3copybuffertoimage4gpu():
   global target2function4gpu
   from gpu3image4xi import manoeuvre3copybuffertoimage4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre3readimage4gpu" usage = " importing target function for Process4Xi">
def importmanoeuvre3readimage4gpu():
   global target2function4gpu
   from gpu3image4xi import manoeuvre3readimage4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre3writeimage4gpu" usage = " importing target function for Process4Xi">
def importmanoeuvre3writeimage4gpu():
   global target2function4gpu
   from gpu3image4xi import manoeuvre3writeimage4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importmanoeuvre3mapimage4gpu" usage = " importing target function for Process4Xi">
def importmanoeuvre3mapimage4gpu():
   global target2function4gpu
   from gpu3image4xi import manoeuvre3mapimage4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importcopy3image4gpu" usage = " importing target function for Process4Xi">
def importcopy3image4gpu():
   global target2function4gpu
   from gpu3image4xi import copy3image4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importcopy3imagetobuffer4gpu" usage = " importing target function for Process4Xi">
def importcopy3imagetobuffer4gpu():
   global target2function4gpu
   from gpu3image4xi import copy3imagetobuffer4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importcopy3buffertoimage4gpu" usage = " importing target function for Process4Xi">
def importcopy3buffertoimage4gpu():
   global target2function4gpu
   from gpu3image4xi import copy3buffertoimage4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importread3image4gpu" usage = " importing target function for Process4Xi">
def importread3image4gpu():
   global target2function4gpu
   from gpu3image4xi import read3image4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importmap3image4gpu" usage = " importing target function for Process4Xi">
def importmap3image4gpu():
   global target2function4gpu
   from gpu3image4xi import map3image4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importrelease3image4gpu" usage = " importing target function for Process4Xi">
def importrelease3image4gpu():
   global target2function4gpu
   from gpu3image4xi import release3image4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importretain3image4gpu" usage = " importing target function for Process4Xi">
def importretain3image4gpu():
   global target2function4gpu
   from gpu3image4xi import retain3image4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importset3imagecallback4gpu" usage = " importing target function for Process4Xi">
def importset3imagecallback4gpu():
   global target2function4gpu
   from gpu3image4xi import set3imagecallback4gpu as target2function4gpu 
#</function4xi>
 
#<function4xi function = "importset3image4gpu" usage = " importing target function for Process4Xi">
def importset3image4gpu():
   global target2function4gpu
   from gpu3image4xi import set3image4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importtrace3image4gpu" usage = " importing target function for Process4Xi">
def importtrace3image4gpu():
   global target2function4gpu
   from gpu3image4xi import trace3image4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importwait3image4gpu" usage = " importing target function for Process4Xi">
def importwait3image4gpu():
   global target2function4gpu
   from gpu3image4xi import wait3image4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importwrite3image4gpu" usage = " importing target function for Process4Xi">
def importwrite3image4gpu():
   global target2function4gpu
   from gpu3image4xi import write3image4gpu as target2function4gpu
#</function4xi>
 

# -------------------------------------------------------------------------------------------------
# Sampler module of GPU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2sampler4gpu" usage = " importing target function for Process4Xi">
def importcreate2sampler4gpu():
   global target2function4gpu
   from gpu2sampler4xi import create2sampler4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importget2samplerinfo4gpu" usage = " importing target function for Process4Xi">
def importget2samplerinfo4gpu():
   global target2function4gpu
   from gpu2sampler4xi import get2samplerinfo4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importrelease2sampler4gpu" usage = " importing target function for Process4Xi">
def importrelease2sampler4gpu():
   global target2function4gpu
   from gpu2sampler4xi import release2sampler4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importretain2sampler4gpu" usage = " importing target function for Process4Xi">
def importretain2sampler4gpu():
   global target2function4gpu
   from gpu2sampler4xi import retain2sampler4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importset2samplercallback4gpu" usage = " importing target function for Process4Xi">
def importset2samplercallback4gpu():
   global target2function4gpu
   from gpu2sampler4xi import set2samplercallback4gpu as target2function4gpu 
#</function4xi>
 
#<function4xi function = "importset2sampler4gpu" usage = " importing target function for Process4Xi">
def importset2sampler4gpu():
   global target2function4gpu
   from gpu2sampler4xi import set2sampler4gpu as target2function4gpu
#</function4xi>
 
#<function4xi function = "importtrace2sampler4gpu" usage = " importing target function for Process4Xi">
def importtrace2sampler4gpu():
   global target2function4gpu
   from gpu2sampler4xi import trace2sampler4gpu as target2function4gpu
#</function4xi>
 
# -------------------------------------------------------------------------------------------------
# Program module of GPU imports for target function                              
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importcreate2programwithsource4gpu" usage = " importing target function for Process4Xi">
def importcreate2programwithsource4gpu():
   global target2function4gpu
   from gpu2kernel4xi import create2programwithsource4gpu as target2function4gpu
#</function4xi> 

#<function4xi function = "importcreate2programwithbinary4gpu" usage = " importing target function for Process4Xi">
def importcreate2programwithbinary4gpu():
   global target2function4gpu
   from gpu2kernel4xi import create2programwithbinary4gpu as target2function4gpu
#</function4xi> 

#<function4xi function = "importadd2programwithsource4gpu" usage = " importing target function for Process4Xi">
def importadd2programwithsource4gpu():
   global target2function4gpu
   from gpu2kernel4xi import add2programwithsource4gpu as target2function4gpu
#</function4xi> 

#<function4xi function = "importadd2programwithbinary4gpu" usage = " importing target function for Process4Xi">
def importadd2programwithbinary4gpu():
   global target2function4gpu
   from gpu2kernel4xi import add2programwithbinary4gpu as target2function4gpu
#</function4xi>

#<function4xi function = "importbuild2program4gpu" usage = " importing target function for Process4Xi">
def importbuild2program4gpu():
   global target2function4gpu
   from gpu2kernel4xi import build2program4gpu as target2function4gpu
#</function4xi>

#<function4xi function = "importbuild2program4gpu" usage = " importing target function for Process4Xi">
def importbuild2program4gpu():
   global target2function4gpu
   from gpu2kernel4xi import build2program4gpu as target2function4gpu
#</function4xi>

#<function4xi function = "importcompile2program4gpuX" usage = " importing target function for Process4Xi">
def importcompile2program4gpu():
   global target2function4gpu
   from gpu2kernel4xi import compile2program4gpu as target2function4gpu
#</function4xi>

#<function4xi function = "importdelete2programwithsource4gpuX" usage = " importing target function for Process4Xi">
def importdelete2programwithsource4gpu():
   global target2function4gpu
   from gpu2kernel4xi import delete2programwithsource4gpu as target2function4gpu
#</function4xi>    

#<function4xi function = "importdelete2programwithbinary4gpuX" usage = " importing target function for Process4Xi">
def importdelete2programwithbinary4gpu():
   global target2function4gpu
   from gpu2kernel4xi import delete2programwithbinary4gpu as target2function4gpu
#</function4xi>    

#<function4xi function = "importget2programinfo4gpuX" usage = " importing target function for Process4Xi">
def importget2programinfo4gpu():
   global target2function4gpu
   from gpu2kernel4xi import get2programinfo4gpu as target2function4gpu
#</function4xi>    

#<function4xi function = "importlink2program4gpuX" usage = " importing target function for Process4Xi">
def importlink2program4gpu():
   global target2function4gpu
   from gpu2kernel4xi import link2program4gpu as target2function4gpu
#</function4xi>    

#<function4xi function = "importrelease2program4gpuX" usage = " importing target function for Process4Xi">
def importrelease2program4gpu():
   global target2function4gpu
   from gpu2kernel4xi import release2program4gpu as target2function4gpu
#</function4xi>    

#<function4xi function = "importretain2program4gpuX" usage = " importing target function for Process4Xi">
def importretain2program4gpu():
   global target2function4gpu
   from gpu2kernel4xi import retain2program4gpu as target2function4gpu
#</function4xi>    

#<function4xi function = "importset2programcallback4gpuX" usage = " importing target function for Process4Xi">
def importset2programcallback4gpu():
   global target2function4gpu
   from gpu2kernel4xi import set2programcallback4gpu as target2function4gpu
#</function4xi>    

#<function4xi function = "importset2programwithsource4gpuX" usage = " importing target function for Process4Xi">
def importset2programwithsource4gpu():
   global target2function4gpu
   from gpu2kernel4xi import set2programwithsource4gpu as target2function4gpu
#</function4xi>    

#<function4xi function = "importset2programwithbinary4gpuX" usage = " importing target function for Process4Xi">
def importset2programwithbinary4gpu():
   global target2function4gpu
   from gpu2kernel4xi import set2programwithbinary4gpu as target2function4gpu
#</function4xi>    

#<function4xi function = "importtrace2program4gpuX" usage = " importing target function for Process4Xi">
def importtrace2program4gpu():
   global target2function4gpu
   from gpu2kernel4xi import trace2program4gpu as target2function4gpu
#</function4xi>    

# -------------------------------------------------------------------------------------------------
# Test modules of GPU imports for target function                             
# -------------------------------------------------------------------------------------------------
#<function4xi function = "importgpu2commandqueue4xi5test" usage = " importing target function for Process4Xi">
def importgpu2commandqueue4xi5test():
   global target2function4gpu
   from gpu2commandqueue4xi5test import gpu2commandqueue4xi5test as target2function4gpu
#</function4xi> 

#<function4xi function = "importgpu2event4xi5test" usage = " importing target function for Process4Xi">
def importgpu2event4xi5test():
   global target2function4gpu
   from gpu2event4xi5test import gpu2event4xi5test as target2function4gpu
#</function4xi>
 
#<function4xi function = "importgpu2image4xi5test" usage = " importing target function for Process4Xi">
def importgpu2image4xi5test():
   global target2function4gpu
   from gpu2image4xi5test import gpu2image4xi5test as target2function4gpu
#</function4xi>
 
#<function4xi function = "importgpu3image4xi5test" usage = " importing target function for Process4Xi">
def importgpu3image4xi5test():
   global target2function4gpu
   from gpu3image4xi5test import gpu3image4xi5test as target2function4gpu
#</function4xi>
 
#<function4xi function = "importgpu2sampler4xi5test" usage = " importing target function for Process4Xi">
def importgpu2sampler4xi5test():
   global target2function4gpu
   from gpu2sampler4xi5test import gpu2sampler4xi5test as target2function4gpu
#</function4xi>
 
#<function4xi function = "importGPU2kernel4xi5test" usage = " importing target function for Process4Xi">
def importgpu2kernel4xi5test():
   global target2function4gpu
   from gpu2kernel4xi5test import gpu2kernel4xi5test as target2function4gpu
#</function4xi> 

# -------------------------------------------------------------------------------------------------
# Error handler of GPU imports for target function                             
# -------------------------------------------------------------------------------------------------
#<function4xi function = "handle2error4gpu" usage = " Error handling for importing target function for Process4Xi">
def handle2error4gpu():
   global import2use4gpu
   print ("HANDLE2ERROR4GPU OF GPU2KERNEL4XI.PY: *** FAILED TO IMPORT TARGET FUNCTION ***")
   keys2import4gpu = import2use4gpu.keys()
   for key2import4gpu in keys2import4gpu:
       print(key2import4gpu,import2use4gpu[key2import4gpu]) 
#</function4xi> 


import2use4gpu = {}
# -------------------------------------------------------------------------------------------------
# Command queue module of GPU imports for target function                           
# -------------------------------------------------------------------------------------------------
import2use4gpu["create2commandqueue4gpu"] = importcreate2commandqueue4gpu        
import2use4gpu["flush2commandqueue4gpu"] = importflush2commandqueue4gpu 
import2use4gpu["finish2commandqueue4gpu"] = importfinish2commandqueue4gpu 
import2use4gpu["get2commandqueueinfo4gpu"] = importget2commandqueueinfo4gpu 
import2use4gpu["release2commandqueue4gpu"] = importrelease2commandqueue4gpu 
import2use4gpu["retain2commandqueue4gpu"] = importretain2commandqueue4gpu 
import2use4gpu["set2commandqueue4gpu"] = importset2commandqueue4gpu 
import2use4gpu["trace2commandqueue4gpu"] = importtrace2commandqueue4gpu 

# -------------------------------------------------------------------------------------------------
# Context module of GPU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4gpu["create2context4gpu"] = importcreate2context4gpu 
import2use4gpu["get2context4gpu"] = importget2context4gpu 
import2use4gpu["release2context4gpu"] = importrelease2context4gpu 
import2use4gpu["retain2context4gpu"] = importretain2context4gpu 
import2use4gpu["set2context4gpu"] = importset2context4gpu 

# -------------------------------------------------------------------------------------------------
# Device module of GPU imports for target function                           
# -------------------------------------------------------------------------------------------------
import2use4gpu["init2gpu4xi"] = importinit2gpu4xi 
import2use4gpu["set2gpu4xi"] = importset2gpu4xi 
import2use4gpu["get2gpu4xi"] = importget2gpu4xi 

# -------------------------------------------------------------------------------------------------
# Event module of GPU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4gpu["create2event4gpu"] = importcreate2event4gpu 
import2use4gpu["get2eventinfo4gpu"] = importget2eventinfo4gpu 
import2use4gpu["release2event4gpu"] = importrelease2event4gpu 
import2use4gpu["retain2event4gpu"] = importretain2event4gpu 
import2use4gpu["set2eventcallback4gpu"] = importset2eventcallback4gpu 
import2use4gpu["set2event4gpu"] = importset2event4gpu 
import2use4gpu["trace2event4gpu"] = importtrace2event4gpu 
import2use4gpu["wait2event4gpu"] = importwait2event4gpu 

# -------------------------------------------------------------------------------------------------
# 2D Image module of GPU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4gpu["create2image4gpu"] = importcreate2image4gpu 
import2use4gpu["get2imageinfo4gpu"] = importget2imageinfo4gpu 
import2use4gpu["get2supportedimageformats4gpu"] = importget2supportedimageformats4gpu 
import2use4gpu["manoeuvre2copyimage4gpu"] = importmanoeuvre2copyimage4gpu 
import2use4gpu["manoeuvre2copyimagetobuffer4gpu"] = importmanoeuvre2copyimagetobuffer4gpu 
import2use4gpu["manoeuvre2copybuffertoimage4gpu"] = importmanoeuvre2copybuffertoimage4gpu 
import2use4gpu["manoeuvre2readimage4gpu"] = importmanoeuvre2readimage4gpu 
import2use4gpu["manoeuvre2writeimage4gpu"] = importmanoeuvre2writeimage4gpu 
import2use4gpu["manoeuvre2mapimage4gpu"] = importmanoeuvre2mapimage4gpu 
import2use4gpu["copy2image4gpu"] = importcopy2image4gpu 
import2use4gpu["copy2imagetobuffer4gpu"] = importcopy2imagetobuffer4gpu 
import2use4gpu["copy2buffertoimage4gpu"] = importcopy2buffertoimage4gpu 
import2use4gpu["read2image4gpu"] = importread2image4gpu 
import2use4gpu["map2image4gpu"] = importmap2image4gpu 
import2use4gpu["release2image4gpu"] = importrelease2image4gpu 
import2use4gpu["retain2image4gpu"] = importretain2image4gpu 
import2use4gpu["set2imagecallback4gpu"] = importset2imagecallback4gpu 
import2use4gpu["set2image4gpu"] = importset2image4gpu 
import2use4gpu["trace2image4gpu"] = importtrace2image4gpu 
import2use4gpu["wait2image4gpu"] = importwait2image4gpu 
import2use4gpu["write2image4gpu"] = importwrite2image4gpu 

# -------------------------------------------------------------------------------------------------
# 3D Image module of GPU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4gpu["create3image4gpu"] = importcreate3image4gpu 
import2use4gpu["get3imageinfo4gpu"] = importget3imageinfo4gpu 
import2use4gpu["get3supportedimageformats4gpu"] = importget3supportedimageformats4gpu 
import2use4gpu["manoeuvre3copyimage4gpu"] = importmanoeuvre3copyimage4gpu 
import2use4gpu["manoeuvre3copyimagetobuffer4gpu"] = importmanoeuvre3copyimagetobuffer4gpu 
import2use4gpu["manoeuvre3copybuffertoimage4gpu"] = importmanoeuvre3copybuffertoimage4gpu 
import2use4gpu["manoeuvre3readimage4gpu"] = importmanoeuvre3readimage4gpu 
import2use4gpu["manoeuvre3writeimage4gpu"] = importmanoeuvre3writeimage4gpu 
import2use4gpu["manoeuvre3mapimage4gpu"] = importmanoeuvre3mapimage4gpu 
import2use4gpu["copy3image4gpu"] = importcopy3image4gpu 
import2use4gpu["copy3imagetobuffer4gpu"] = importcopy3imagetobuffer4gpu 
import2use4gpu["copy3buffertoimage4gpu"] = importcopy3buffertoimage4gpu 
import2use4gpu["read3image4gpu"] = importread3image4gpu 
import2use4gpu["map3image4gpu"] = importmap3image4gpu 
import2use4gpu["release3image4gpu"] = importrelease3image4gpu 
import2use4gpu["retain3image4gpu"] = importretain3image4gpu 
import2use4gpu["set3imagecallback4gpu"] = importset3imagecallback4gpu 
import2use4gpu["set3image4gpu"] = importset3image4gpu 
import2use4gpu["trace3image4gpu"] = importtrace3image4gpu 
import2use4gpu["wait3image4gpu"] = importwait3image4gpu 
import2use4gpu["write3image4gpu"] = importwrite3image4gpu 

# -------------------------------------------------------------------------------------------------
# Sampler module of GPU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4gpu["create2sampler4gpu"] = importcreate2sampler4gpu 
import2use4gpu["get2samplerinfo4gpu"] = importget2samplerinfo4gpu 
import2use4gpu["release2sampler4gpu"] = importrelease2sampler4gpu 
import2use4gpu["retain2sampler4gpu"] = importretain2sampler4gpu 
import2use4gpu["set2samplercallback4gpu"] = importset2samplercallback4gpu 
import2use4gpu["set2sampler4gpu"] = importset2sampler4gpu 
import2use4gpu["trace2sampler4gpu"] = importtrace2sampler4gpu 

# -------------------------------------------------------------------------------------------------
# Program module of GPU imports for target function                              
# -------------------------------------------------------------------------------------------------
import2use4gpu["trace2program4gpu"] = importtrace2program4gpu
import2use4gpu["set2programwithbinary4gpu"] = importset2programwithbinary4gpu
import2use4gpu["set2programwithsource4gpu"] = importset2programwithsource4gpu
import2use4gpu["set2programcallback4gpu"] = importset2programcallback4gpu
import2use4gpu["retain2program4gpu"] = importretain2program4gpu
import2use4gpu["release2program4gpu"] = importrelease2program4gpu
import2use4gpu["link2program4gpu"] = importlink2program4gpu
import2use4gpu["get2programinfo4gpu"] = importget2programinfo4gpu
import2use4gpu["delete2programwithbinary4gpu"] = importdelete2programwithbinary4gpu
import2use4gpu["delete2programwithsource4gpu"] = importdelete2programwithsource4gpu
import2use4gpu["compile2program4gpu"] = importcompile2program4gpu
import2use4gpu["build2program4gpu"] = importbuild2program4gpu
import2use4gpu["build2program4gpu"] = importbuild2program4gpu
import2use4gpu["add2programwithbinary4gpu"] = importadd2programwithbinary4gpu
import2use4gpu["add2programwithsource4gpu"] = importadd2programwithsource4gpu
import2use4gpu["create2programwithbinary4gpu"] = importcreate2programwithbinary4gpu
import2use4gpu["create2programwithsource4gpu"] = importcreate2programwithsource4gpu



# -------------------------------------------------------------------------------------------------
# Test modules of GPU imports for target function                             
# -------------------------------------------------------------------------------------------------
import2use4gpu["gpu2commandqueue4xi5test"] = importgpu2commandqueue4xi5test 
import2use4gpu["gpu2event4xi5test"] = importgpu2event4xi5test 
import2use4gpu["gpu2image4xi5test"] = importgpu2image4xi5test 
import2use4gpu["gpu3image4xi5test"] = importgpu3image4xi5test 
import2use4gpu["gpu2sampler4xi5test"] = importgpu2sampler4xi5test 
import2use4gpu["gpu2kernel4xi5test"] = importgpu2kernel4xi5test 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "trace2kernel4gpu" usage = "
#
#   print('=' *33)          
#   print('(Step 15A/25) OF GPU2KERNEL4XI5TEST - TRYING TO TRACE GPU: ', time4xi())      
#   print('=' *33)      
#   trace2gpu = None
#   try:
#       trace2gpu = trace2kernel4gpu() 
#       print('(Step 15B/25) OF GPU2KERNEL4XI5TEST - TRACE2GPU: ', trace2gpu)  
#       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2kernel4gpu-failed {}; time: 04:32:25 11/07/11 Pacific Standard Time}', 'CLASS': 'trace2kernel4gpu', 'METHOD': 'trace2kernel4gpu'} 
#   except:
#       print('(Step 15Z1/25) OF GPU2KERNEL4XI5TEST *** FAILED TO TRACE GPU: ', trace2gpu) 
#      
#">

# -------------------------------------------------------------------------------------------------   
def trace2kernel4gpu(task4trace2gpu=None):  

    '''
    creates callback to gpu2kernel4xi
    '''
    
    global gpu2trace4xi
    global folder2lib4gpu    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "trace2kernel4gpu" 
    method2print = "trace2kernel4gpu"

    message2print = "{(Step 1A/33) started" + "; trace2kernel4gpu-started: " + str(trace2kernel4gpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                               
    # 
  
             
    message2print = "{(Step 1B/33) passed" + "; task4trace2gpu-to-trace2kernel4gpu: " + str(task4trace2gpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              # 
    # 

    code2ack  = ' '
    code2nack = ' '    
    
    gpu2trace4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
      
    if task4trace2gpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY GPUIZED CALLBACK !
        # -----------------------------------------------------------------------------------------    
        task4trace2gpu = {}                                                                                            
            
        task4trace2gpu['CLASS']   = "trace2kernel4gpu"                                                              # GPUIZE ZPU TRACE!
        task4trace2gpu['METHOD']  = "trace2kernel4gpu" 
        task4trace2gpu['MESSAGE'] = "{(Step 2Z0/33) failed" + "; trace2kernel4gpu-failed " + str(gpu2trace4xi)  + "; time: " + time4xi() + "}"               
  
        
        message2print = "{(Step 2A/33) set" + "; task4trace2gpu-set-by-trace2kernel4gpu to: " + str(task4trace2gpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
    
        code2ack = code2ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # PARAMETER GIVEN BY GPUIZED CALLBACK !
    # -----------------------------------------------------------------------------------------          
    message2print = "{(Step 3A/33) try" + "; task4trace2gpu-try-by-trace2kernel4gpu: " + str(task4trace2gpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)    
    # 

    try:
        gpu2trace4xi ['CLASS']  = task4trace2gpu['CLASS']                                                           # GPUIZE GPU TRACE!
        gpu2trace4xi ['METHOD']  = task4trace2gpu['METHOD']
        gpu2trace4xi ['MESSAGE'] = task4trace2gpu['MESSAGE']
        
        message2print = "{(Step 3B/33) set" + "; gpu2trace4xi-set-by-trace2kernel4gpu to: " + str(gpu2trace4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 

        code2ack = code2ack + '3B-'         
    except:
        message2print = "{(Step 3Z1/33) failed" + "; gpu-trace-failed-by-trace2kernel4gpu: " + str(task4trace2gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z1-' 
        
    try:
        # =================================
        # SIMULATE GPU TRACE
        # =================================
        class2print   = task4trace2gpu['CLASS']                                                                     # GPUIZE GPU TRACE!
        method2print  = task4trace2gpu['METHOD']
        
        message2print = task4trace2gpu['MESSAGE']
        print4xi(class2print, method2print, message2print)                                                          # GPUIZED TRACE FOR OPENCL CALLBACK!
        # 
        
        
        code2ack = code2ack + '3C-'         
    except:
        class2print = "trace2kernel4gpu"                                                                            # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2kernel4gpu"   
        message2print = "{(Step 3Z2/33) failed" + "; simulation-of-gpu-trace-failed-by-trace2kernel4gpu: " + str(task4trace2gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z2-' 
        
    try:
        # =================================
        # NORMAL TRACE
        # =================================                     
        class2print = "trace2kernel4gpu"                                                                            # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2kernel4gpu"                
        message2print = "{(Step 3D/33) performed" + "; gpu-trace-performed-by-trace2kernel4gpu: " + str(task4trace2gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        code2ack = code2ack + '3D-' 
    except:
        message2print = "{(Step 3Z3/33) failed" + "; gpu-trace-failed-by-trace2kernel4gpu: " + str(task4trace2gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z3-' 

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; gpu2trace4xi-returning-by-trace2kernel4gpu: " + str(gpu2trace4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    
    return gpu2trace4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "execute2kernel4gpu" usage = "
#
#   response4execute2kernel = execute2kernel4gpu()
#      
#">

# -------------------------------------------------------------------------------------------------   
def execute2kernel4gpu(task4execute2gpu=None):  

    '''
    executes kernel to GPU 
    '''
    global folder2lib4gpu  
        
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "execute2kernel4gpu" 
    method2print = "execute2kernel4gpu"

    message2print = "{(Step 1A/33) started" + "; execute2kernel4gpu-started: " + str(execute2kernel4gpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # 
  
          
    message2print = "{(Step 1B/33) passed" + "; task4execute2gpu-to-execute2kernel4gpu: " + str(task4execute2gpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                            # 
    # 
    status4execute2gpu = 'CL_SUCCESS'
    
    code2ack  = ' '
    code2nack = ' '    
    
                                                                                                                  #       
    if task4execute2gpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN - BUILD THEM
        # -----------------------------------------------------------------------------------------    
        task4execute2gpu = {}                                                                                            

        message2print = "{(Step 2A/33) set" + "; task4execute2gpu-set-by-execute2kernel4gpu to: " + str(task4execute2gpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
    
        code2ack = code2ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # 
    # -----------------------------------------------------------------------------------------          
    status4execute2gpu = 'CL_SUCCESS'
    message2print = "{(Step 3A/33) set" + "; status4execute2gpu-set-by-execute2kernel4gpu to: " + str(status4execute2gpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; status4execute2gpu-returning-by-execute2kernel4gpu: " + str(status4execute2gpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    
    return status4execute2gpu
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2kernelcallback4gpu" usage = "
#
#   print('(Step 14A/700) OF GPU2KERNEL4XI5TEST - TRYING TO SET PROGRAM CALLBACK OF GPU ', time4xi())  
#   try:                               
#       response4setcallback2program = set2kernelcallback4gpu()                                                                                                 
#       print('(Step 14B/700) OF GPU2KERNEL4XI5TEST - SET PROGRAM CALLBACK OF GPU TO: ', response4setcallback2program)   
#       # 
#   except:
#       print('(Step 14Z1/700) OF GPU2KERNEL4XI5TEST *** FAILED TO SET PROGRAM CALLBACK OF GPU ', time4xi())  
# 
#">

# ------------------------------------------------------------------------------------------------- 
 
def set2kernelcallback4gpu(task4set2kernelcallback=None):

    '''
    registers callback function to gpu2kernel4xi
    '''
    global gpu2kernel4xi
    global folder2lib4gpu  
        
    class2print = "set2kernelcallback4gpu" 
    method2print = "set2kernelcallback4gpu"
    
    message2print = "{(Step 2A/70) started" + "; set2kernelcallback4gpu-started: " + str(set2kernelcallback4gpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # set2kernelcallback4gpu-started
    
    try:
    
        gpu2kernel4xi ['CALLBACK'] = trace2kernel4gpu
        
        
        message2print = "{(Step 2C/70) returning" + "; callback-of-gpu2kernel4xi-returning-by-set2kernelcallback4gpu: " + str(gpu2kernel4xi ['CALLBACK']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print) 
        # 
        
        return gpu2kernel4xi ['CALLBACK']      
        
    except:
        message2print = "{(Step 2Z1/70) failed" + "; callback2program-failed-by-set2kernelcallback4gpu: " + str(callback2program) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         

        message2print = "{(Step 2Z2/70) returning" + "; trace2kernel4gpu-returning-by-set2kernelcallback4gpu: " + str(trace2kernel4gpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)      
            
        return trace2kernel4gpu
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "execute2task4gpu" usage = "
#
#   try:                               
#       response4execute2task = execute2task4gpu(task4execute2task)
#       print('(Step 99B/700) OF GPU2KERNEL4XI5TEST - BUILT PROGRAM OF GPU DEVICE ', time4xi())
#   except:
#       print('(Step 99Z1/700) OF GPU2KERNEL4XI5TEST *** FAILED TO EXECUTE PROGRAM OF GPU DEVICE ', time4xi())  
#                 
#">

# -------------------------------------------------------------------------------------------------   
def execute2task4gpu(task4execute2gpu=None):

    '''
    executes  program to GPU 
    '''
    global gpu2execute4xi
    global folder2lib4gpu  
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "execute2task4gpu"
    method2print = "execute2task4gpu" 

    message2print = "{(Step 1A/33) started" + "; execute2task4gpu-started: " + str(execute2task4gpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # execute2task4gpu-started
    
    
    message2print = "{(Step 1B/33) passed" + "; task4execute2gpu-to-execute2task4gpu: " + str(task4execute2gpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
             
    code2ack  = ' '
    code2nack = ' '

    message2print = "{(Step 1A/70) try" + "; execute-try-by-execute2task4gpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    try:             
        if task4execute2gpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - EXECUTE TASK TO GPU! 
            # -----------------------------------------------------------------------------------------    
            task4execute2gpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4execute2gpu-initialized-by-execute2task4gpu: " + str(task4execute2gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1) KERNEL REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            program2gpu = {}
            message2print = "{(Step 1A/70) trying" + "; program2gpu-trying-by-create2kernel4gpu: " + str(program2gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                program2gpu = create2kernel4gpu()                                                                     
                
                message2print = "{(Step 1B/70) got" + "; program2gpu-got-by-create2kernel4gpu: " + str(program2gpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '1B-'         
            except:
                message2print = "{(Step 1Z/70) failed" + "; program2gpu-failed-by-create2kernel4gpu: " + str(program2gpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '1Z-'
                
            task4execute2gpu ['PROGRAM'] = program2gpu
            
            # ---------------------------------------------------------------------------------------------
            # (2) OPTIONS - REQUIRED                         
            # --------------------------------------------------------------------------------------------- 
            task4execute2task  ['OPTIONS'] = {}
          
            
            # ---------------------------------------------------------------------------------------------
            # (3) CALLBACK - REQUIRED                         
            # --------------------------------------------------------------------------------------------- 
            function2callback4gpu = trace2kernel4gpu                                                               # CALLBACK FUNCTION FOR GPU
            
            message2print = "{(Step 3A/70) initialized" + "; function2callback4gpu-created-by-execute2task4gpu: " + str(function2callback4gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '3A-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "execute2task4gpu"                                                            # MODULE OF GPUIZED OPENCL MODULE
            args2callback['METHOD']  = "execute2task4gpu"                                                            # FUNCTION OF GPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 3Z1/33) failed" + "; execute2task4gpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR GPUIZED OPENCL TO RETURN    

            message2print = "{(Step 3B/70) initialized" + "; args2callback-created-by-execute2task4gpu: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '3B-'
            
           
            task4execute2gpu['DATA2CALLBACK'] = args2callback                                                       # USER DATA PASSED TO GPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4execute2gpu['FUNCTION2CALLBACK'] = function2callback4gpu                                           # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '3C-'

            
            # ---------------------------------------------------------------------------------------------
            # (4) USERDATA - REQUIRED                         
            # ---------------------------------------------------------------------------------------------
            task4execute2gpu['USERDATA'] = None 
            
            message2print = "{(Step 4A/70) initialized" + "; task4execute2gpu-created-by-execute2task4gpu: " + str(task4execute2gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
        else:
            pass                                                                  
    except:
        message2print = "{(Step 1Z1/70) yielded" + "; task4execute2gpu-yielded-by-execute2task4gpu: " + str(task4execute2gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-'  
                                                                                                 
    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4execute2gpu.keys()        
    message2print = "{(Step 9/70) using" + "; keys2task-using-by-execute2task4gpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # 
  
    # ---------------------------------------------------------------------------------------------
    # (Step 10A/70) - CREATE EXECUTE FOR GPU ! 
    # ---------------------------------------------------------------------------------------------    
    gpu2execute4xi = {}                                                                                                                                                                               
    message2print = "{(Step 10A/70) initialized" + "; gpu2execute4xi-initialized-by-execute2task4gpu to: " + str(gpu2execute4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {}
    
    message2print = "{(Step 11A0/70) try" + "; program-try-by-execute2task4gpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 11A/70) - PROGRAM - REQUIRED 
        # -----------------------------------------------------------------------------------------               
        gpu2execute4xi['PROGRAM'] = task4execute2gpu['PROGRAM']
        
        message2print = "{(Step 11A1/70) initialized" + "; program-created-by-execute2task4gpu: " + str(gpu2execute4xi['PROGRAM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '11A1-'
    except: 
        message2print = "{(Step 11Z1/70) yielded" + "; program-gpu2execute4xi-yielded-by-execute2task4gpu: " + str(gpu2execute4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '11Z1-' 
           
    message2print = "{(Step 12A0/70) try" + "; options-try-by-execute2task4gpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 12A/70) - OPTIONS - REQUIRED 
        # -----------------------------------------------------------------------------------------               
        gpu2execute4xi['OPTIONS'] = task4execute2gpu['OPTIONS']
        
        message2print = "{(Step 12A1/70) initialized" + "; options-created-by-execute2task4gpu: " + str(gpu2execute4xi['OPTIONS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '12A1-'
    except: 
        message2print = "{(Step 12Z1/70) yielded" + "; options-gpu2execute4xi-yielded-by-execute2task4gpu: " + str(gpu2execute4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '12Z1-' 
        
    message2print = "{(Step 13A0/70) try" + "; callback-try-by-execute2task4gpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 13A/70) - CALLBACK - REQUIRED 
        # -----------------------------------------------------------------------------------------               
        gpu2execute4xi['CALLBACK'] = task4execute2gpu['CALLBACK']
        
        message2print = "{(Step 13A1/70) initialized" + "; callback-created-by-execute2task4gpu: " + str(gpu2execute4xi['CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '13A1-'
    except: 
        message2print = "{(Step 13Z1/70) yielded" + "; callback-gpu2execute4xi-yielded-by-execute2task4gpu: " + str(gpu2execute4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '13Z1-' 
        
    message2print = "{(Step 14A0/70) try" + "; userdata-try-by-execute2task4gpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 14A/70) - USERDATA - REQUIRED 
        # -----------------------------------------------------------------------------------------               
        gpu2execute4xi['USERDATA'] = task4execute2gpu['USERDATA']
        
        message2print = "{(Step 14A1/70) initialized" + "; userdata-created-by-execute2task4gpu: " + str(gpu2execute4xi['USERDATA'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '14A1-'
    except: 
        message2print = "{(Step 14Z1/70) yielded" + "; userdata-gpu2execute4xi-yielded-by-execute2task4gpu: " + str(gpu2execute4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '14Z1-'
                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    gpu2execute4xi ['ACK2RETURN']  = code2ack
    gpu2execute4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; gpu2execute4xi-returning-by-execute2task4gpu: " + str(gpu2execute4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
    
    return gpu2execute4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "create2kernel4gpu" usage = "
#     
#   print('(Step 2A1/700) OF GPU2KERNEL4XI5TEST - TRYING TO CREATE KERNEL FOR GPU ', time4xi())       
#   try:
#       response4create2kernel = create2kernel4gpu()
#   except:
#       print('(Step 2Z1/700) OF GPU2KERNEL4XI5TEST *** FAILED TO CREATE KERNEL FOR GPU *** ', time4xi())
#          
#">

# -------------------------------------------------------------------------------------------------   
def create2kernel4gpu(task4kernel2gpu=None):

    '''
    creates kernel for GPU
    '''
    global gpu2kernel4xi                                                                                            # Kernel of GPU    
    global folder2lib4gpu  
    global kb2kernel4gpu                                                                                            # Knowledgebase for kernel      
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "create2kernel4gpu" 
    method2print = "create2kernel4gpu"

    message2print = "{(Step 1A/70) started" + "; create2kernel4gpu-started: " + str(create2kernel4gpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function create2kernel4gpu at 0x0000000003AAD8C8> 
  
             
    message2print = "{(Step 1B/70) passed" + "; task4kernel2gpu-to-create2kernel4gpu: " + str(task4kernel2gpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    #1: None
    #2: 
    # 

    code2ack  = ' '
    code2nack = ' '    
    
    gpu2kernel4xi = {}                                                                                              # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4kernel2gpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO GPU! 
            # -----------------------------------------------------------------------------------------    
            
            task4kernel2gpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4kernel2gpu-created-by-create2kernel4gpu: " + str(task4kernel2gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - EXTRA                 
            # ---------------------------------------------------------------------------------------------
                        
            function2callback4gpu = trace2kernel4gpu                                                                # CALLBACK FUNCTION FOR GPU
            
            message2print = "{(Step 1D/70) initialized" + "; function2callback4gpu-created-by-create2kernel4gpu: " + str(function2callback4gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2kernel4gpu at 0x0000000003AAD6C8> 
            
            code2ack  = code2ack + '1D-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "create2kernel4gpu"                                                          # MODULE OF GPUIZED OPENCL MODULE
            args2callback['METHOD']  = "create2kernel4gpu"                                                          # FUNCTION OF GPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create2kernel4gpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR GPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args2callback-created-by-create2kernel4gpu: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2kernel4gpu-failed ; time: 03:33:11 12/06/11 Pacific Standard Time}',
            #  'CLASS': 'create2kernel4gpu',
            #  'METHOD': 'create2kernel4gpu'}; time: 03:33:11 12/06/11 Pacific Standard Time}

            code2ack  = code2ack + '1E-'
              
            task4kernel2gpu['DATA2CALLBACK'] = args2callback                                                        # USER DATA PASSED TO GPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4kernel2gpu['FUNCTION2CALLBACK'] = function2callback4gpu                                            # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4kernel2gpu-created-by-create2kernel4gpu: " + str(task4kernel2gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2kernel4gpu at 0x0000000003AAD6C8>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2kernel4gpu-failed ; time: 03:33:11 12/06/11 Pacific Standard Time}',
            #  'CLASS': 'create2kernel4gpu',
            #  'METHOD': 'create2kernel4gpu'}}; time: 03:33:11 12/06/11 Pacific Standard Time}


            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - EXTRA                      
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-create2kernel4gpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-create2kernel4gpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 03:33:11 12/06/11 Pacific Standard Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-create2kernel4gpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4kernel2gpu['PLATFORM'] = platform2xi                                                               # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - EXTRA                   
            # ---------------------------------------------------------------------------------------------    
            context2gpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2gpu-trying-by-create2kernel4gpu: " + str(context2gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2gpu = get2context4gpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2gpu-got-by-create2kernel4gpu: " + str(context2gpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2gpu-failed-by-create2kernel4gpu: " + str(context2gpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4kernel2gpu ['CONTEXT'] = context2gpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------    
            device2gpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2gpu-trying-by-create2kernel4gpu: " + str(device2gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2gpu = get2gpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2gpu-got-by-create2kernel4gpu: " + str(device2gpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2gpu-failed-by-create2kernel4gpu: " + str(device2gpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
                
            task4kernel2gpu ['DEVICE'] = device2gpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) ERROR CODE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            task4kernel2gpu ['CODE2ERROR'] = 0             
            code2ack  = code2ack + '5B-'              

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - EXTRA                          
            # ---------------------------------------------------------------------------------------------         
            queue2gpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2gpu-trying-by-create2kernel4gpu: " + str(queue2gpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2gpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2gpu-got-by-create2kernel4gpu: " + str(queue2gpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003C63DD8>
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2gpu-failed-by-create2kernel4gpu: " + str(queue2gpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4kernel2gpu ['QUEUE'] = queue2gpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - EXTRA                              
            # ---------------------------------------------------------------------------------------------         
            pipe2gpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2gpu-trying-by-create2kernel4gpu: " + str(pipe2gpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2gpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2gpu-got-by-create2kernel4gpu: " + str(pipe2gpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 384>, <read-write PipeConnection, handle 388>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2gpu-failed-by-create2kernel4gpu: " + str(pipe2gpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4kernel2gpu ['PIPE'] = pipe2gpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2gpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2gpu-trying-by-create2kernel4gpu: " + str(commandqueueinfo2gpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2gpu = get2commandqueueinfo4gpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2gpu-got-by-create2kernel4gpu: " + str(commandqueueinfo2gpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK' ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2gpu-failed-by-create2kernel4gpu: " + str(commandqueueinfo2gpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4kernel2gpu ['COMMANDQUEUE'] = commandqueueinfo2gpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2gpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2gpu-trying-by-create2kernel4gpu: " + str(bufferinfo2gpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2gpu = get2bufferinfo4gpu()                                                               # SYSTEM CREATES BUFFER FOR GPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2gpu-got-by-create2kernel4gpu: " + str(bufferinfo2gpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK' ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2gpu-failed-by-create2kernel4gpu: " + str(bufferinfo2gpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4kernel2gpu ['BUFFER'] = bufferinfo2gpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            eventinfo2gpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2gpu-trying-by-create2kernel4gpu: " + str(eventinfo2gpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2gpu = get2eventinfo4gpu()                                                                 # SYSTEM CREATES EVENT IF NECESSAY   
                message2print = "{(Step 10B/70) got" + "; eventinfo2gpu-got-by-create2kernel4gpu: " + str(eventinfo2gpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': ... 
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2gpu-failed-by-create2kernel4gpu: " + str(eventinfo2gpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4kernel2gpu ['EVENT'] = eventinfo2gpu             

            
            task4kernel2gpu ['CODE4PY']       = {}                                                                  
            task4kernel2gpu ['CODE4CPP']      = {}                                                                  
            task4kernel2gpu ['CODE4C']        = {} 
            task4kernel2gpu ['CODE4CL']       = {} 
                       
            task4kernel2gpu ['CODE4PYC']      = {}                                                                  
            task4kernel2gpu ['CODE4BIN']      = {}
 
            task4kernel2gpu ['DIRECTORY4PY']  = {}                                                                  
            task4kernel2gpu ['DIRECTORY4CPP'] = {}
            task4kernel2gpu ['DIRECTORY4C']   = {}
            task4kernel2gpu ['DIRECTORY4CL']  = {}
                                                                                                       
            task4kernel2gpu ['DIRECTORY4PYC'] = {}                                                                  
            task4kernel2gpu ['DIRECTORY4BIN'] = {}                                                                  
                                               
            # ---------------------------------------------------------------------------------------------
            # (12A0) DIRECTORY4PY - PYTHON                 
            # ---------------------------------------------------------------------------------------------             
            dir2py4gpu = folder2lib4gpu                                                                                                                                                            
            message2print = "{(Step 12A0/70) set" + "; dir2py4gpu-set-by-create2kernel4gpu to: " + str(dir2py4gpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print)  
            # folder2lib4gpu 
            code2ack  = code2ack + '12A0-' 
        
            task4kernel2gpu ['DIRECTORY4PY'] = dir2py4gpu
            
                        
            # ---------------------------------------------------------------------------------------------
            # (14A0) CODE4PY - PYTHON         
            # ---------------------------------------------------------------------------------------------             
            message2print = "{(Step 14A0/70) using" + "; DirectoryWalker-using-by-create2kernel4gpu: " + str(DirectoryWalker)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            for file2walk in DirectoryWalker(dir2py4gpu):
                if os.path.isdir(file2walk):
                    message2print = "{(Step 14B0/70) examining" + "; directory-examining-by-create2kernel4gpu: " + str(file2walk)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                      
                if os.path.isfile(file2walk):
                    message2print = "{(Step 14C0/70) examining" + "; file-examining-by-create2kernel4gpu: " + str(file2walk)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    #folder2lib4gpu + gpu2sampler4xi5test.py; gpu2dummy4xi5test.cpp 
                    
                    message2print = "{(Step 14D0/70) calling" + "; fetch2kernel4gpu-calling-by-create2kernel4gpu: " + str(fetch2kernel4gpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # <function fetch2kernel4gpu at 0x0000000003AADB48>
                    
                    message2print = "{(Step 14E0/70) passing" + "; file2walk-passing-to-fetch2kernel4gpu-by-create2kernel4gpu: " + str(file2walk) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    #folder2lib4gpu + gpu2sampler4xi5test.py; gpu2dummy4xi5test.cpp 
                    
                    fetch2kernel4gpu(file2walk)                                                                     # FETCHES KERNEL FILES INTO FETCH2CODE4GPU DICTIONARY 
                    
                    message2print = "{(Step 14E0/70) performed" + "; fetch2kernel4gpu-performed-by-create2kernel4gpu: " + str(fetch2kernel4gpu) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                      
                    # <function fetch2kernel4gpu at 0x0000000003AADB48>
                    
                    # message2print = "{(Step 14F0/70) got" + "; fetch2code4gpu-got-by-create2kernel4gpu: " + str(fetch2code4gpu) + "; time: " + time4xi() + "}"
                    # print4xi(class2print, method2print, message2print)                                      
                    # ...  {1: '...profile = "11" />\n', 2: '# --
            
            
                     
            keys2explore4gpu = fetch2code4gpu.keys()
            message2print = "{(Step 14G0/70) using" + "; keys2explore4gpu-using-by-create2kernel4gpu: " + str(keys2explore4gpu) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)             
            # folder2lib4gpu + gpu2sampler4xi5test.py; gpu2dummy4xi5test.cpp
            
            for key2explore4gpu in keys2explore4gpu:
                name2program4gpu=os.path.splitext(key2explore4gpu)                                                  # gpu2sampler4xi5test + py
                key2extension4gpu = name2program4gpu[1]                                                              
                if key2extension4gpu == '.py':                          
                    task4kernel2gpu ['CODE4PY'] [key2explore4gpu] = fetch2code4gpu[key2explore4gpu]                 # Python code(s)      

            # ---------------------------------------------------------------------------------------------
            # (14H0) SIZE4PY - PYTHON SOURCE            
            # ---------------------------------------------------------------------------------------------                                                
            size2py4gpu  = {}
            keys2py4gpu = fetch2code4gpu.keys()                                                                     # gpu2sampler4xi5test.py
            for key2py4gpu in keys2py4gpu:
                name2program4gpu=os.path.splitext(key2py4gpu)                                                       # gpu2sampler4xi5test + py
                key2extension4gpu = name2program4gpu[1]                                                             # py   
                if key2extension4gpu == '.py':                          
                    size2py4gpu[key2py4gpu] = len(fetch2code4gpu[key2py4gpu])                                       # length of saved py code 

            task4kernel2gpu ['SIZE4PY'] = size2py4gpu
                               
            message2print = "{(Step 14H0/70) set" + "; size2py4gpu-set-by-create2kernel4gpu to: " + str(size2py4gpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
            # {folder2lib4gpu + gpu2sampler4xi5test.py': 1337}
            code2ack  = code2ack + '14H1-' 
            

            # ---------------------------------------------------------------------------------------------
            # (12A1) C, C++, CL SOURCE                   
            # ---------------------------------------------------------------------------------------------             
            dir2c4gpu = folder2lib4gpu                                                                                                                                                            
            message2print = "{(Step 12A1/70) set" + "; dir2c4gpu-set-by-create2kernel4gpu to: " + str(dir2c4gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # folder2lib4gpu 
            code2ack  = code2ack + '12A1-' 
        
            task4kernel2gpu ['DIRECTORY4CPP']  = dir2c4gpu
            task4kernel2gpu ['DIRECTORY4C']    = dir2c4gpu
            task4kernel2gpu ['DIRECTORY4CLPP'] = dir2c4gpu
            task4kernel2gpu ['CODE4CPP'] = {} 
            task4kernel2gpu ['CODE4C']   = {} 
            task4kernel2gpu ['CODE4CL']  = {}            
            
            # ---------------------------------------------------------------------------------------------
            # (14A1) C, C++, CL SOURCE   
            # ---------------------------------------------------------------------------------------------             
            keys2c4gpu = fetch2code4gpu.keys()
            for key2c4gpu in keys2c4gpu:
                name2c4gpu=os.path.splitext(key2c4gpu)                                                  
                key2extension4gpu = name2c4gpu[1]                                                             # C++, C, CL 
                if key2extension4gpu == '.cpp':                          
                    task4kernel2gpu ['CODE4CPP'] [key2c4gpu] = fetch2code4gpu[key2c4gpu]                     
                    message2print = "{(Step 14G1/70) set" + "; key2c4gpu-set-by-create2kernel4gpu to: " + str(key2c4gpu)  + "; time: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print)  
                    # folder2lib4gpu + gpu2dummy4xi5test.cpp
                if key2extension4gpu == '.c':                          
                    task4kernel2gpu ['CODE4C'] [key2c4gpu] = fetch2code4gpu[key2c4gpu]                     
                    message2print = "{(Step 14G2/70) set" + "; key2c4gpu-set-by-create2kernel4gpu to: " + str(key2c4gpu)  + "; time: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print)  
                    # folder2lib4gpu +
                if key2extension4gpu == '.cl':                          
                    task4kernel2gpu ['CODE4CL'] [key2c4gpu] = fetch2code4gpu[key2c4gpu]                     
                    message2print = "{(Step 14G3/70) set" + "; key2c4gpu-set-by-create2kernel4gpu to: " + str(key2c4gpu)  + "; time: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print)  
                    # folder2lib4gpu + 
                                                                
            # ----------------------------------------------------------------------------------------------
            # (14H1) SIZE4C - C C++ CL SOURCE HAS THE SAME FOLDER AS FOR PYTHON SOURCE - ALREADY FETCHED           
            # ----------------------------------------------------------------------------------------------                                                
            size2c4gpu  = {}
            keys2c4gpu = fetch2code4gpu.keys()                                                                # 
            for key2c4gpu in keys2c4gpu:
                name2program4gpu=os.path.splitext(key2c4gpu)                                                  # 
                key2extension4gpu = name2program4gpu[1]                                                       #    
                if key2extension4gpu == '.cpp':                          
                    size2c4gpu[key2c4gpu] = len(fetch2code4gpu[key2c4gpu])                                    # length of saved C++ code
                    task4kernel2gpu ['SIZE4CPP'] = size2c4gpu 
                if key2extension4gpu == '.c':                          
                    size2c4gpu[key2c4gpu] = len(fetch2code4gpu[key2c4gpu])                                    # length of saved C code
                    task4kernel2gpu ['SIZE4C'] = size2c4gpu
                if key2extension4gpu == '.cl':                          
                    size2c4gpu[key2c4gpu] = len(fetch2code4gpu[key2c4gpu])                                    # length of saved CL code
                    task4kernel2gpu ['SIZE4CL'] = size2c4gpu 
                                       
            
                               
            message2print = "{(Step 14H1/70) set" + "; size2c4gpu-set-by-create2kernel4gpu to: " + str(size2c4gpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
            # folder2lib4gpu + gpu2dummy4xi5test.cpp': 331}
            code2ack  = code2ack + '14H1-' 
            
                                                  
            # ---------------------------------------------------------------------------------------------
            # (12B2) DIRECTORY FOR PYTHON CACHE - REQUIRED                      
            # ---------------------------------------------------------------------------------------------             
            dir2pycache4gpu = folder2lib4gpu + '__pycache__//'                                                                                                                                                           
            message2print = "{(Step 12B2/70) set" + "; dir2pycache4gpu-set-by-create2kernel4gpu to: " + str(dir2pycache4gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # folder2lib4gpu + '__pycache__//'
            code2ack  = code2ack + '12B2-' 
        
            task4kernel2gpu ['DIRECTORY4PYC'] = dir2pycache4gpu
            
                        
            # ---------------------------------------------------------------------------------------------
            # (14A2) CODE4PYC - REQUIRED              
            # ---------------------------------------------------------------------------------------------             
            message2print = "{(Step 14A2/70) using" + "; DirectoryWalker-using-by-create2kernel4gpu: " + str(DirectoryWalker)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            for file2walk in DirectoryWalker(dir2pycache4gpu):
                if os.path.isdir(file2walk):
                    message2print = "{(Step 14B2/70) examining" + "; directory-examining-by-create2kernel4gpu: " + str(file2walk)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                      
                if os.path.isfile(file2walk):
                    message2print = "{(Step 14C2/70) examining" + "; file-examining-by-create2kernel4gpu: " + str(file2walk)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    #folder2lib4gpu + __pycache__//gpu2sampler4xi5test.cpython-33.pyc
                    
                    message2print = "{(Step 14D2/70) calling" + "; fetch2kernel4gpu-calling-by-create2kernel4gpu: " + str(fetch2kernel4gpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # <function fetch2kernel4gpu at 0x0000000003AADB48>
                    
                    message2print = "{(Step 14E2/70) passing" + "; file2walk-passing-to-fetch2kernel4gpu-by-create2kernel4gpu: " + str(file2walk) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    #folder2lib4gpu + __pycache__//gpu2sampler4xi5test.cpython-33.pyc
                    
                    fetch2kernel4gpu(file2walk)                                                                     # fetches kernel files into fetch2code4gpu dictionary 
                    
                    message2print = "{(Step 14F2/70) performed" + "; fetch2kernel4gpu-performed-by-create2kernel4gpu: " + str(fetch2kernel4gpu) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                      
                    # <function fetch2kernel4gpu at 0x0000000003AADB48>
                    
                    # message2print = "{(Step 14G2/70) got" + "; fetch2code4gpu-got-by-create2kernel4gpu: " + str(fetch2code4gpu) + "; time: " + time4xi() + "}"
                    # print4xi(class2print, method2print, message2print)                                      
                    # 
            
            task4kernel2gpu ['CODE4PYC'] = {}
            keys2pyc4gpu = fetch2code4gpu.keys()
            for key2pyc4gpu in keys2pyc4gpu:
                name2program4gpu=os.path.splitext(key2pyc4gpu)                                                      # gpu2sampler4xi5test.cpython-33.pyc
                key2extension4gpu = name2program4gpu[1]                                                             # pyc 
   
                if key2extension4gpu == '.pyc':                          
                    task4kernel2gpu ['CODE4PYC'] [key2pyc4gpu] = fetch2code4gpu[key2pyc4gpu]                        # binary code(s)
                    message2print = "{(Step 14H2/70) set" + "; key2pyc4gpu-set-by-create2kernel4gpu to: " + str(key2pyc4gpu)  + "; time: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print)  
                    #    
                                     
            size2pyc4gpu  = {}

            for key2pyc4gpu in keys2pyc4gpu:
                name2program4gpu=os.path.splitext(key2pyc4gpu)                                                      # gpu2sampler4xi5test.cpython-32 + pyc
                key2extension4gpu = name2program4gpu[1]                                                             # pyc 
                if key2extension4gpu == '.pyc':                          
                    size2pyc4gpu[key2pyc4gpu] = len(fetch2code4gpu[key2pyc4gpu])                                    # length of saved binary code
                    message2print = "{(Step 14I2/70) set" + "; key2pyc4gpu-set-by-create2kernel4gpu to: " + str(key2pyc4gpu)  + "; time: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print)  
                    # 
                                              
            # ---------------------------------------------------------------------------------------------
            # (14I2) SIZE4PYC - REQUIRED              
            # ---------------------------------------------------------------------------------------------          
            task4kernel2gpu ['SIZE4PYC'] = size2pyc4gpu
                               
            message2print = "{(Step 14J2/70) set" + "; size2pyc4gpu-set-by-create2kernel4gpu to: " + str(size2pyc4gpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
            # 
            code2ack  = code2ack + '14J2-' 

                                               
            # ---------------------------------------------------------------------------------------------
            # (12A3) DIRECTORY FOR C++, C, CL BIN - REQUIRED                    
            # ---------------------------------------------------------------------------------------------             
            dir2bin4gpu = folder2lib4gpu + 'bin//'                                                                                                                                                           
            message2print = "{(Step 12A3/70) set" + "; dir2bin4gpu-set-by-create2kernel4gpu to: " + str(dir2bin4gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # folder2lib4gpu + 'bin//'
            code2ack  = code2ack + '12A3-' 
        
            task4kernel2gpu ['DIRECTORY4BIN'] = dir2bin4gpu
            
                        
            # ---------------------------------------------------------------------------------------------
            # (14A3) CODE4BIN FOR C++, C, CL - REQUIRED              
            # ---------------------------------------------------------------------------------------------             
            message2print = "{(Step 14A3/70) using" + "; DirectoryWalker-using-by-create2kernel4gpu: " + str(DirectoryWalker)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            for file2walk in DirectoryWalker(dir2bin4gpu):
                if os.path.isdir(file2walk):
                    message2print = "{(Step 14B3/70) examining" + "; directory-examining-by-create2kernel4gpu: " + str(file2walk)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                      
                if os.path.isfile(file2walk):
                    message2print = "{(Step 14C3/70) examining" + "; file-examining-by-create2kernel4gpu: " + str(file2walk)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    #folder2lib4gpu + bin//gpu2dummy4xi5test.exe
                    
                    message2print = "{(Step 14D3/70) calling" + "; fetch2kernel4gpu-calling-by-create2kernel4gpu: " + str(fetch2kernel4gpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # <function fetch2kernel4gpu at 0x0000000003AADB48>
                    
                    message2print = "{(Step 14E3/70) passing" + "; file2walk-passing-to-fetch2kernel4gpu-by-create2kernel4gpu: " + str(file2walk) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    # folder2lib4gpu + bin//gpu2dummy4xi5test.exe
                    
                    fetch2kernel4gpu(file2walk)                                                                     # fetches kernel files into fetch2code4gpu dictionary 
                    
                    message2print = "{(Step 14E3/70) performed" + "; fetch2kernel4gpu-performed-by-create2kernel4gpu: " + str(fetch2kernel4gpu) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                      
                    # <function fetch2kernel4gpu at 0x0000000003AADB48>
                    
                    message2print = "{(Step 14F3/70) got" + "; fetch2code4gpu-got-by-create2kernel4gpu: " + str(fetch2code4gpu) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                      
                    # 

            task4kernel2gpu ['CODE4BIN'] = {}
            
            keys2bin4gpu = fetch2code4gpu.keys()
            message2print = "{(Step 14G3/70) using" + "; keys2bin4gpu-using-by-create2kernel4gpu: " + str(keys2bin4gpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # [folder2lib4gpu + 'gpu2dummy4xi5test.cpp', 'gpu2sampler4xi5test.py']
            
            for key2bin4gpu in keys2bin4gpu:
                name2program4gpu=os.path.splitext(key2bin4gpu)                                                      # gpu2dummy4xi5test.exe
                key2extension4gpu = name2program4gpu[1]                                                             # bin 
   
                if key2extension4gpu == '.exe':                          
                    task4kernel2gpu ['CODE4BIN'] [key2bin4gpu] = fetch2code4gpu[key2bin4gpu]                        # binary code(s)   
                                        
            size2bin4gpu  = {}
            keys2bin4gpu = fetch2code4gpu.keys()                                                                    # gpu2dummy4xi5test.exe
            for key2bin4gpu in keys2bin4gpu:
                name2program4gpu=os.path.splitext(key2bin4gpu)                                                      # gpu2sampler4xi5test.cpython-32 + bin
                key2extension4gpu = name2program4gpu[1]                                                             # bin 
                if key2extension4gpu == '.exe':                          
                    size2bin4gpu[key2bin4gpu] = len(fetch2code4gpu[key2bin4gpu])                                    # length of saved binary code
                        
            # ---------------------------------------------------------------------------------------------
            # (14H3) SIZE4BIN - REQUIRED              
            # ---------------------------------------------------------------------------------------------          
            task4kernel2gpu ['SIZE4BIN'] = size2bin4gpu
                               
            message2print = "{(Step 14H3/70) set" + "; size2bin4gpu-set-by-create2kernel4gpu to: " + str(size2bin4gpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
            # {}
            code2ack  = code2ack + '14H3-' 

            # ---------------------------------------------------------------------------------------------
            # (14J) COUNT4PY, COUNT4C,COUNT4PYC,COUNT4BIN                
            # ---------------------------------------------------------------------------------------------             
            task4kernel2gpu ['COUNT4PY']     = 0
            task4kernel2gpu ['COUNT4CPP']    = 0
            task4kernel2gpu ['COUNT4C']      = 0
            task4kernel2gpu ['COUNT4CL']     = 0
            task4kernel2gpu ['COUNT4PYC']    = 0
            task4kernel2gpu ['COUNT4BIN']    = 0

            keys2code4gpu = fetch2code4gpu.keys()
            message2print = "{(Step 14J/70) using" + "; keys2code4gpu-using-by-create2kernel4gpu: " + str(keys2code4gpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # [folder2lib4gpu + 'gpu2dummy4xi5test.cpp', folder2lib4gpu + 'gpu2sampler4xi5test.py']

            
            for key2code4gpu in keys2code4gpu:
                name2program4gpu=os.path.splitext(key2code4gpu)                                                    
                key2extension4gpu = name2program4gpu[1]                                                             
                if key2extension4gpu == '.py':                          
                    task4kernel2gpu ['COUNT4PY']     = task4kernel2gpu ['COUNT4PY']  + 1 
                if key2extension4gpu == '.cpp':                          
                    task4kernel2gpu ['COUNT4CPP']    = task4kernel2gpu ['COUNT4CPP'] + 1                                            
                if key2extension4gpu == '.c':                          
                    task4kernel2gpu ['COUNT4C']      = task4kernel2gpu ['COUNT4C']   + 1              
                if key2extension4gpu == '.cl':                          
                    task4kernel2gpu ['COUNT4CL']     = task4kernel2gpu ['COUNT4CL']  + 1                                  
                if key2extension4gpu == '.pyc':                          
                    task4kernel2gpu ['COUNT4PYC']    = task4kernel2gpu ['COUNT4PYC'] + 1                            
                if key2extension4gpu == '.exe':                          
                    task4kernel2gpu ['COUNT4BIN']    = task4kernel2gpu ['COUNT4BIN'] + 1                                                                                 
            
            message2print = "{(Step 14K/70) set" + "; count4py-set-by-create2kernel4gpu to: " + str(task4kernel2gpu ['COUNT4PY'])  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
            # 1
            
            message2print = "{(Step 14L/70) set" + "; count4cpp-set-by-create2kernel4gpu to: " + str(task4kernel2gpu ['COUNT4CPP'])  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
            # 1
            message2print = "{(Step 14M/70) set" + "; count4c-set-by-create2kernel4gpu to: " + str(task4kernel2gpu ['COUNT4C'])  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
            # 0 
            message2print = "{(Step 14N/70) set" + "; count4cl-set-by-create2kernel4gpu to: " + str(task4kernel2gpu ['COUNT4CL'])  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
            # 0                         
            
            message2print = "{(Step 14M/70) set" + "; count4pyc-set-by-create2kernel4gpu to: " + str(task4kernel2gpu ['COUNT4PYC'])  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
            # 0
            
            message2print = "{(Step 14N/70) set" + "; count4bin-set-by-create2kernel4gpu to: " + str(task4kernel2gpu ['COUNT4BIN'])  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
            # 0                                                
            code2ack  = code2ack + '14N-'
            
            # ---------------------------------------------------------------------------------------------
            # (15) LANGUAGE - EXTRA                             
            # ---------------------------------------------------------------------------------------------             
            task4kernel2gpu ['LANGUAGE'] = {}
            
            for key2code4gpu in keys2code4gpu:
                name2program4gpu=os.path.splitext(key2code4gpu)                                                     
                key2extension4gpu = name2program4gpu[1]                                                             
                if key2extension4gpu == '.py':                          
                    task4kernel2gpu ['LANGUAGE'] ['PY']  = "Python"  
                if key2extension4gpu == '.cpp':                          
                    task4kernel2gpu ['LANGUAGE'] ['CPP'] = "C++"
                if key2extension4gpu == '.c':                          
                    task4kernel2gpu ['LANGUAGE'] ['C']   = "C"  
                if key2extension4gpu == '.cl':                          
                    task4kernel2gpu ['LANGUAGE'] ['CL']  = "CL"                                     
                    
            message2print = "{(Step 15B/70) set" + "; language2task4gpu-set-by-create2kernel4gpu to: " + str(task4kernel2gpu ['LANGUAGE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CPP': 'C++', 'PY': 'Python'}
            code2ack  = code2ack + '15B-' 
                      
            # ---------------------------------------------------------------------------------------------
            # (16) COUNT2REFERENCE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            count2reference2gpu  = 1
           
            message2print = "{(Step 16B/70) set" + "; count2reference2gpu-set-by-create2kernel4gpu to: " + str(count2reference2gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1 
            code2ack  = code2ack + '16B-' 
        
            task4kernel2gpu ['COUNT2REFERENCE'] = count2reference2gpu                                 
                                    
            # ---------------------------------------------------------------------------------------------
            # (17) PROGRAM - REQUIRED                           
            # ---------------------------------------------------------------------------------------------             
            task4kernel2gpu ['PROGRAM'] = None
            message2print = "{(Step 17A/70) try" + "; program2gpu-try-by-create2kernel4gpu: " + str(task4kernel2gpu ['PROGRAM'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #             
            try:
                task4kernel2gpu ['PROGRAM'] = create2programwithsource4gpu()
                keys2program = task4kernel2gpu ['PROGRAM'].keys()
                
                message2print = "{(Step 17B/70) set" + "; program-set-by-create2kernel4gpu to: " + str(keys2program)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # ['CODE2ERROR' ... 
                #  'COUNT2REFERENCE', 'NACK2RETURN', 'CONTEXT', 'COMMANDQUEUE', 'DEVICE', ... 'PIPE']

                code2ack  = code2ack + '17B-'
            except:
                message2print = "{(Step 17Z1/70) failed" + "; task4kernel2gpu-failed-by-create2kernel4gpu: " + str(task4kernel2gpu ['PROGRAM']) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                #             
                code2nack  = code2nack + '17Z-'
                                    
            # ---------------------------------------------------------------------------------------------
            # (18) FUNCTION - REQUIRED                
            # --------------------------------------------------------------------------------------------- 
            task4kernel2gpu ['FUNCTION'] = {}
            message2print = "{(Step 18A0/70) set" + "; function4gpu-set-by-create2kernel4gpu to: " + str(task4kernel2gpu ['FUNCTION'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
             
            task4kernel2gpu ['FUNCTION'] ['.py'] = {}
            message2print = "{(Step 18A1/70) set" + "; pythonfunction4gpu-set-by-create2kernel4gpu to: " + str(task4kernel2gpu ['FUNCTION'] ['.py'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
                        
            tag2search4gpu  = kb2kernel4gpu ['.py'] ['TAG']
            message2print = "{(Step 18A2/70) using" + "; tag2search4gpu-using-by-create2kernel4gpu: " + str(tag2search4gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # ... function =
            
            value2search4gpu    = kb2kernel4gpu ['.py'] ['VALUE']
            message2print = "{(Step 18A3/70) using" + "; value2search4gpu-using-by-create2kernel4gpu: " + str(value2search4gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # gpu2sampler4xi5test
            
            attribute2search4gpu= kb2kernel4gpu ['.py'] ['ATTRIBUTE'] 
            message2print = "{(Step 18A4/70) using" + "; attribute2search4gpu-using-by-create2kernel4gpu: " + str(attribute2search4gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # usage =
            try:
                keys2code4gpu = task4kernel2gpu ['PROGRAM'] ['CODE4SOURCE'].keys()
                for key2code4gpu in keys2code4gpu:
                    # ---------------------------------------------------------------------------------------------
                    # SOURCE FILE
                    # ---------------------------------------------------------------------------------------------
                    code2examine4gpu = task4kernel2gpu ['PROGRAM'] ['CODE4SOURCE'] [key2code4gpu]
                    keys2line4gpu = code2examine4gpu.keys()
                    for key2line4gpu in keys2line4gpu:
                        # -----------------------------------------------------------------------------------------
                        # SOURCE LINE
                        # -----------------------------------------------------------------------------------------
                        line2examine4gpu = task4kernel2gpu ['PROGRAM'] ['CODE4SOURCE'] [key2code4gpu] [key2line4gpu]
                        # message2print = "{(Step 18A5/70) examining" + "; line2examine4gpu-examining-by-create2kernel4gpu: " + str(line2examine4gpu)  + "; time: " + time4xi() + "}"
                        # print4xi(class2print, method2print, message2print) 
                        #                             
                        
                        index2pattern = str.find(line2examine4gpu,tag2search4gpu)  
                        if index2pattern == -1:
                            pass
                        else:
                            # -------------------------------------------------------------------------------------
                            # FUNCTION TAG
                            # -------------------------------------------------------------------------------------
                            message2print = "{(Step 18B/70) examining" + "; line2function4gpu-examining-by-create2kernel4gpu: " + str(line2examine4gpu)  + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print) 
                            # ... function = "set2trace4sampler" ... # <... gpu2sampler4xi5test" usage = "
                             
                            index2function = str.find(line2examine4gpu,value2search4gpu)
                            if index2function == -1:
                                message2print = "{(Step 18C/70) skipped" + "; line2function4gpu-skipped-by-create2kernel4gpu: " + str(line2examine4gpu)  + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print) 
                                # <... set2trace4sampler" ... # ...function = "gpu2sampler4xi5test" usage = " ...function = "main" usage = "
                                
                            else:
                                # ----------------------------------------------------------------------------------
                                # KERNEL FUNCTION  
                                # ----------------------------------------------------------------------------------
                                index2usage = str.find(line2examine4gpu,attribute2search4gpu)             # ...function = "gpu2sampler4xi5test" usage = "
                                if index2usage  == -1:                                                
                                    # ------------------------------------------------------------------------------
                                    # NOT OK KERNEL FUNCTION TAG LINE - USAGE ATTRIBUTE MISSING
                                    # ------------------------------------------------------------------------------
                                    message2print = "{(Step 18D/70) warning" + "; line2function4gpu-warning-by-create2kernel4gpu: " + str(line2examine4gpu)  + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print) 
                                    # 
                                    
                                else:
                                    # ------------------------------------------------------------------------------
                                    # OK KERNEL FUNCTION TAG LINE
                                    # ------------------------------------------------------------------------------    
                                    message2print = "{(Step 18E/70) accepted" + "; line2examine4gpu-accepted-by-create2kernel4gpu: " + str(line2examine4gpu)  + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print) 
                                    # ...function = "gpu2sampler4xi5test" usage = "
                                    # 0123456789012345678901234567890123456789012345678901234
                                    
                                    key2pythonfunction4gpu = line2examine4gpu[index2function:index2usage-2]
                                    
                                    task4kernel2gpu ['FUNCTION'] ['.py'] [key2pythonfunction4gpu] = {}                                  

                                    message2print = "{(Step 18F/70) used" + "; key2pythonfunction4gpu-used-by-create2kernel4gpu: " + str(key2pythonfunction4gpu)  + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print) 
                                    # gpu2sampler4xi5test
                                    
                                    code2ack  = code2ack + '18F-'
                            # -------------------------------------------------------------------------------------
                            # 
                            # -------------------------------------------------------------------------------------                                        
                             
            except:
                message2print = "{(Step 18Z1/70) failed" + "; task4kernel2gpu-failed-by-create2kernel4gpu: " + str(task4kernel2gpu ['FUNCTION']) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                #             
                code2nack  = code2nack + '18Z-'            
                                    
            # ---------------------------------------------------------------------------------------------
            # (19) FUNCTION - KERNEL FUNCTIONS FROM FECTHED C++ FILE(S) - NOT FROM PROGRAM OBJECT                
            # --------------------------------------------------------------------------------------------- 
            task4kernel2gpu ['FUNCTION'] ['.cpp'] = {}            
            message2print = "{(Step 19A0F/70) set" + "; cplusfunction4gpu-set-by-create2kernel4gpu to: " + str(task4kernel2gpu ['FUNCTION'] ['.cpp'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}   
            
            tag2search4gpu      = kb2kernel4gpu ['.cpp'] ['TAG']
            message2print = "{(Step 19A2/70) using" + "; tag2search4gpu-using-by-create2kernel4gpu: " + str(tag2search4gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <...            
            attribute2search4gpu= kb2kernel4gpu ['.cpp'] ['ATTRIBUTE'] 
            message2print = "{(Step 19A4/70) using" + "; attribute2search4gpu-using-by-create2kernel4gpu: " + str(attribute2search4gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # usage = kernel
            

            try:
                keys2code4gpu = task4kernel2gpu ['PROGRAM'] ['CODE4SOURCE'].keys()
                for key2code4gpu in keys2code4gpu:
                    # ---------------------------------------------------------------------------------------------
                    # SOURCE FILE
                    # ---------------------------------------------------------------------------------------------
                    code2examine4gpu = task4kernel2gpu ['PROGRAM'] ['CODE4SOURCE'] [key2code4gpu]
                    message2print = "{(Step 19A5/70) using" + "; key2code4gpu-using-by-create2kernel4gpu: " + str(key2code4gpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # gpu2dummy4xi5test.cpp
                       
                    keys2line4gpu = code2examine4gpu.keys()
                    # message2print = "{(Step 19A6/70) using" + "; keys2line4gpu-using-by-create2kernel4gpu: " + str(keys2line4gpu)  + "; time: " + time4xi() + "}"
                    # print4xi(class2print, method2print, message2print) 
                    # 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 ...                    
                    for key2line4gpu in keys2line4gpu:
                        # -----------------------------------------------------------------------------------------
                        # SOURCE LINE
                        # -----------------------------------------------------------------------------------------
                        line2examine4gpu = task4kernel2gpu ['PROGRAM'] ['CODE4SOURCE'] [key2code4gpu] [key2line4gpu]
                        # message2print = "{(Step 19A7/70) examining" + "; line2examine4gpu-examining-by-create2kernel4gpu: " + str(line2examine4gpu)  + "; time: " + time4xi() + "}"
                        # print4xi(class2print, method2print, message2print) 
                        # /*<... nbody_kern" usage = "kernel example 1"*/                        
                        
                        index2pattern = str.find(line2examine4gpu,tag2search4gpu)  
                        if index2pattern == -1:
                            pass
                        else:
                            # -------------------------------------------------------------------------------------
                            # FUNCTION TAG 
                            # -------------------------------------------------------------------------------------
                            message2print = "{(Step 19B1/70) examining" + "; line2function4gpu-examining-by-create2kernel4gpu: " + str(line2examine4gpu)  + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # ... function = "OCLError" usage = "error code checking"*/
                            # ... function = "nbody_kern" usage = "kernel example 1"*/
                            
                            message2print = "{(Step 19B2/70) using" + "; attribute2search4gpu-using-by-create2kernel4gpu: " + str(attribute2search4gpu)  + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print) 
                            #                                                         
                            index2usage = str.find(line2examine4gpu,attribute2search4gpu)
                            if index2usage == -1:
                                # ----------------------------------------------------------------------------------
                                # NON KERNEL FUNCTION  
                                # ----------------------------------------------------------------------------------                            
                                message2print = "{(Step 19C/70) skipped" + "; line2function4gpu-skipped-by-create2kernel4gpu: " + str(line2examine4gpu)  + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print) 
                                # ... function = "OCLError" usage = "error code checking"*/
                                
                            else:
                                # ----------------------------------------------------------------------------------
                                # KERNEL FUNCTION  
                                # ----------------------------------------------------------------------------------
                                message2print = "{(Step 19E/70) accepted" + "; line2examine4gpu-accepted-by-create2kernel4gpu: " + str(line2examine4gpu)  + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print) 
                                # ... function = "nbody_kern" usage = "kernel example 1"*/; 
                                # ... function = "nbody2_kern" usage = "kernel example 2"*/ 
                                # 012345678901234567890123456789012345678901234567890
                                 
                                key2kernelfunction4gpu = line2examine4gpu[27:index2usage-2]  
                                task4kernel2gpu ['FUNCTION'] ['.cpp'] [key2kernelfunction4gpu] =  {}                                       

                                message2print = "{(Step 19F/70) used" + "; key2kernelfunction4gpu-used-by-create2kernel4gpu: " + str(key2kernelfunction4gpu)  + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print) 
                                # nbody_kern; nbody2_kern 
                                    
                                code2ack  = code2ack + '19F-'
                                    
                                    
            except:
                message2print = "{(Step 19Z1/70) failed" + "; task4kernel2gpu-failed-by-create2kernel4gpu: " + str(task4kernel2gpu ['FUNCTION']) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                #   
                #             
                code2nack  = code2nack + '19Z-'           
                                                      
            # ---------------------------------------------------------------------------------------------
            # EOF CREATING TASK4KERNEL2GPU                  
            # ---------------------------------------------------------------------------------------------                                                                                                                                                          
        else:
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4kernel2gpu-yielded-by-create2kernel4gpu: " + str(task4kernel2gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-' 

    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4kernel2gpu.keys()        
    message2print = "{(Step 19A/70) using" + "; keys2task-using-by-create2kernel4gpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    #1 this program created:
    # ['BUFFER'  ... 'LANGUAGE', 'PIPE', 
    #  'PLATFORM', 'PROGRAM', 'QUEUE' ....']
    #2 caller passed:
    # ['FUNCTION', 'PROGRAM']
    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - CREATE PROGRAM OBJECT FOR GPU - BUILD REQUIRED ONES IF USER HAS FORGOTTEN ! 
    # ---------------------------------------------------------------------------------------------    
    gpu2kernel4xi = {}                                                                                             # CREATE INITIALIZES DICTIONARY!                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; gpu2kernel4xi-initialized-by-create2kernel4gpu to: " + str(gpu2kernel4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {}
    
    if (('DATA2CALLBACK' not in keys2task) or ('FUNCTION2CALLBACK' not in keys2task)):
        try:            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - REQUIRED                
            # ---------------------------------------------------------------------------------------------
                     
            function2callback4gpu = trace2kernel4gpu                                                                # CALLBACK FUNCTION FOR GPU
            
            message2print = "{(Step 1D/70) initialized" + "; function2callback4gpu-created-by-create2kernel4gpu: " + str(function2callback4gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2kernel4gpu at 0x0000000003AAD6C8> 
            
            code2ack  = code2ack + '1D-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "create2kernel4gpu"                                                          # MODULE OF GPUIZED OPENCL MODULE
            args2callback['METHOD']  = "create2kernel4gpu"                                                          # FUNCTION OF GPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create2kernel4gpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR GPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args2callback-created-by-create2kernel4gpu: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2kernel4gpu-failed ; time: 03:33:11 12/06/11 Pacific Standard Time}',
            #  'CLASS': 'create2kernel4gpu',
            #  'METHOD': 'create2kernel4gpu'}; time: 03:33:11 12/06/11 Pacific Standard Time}

            code2ack  = code2ack + '1E-'
              
            task4kernel2gpu['DATA2CALLBACK'] = args2callback                                                        # USER DATA PASSED TO GPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4kernel2gpu['FUNCTION2CALLBACK'] = function2callback4gpu                                            # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4kernel2gpu-created-by-create2kernel4gpu: " + str(task4kernel2gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2kernel4gpu at 0x0000000003AAD6C8>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2kernel4gpu-failed ; time: 03:33:11 12/06/11 Pacific Standard Time}',
            #  'CLASS': 'create2kernel4gpu',
            #  'METHOD': 'create2kernel4gpu'}}; time: 03:33:11 12/06/11 Pacific Standard Time}
        except: 
            message2print = "{(Step 1Z2/70) yielded" + "; callbacks-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-'     
                
    if 'DATA2CALLBACK' in keys2task:
        message2print = "{(Step 21A0/70) try" + "; data2callback-try-by-create2kernel4gpu: {}" + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)      
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 21A/70) - DATA2CALLBACK - REQUIRED 
            # -----------------------------------------------------------------------------------------               
            gpu2kernel4xi['DATA2CALLBACK'] = task4kernel2gpu['DATA2CALLBACK']
            
            message2print = "{(Step 21A1/70) initialized" + "; data2callback-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2kernel4gpu-failed ; time: 04:42:53 12/06/11 Pacific Standard Time}',
            #  'CLASS': 'create2kernel4gpu',
            #  'METHOD': 'create2kernel4gpu'}
    
            code2ack  = code2ack + '21A1-'
        except: 
            message2print = "{(Step 21Z1/70) yielded" + "; data2callback-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '21Z1-'
    
    if 'FUNCTION2CALLBACK' in keys2task:    
        message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-create2kernel4gpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 22B/70) - FUNCTION2CALLBACK - REQUIRED
            # -----------------------------------------------------------------------------------------               
            gpu2kernel4xi['FUNCTION2CALLBACK'] = task4kernel2gpu['FUNCTION2CALLBACK']
            
            message2print = "{(Step 22B1/70) initialized" + "; function2callback-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function trace2kernel4gpu at 0x000000000422F6C8>
            
            code2ack  = code2ack + '22B1-'
        except: 
            message2print = "{(Step 22Z2/70) yielded" + "; function2callback-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '22Z2-' 
        
        message2print = "{(Step 22C0/70) try" + "; platform-try-by-create2kernel4gpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        #

        
    if 'PLATFORM' in keys2task:         
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 22C/70) - PLATFORM - EXTRA
            # -----------------------------------------------------------------------------------------               
            gpu2kernel4xi['PLATFORM'] = task4kernel2gpu['PLATFORM']
            
            message2print = "{(Step 22C1/70) initialized" + "; platform-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
    
    
            code2ack  = code2ack + '22C1-'
        except: 
            message2print = "{(Step 22Z3/70) yielded" + "; platform-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '22Z3-' 
            
    
    if 'CONTEXT' in keys2task:        
        message2print = "{(Step 23A/70) try" + "; context2gpu-try-by-create2kernel4gpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 23/70) - CONTEXT  - EXTRA
            # -----------------------------------------------------------------------------------------               
            gpu2kernel4xi['CONTEXT'] = task4kernel2gpu['CONTEXT']
            
            message2print = "{(Step 23B/70) initialized" + "; context-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK' ...      
            
            code2ack  = code2ack + '23B-'
        except: 
            message2print = "{(Step 23Z1/70) yielded" + "; context-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
        
            code2nack  = code2nack + '23Z1-' 
    
    if 'DEVICE' in keys2task:        
        message2print = "{(Step 24A/70) try" + "; device-try-by-create2kernel4gpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 24/70) - DEVICE - REQUIRED
            # -----------------------------------------------------------------------------------------               
            gpu2kernel4xi['DEVICE'] = task4kernel2gpu['DEVICE']
            
            message2print = "{(Step 24B/70) initialized" + "; device-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['DEVICE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ...
            
            code2ack  = code2ack + '24B-'
        except: 
            message2print = "{(Step 24Z1/70) yielded" + "; device-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '24Z1-'
    
    if 'CODE2ERROR' in keys2task:                     
        message2print = "{(Step 25A/70) try" + "; code2error-try-by-create2kernel4gpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 25/70) - ERROR CODE - REQUIRED
            # -----------------------------------------------------------------------------------------               
            gpu2kernel4xi['CODE2ERROR'] = task4kernel2gpu['CODE2ERROR']
            
            message2print = "{(Step 25B/70) initialized" + "; code2error-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 0
            
            code2ack  = code2ack + '25B-'
        except: 
            message2print = "{(Step 25Z1/70) yielded" + "; code2error-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '25Z1-'
                            
    if 'QUEUE' not in keys2task: 
        try:
            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED                          
            # ---------------------------------------------------------------------------------------------         
            queue2gpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2gpu-trying-by-create2kernel4gpu: " + str(queue2gpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2gpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2gpu-got-by-create2kernel4gpu: " + str(queue2gpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003C63DD8>
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2gpu-failed-by-create2kernel4gpu: " + str(queue2gpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4kernel2gpu ['QUEUE'] = queue2gpu
        except:
            message2print = "{(Step 26Z1/70) yielded" + "; queue-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '26Z1-'             

    if 'QUEUE' in keys2task:          
        message2print = "{(Step 26A/70) try" + "; queue-try-by-create2kernel4gpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 26/70) - QUEUE - REQUIRED
            # -----------------------------------------------------------------------------------------               
            gpu2kernel4xi['QUEUE'] = task4kernel2gpu['QUEUE']
            
            message2print = "{(Step 26B/70) initialized" + "; queue-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['QUEUE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <queue4xi.Queue4Xi object at 0x0000000003E72828> 
            
            code2ack  = code2ack + '26B-'
        except: 
            message2print = "{(Step 26Z2/70) yielded" + "; queue-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '26Z2-' 
              
    if 'PIPE' not in keys2task:
        try:             
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------         
            pipe2gpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2gpu-trying-by-create2kernel4gpu: " + str(pipe2gpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2gpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2gpu-got-by-create2kernel4gpu: " + str(pipe2gpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 384>, <read-write PipeConnection, handle 388>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2gpu-failed-by-create2kernel4gpu: " + str(pipe2gpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4kernel2gpu ['PIPE'] = pipe2gpu 
        except: 
            message2print = "{(Step 27Z1/70) yielded" + "; pipe-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '27Z1-'
                                 
    if 'PIPE' in keys2task:                                
        message2print = "{(Step 27A/70) try" + "; pipe-try-by-create2kernel4gpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                     
        try:
            # -----------------------------------------------------------------------------------------
            # (Step 27/70) - PIPE - REQUIRED
            # -----------------------------------------------------------------------------------------               
            gpu2kernel4xi['PIPE'] = task4kernel2gpu['PIPE']
            
            message2print = "{(Step 27B/70) initialized" + "; pipe-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['PIPE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>)
            
            code2ack  = code2ack + '27B-'
        except: 
            message2print = "{(Step 27Z2/70) yielded" + "; pipe-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '27Z2-' 

    if 'COMMANDQUEUE' in keys2task:
        message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-create2kernel4gpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            gpu2kernel4xi['COMMANDQUEUE']  = task4kernel2gpu['COMMANDQUEUE']  
     
            message2print = "{(Step 28B/70) initialized" + "; commandqueue-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK' ...
                    
            code2ack  = code2ack + '28B-'
        except: 
            message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                  
            code2nack  = code2nack + '28Z1-'    
    
    if 'BUFFER' in keys2task:
        message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-create2kernel4gpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            gpu2kernel4xi['BUFFER']  = task4kernel2gpu['BUFFER']  
     
            message2print = "{(Step 29B/70) initialized" + "; bufferinfo-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['BUFFER'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0 ...
            
            
            code2ack  = code2ack + '29B-'
        except: 
            message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '29Z1-'            
    
    if 'EVENT' in keys2task:
        message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-create2kernel4gpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
                            
            gpu2kernel4xi['EVENT']  = task4kernel2gpu['EVENT']  
     
            message2print = "{(Step 30B/70) initialized" + "; eventinfo-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['EVENT'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'DATA2CALLBACK' ... 
            
            code2ack  = code2ack + '30B-'
        except: 
            message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '30Z1-'
                        
    
    if 'COUNT4PY' in keys2task:               
        message2print = "{(Step 31A1/70) try" + "; count2py4gpu-try-by-create2kernel4gpu: " + str(task4kernel2gpu['COUNT4PY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 1
        try:            
            # ---------------------------------------------------------------------------------------------
            # (31) COUNT4PY - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['COUNT4PY']  = task4kernel2gpu['COUNT4PY']  
    
            message2print = "{(Step 31A2/70) initialized" + "; count2py4gpu-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['COUNT4PY'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1
            code2ack  = code2ack + '31A2-'
        except: 
            message2print = "{(Step 31Z1/70) yielded" + "; count2py4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '31Z1-'     
    
    if 'COUNT4CPP' in keys2task:               
        message2print = "{(Step 31B1/70) try" + "; count2cpp4gpu-try-by-create2kernel4gpu: " + str(task4kernel2gpu['COUNT4CPP'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 1
        try:            
            # ---------------------------------------------------------------------------------------------
            # (31) COUNT4CPP - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['COUNT4CPP']  = task4kernel2gpu['COUNT4CPP']  
    
            message2print = "{(Step 31B2/70) initialized" + "; count2cpp4gpu-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['COUNT4CPP'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1
            code2ack  = code2ack + '31B2-'
        except: 
            message2print = "{(Step 31Z3/70) yielded" + "; count2cpp4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '31Z3-'     
            
    
    if 'COUNT4C' in keys2task:               
        message2print = "{(Step 31C1/70) try" + "; count2c4gpu-try-by-create2kernel4gpu: " + str(task4kernel2gpu['COUNT4C'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 1
        try:            
            # ---------------------------------------------------------------------------------------------
            # (31) COUNT4C - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['COUNT4C']  = task4kernel2gpu['COUNT4C']  
    
            message2print = "{(Step 31C2/70) initialized" + "; count2c4gpu-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['COUNT4C'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1
            code2ack  = code2ack + '31C2-'
        except: 
            message2print = "{(Step 31Z3/70) yielded" + "; count2c4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '31Z3-'     
    
    if 'COUNT4CL' in keys2task:               
        message2print = "{(Step 31D1/70) try" + "; count2cl4gpu-try-by-create2kernel4gpu: " + str(task4kernel2gpu['COUNT4CL'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 1
        try:            
            # ---------------------------------------------------------------------------------------------
            # (31) COUNT4CL - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['COUNT4CL']  = task4kernel2gpu['COUNT4CL']  
    
            message2print = "{(Step 31D2/70) initialized" + "; count2cl4gpu-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['COUNT4CL'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1
            code2ack  = code2ack + '31D2-'
        except: 
            message2print = "{(Step 31Z3/70) yielded" + "; count2cl4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '31Z3-'     
                
    if 'COUNT4PYC' in keys2task:               
        message2print = "{(Step 31E1/70) try" + "; count4pyc-try-by-create2kernel4gpu: " + str(task4kernel2gpu['COUNT4PYC'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        try:            
            # ---------------------------------------------------------------------------------------------
            # (31) COUNT4PYC - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['COUNT4PYC']  = task4kernel2gpu['COUNT4PYC']  
    
            message2print = "{(Step 31E2/70) initialized" + "; count4pyc-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['COUNT4PYC'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '31E2-'
        except: 
            message2print = "{(Step 31Z4/70) yielded" + "; count4pyc-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '31Z4-'     
    
    if 'COUNT4BIN' in keys2task:               
        message2print = "{(Step 31F1/70) try" + "; count4bin-try-by-create2kernel4gpu: " + str(task4kernel2gpu['COUNT4BIN'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        try:            
            # ---------------------------------------------------------------------------------------------
            # (31) COUNT4BIN - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['COUNT4BIN']  = task4kernel2gpu['COUNT4BIN']  
    
            message2print = "{(Step 31F2/70) initialized" + "; count4bin-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['COUNT4BIN'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0
            code2ack  = code2ack + '31F2-'
        except: 
            message2print = "{(Step 31Z5/70) yielded" + "; count4bin-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '31Z5-' 
                            
    if 'DIRECTORY4PY' not in keys2task:
        try:             
            # ---------------------------------------------------------------------------------------------
            # (32A) DIRECTORY4PY - REQUIRED                
            # ---------------------------------------------------------------------------------------------             
            dir2py4gpu = folder2lib4gpu                                                                                                                                                            
            message2print = "{(Step 32A/70) set" + "; dir2py4gpu-set-by-create2kernel4gpu to: " + str(dir2py4gpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print)  
            # folder2lib4gpu 
            code2ack  = code2ack + '32A-' 
        
            task4kernel2gpu ['DIRECTORY4PY'] = dir2py4gpu
        except: 
            message2print = "{(Step 32AZ/70) yielded" + "; directory4py-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '32AZ-'                         
               
    if 'DIRECTORY4PY' in keys2task:
        message2print = "{(Step 32A1/70) try" + "; directory4py-try-by-create2kernel4gpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (32) DIRECTORY4PY - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['DIRECTORY4PY']  = task4kernel2gpu['DIRECTORY4PY']  
     
            message2print = "{(Step 32B1/70) initialized" + "; directory2py4gpu-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['DIRECTORY4PY'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # folder2lib4gpu
            code2ack  = code2ack + '32B1-'
        except: 
            message2print = "{(Step 32Z1/70) yielded" + "; directory2py4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '32Z1-' 
             
    if 'DIRECTORY4CPP' not in keys2task:
        try:             
            # ---------------------------------------------------------------------------------------------
            # (32A0) DIRECTORY4CPP - C++                 
            # ---------------------------------------------------------------------------------------------             
            dir2cpp4gpu = folder2lib4gpu                                                                                                                                                            
            message2print = "{(Step 32A0/70) set" + "; dir2cpp4gpu-set-by-create2kernel4gpu to: " + str(dir2cpp4gpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print)  
            # folder2lib4gpu 
            code2ack  = code2ack + '32A0-' 
        
            task4kernel2gpu ['DIRECTORY4CPP'] = dir2cpp4gpu
        except: 
            message2print = "{(Step 32Z0/70) yielded" + "; directory4py-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '32Z0-'
                                        
    if 'DIRECTORY4CPP' in keys2task:
        message2print = "{(Step 32A2/70) try" + "; directory2cpp4gpu-try-by-create2kernel4gpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (32) DIRECTORY4CPP - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['DIRECTORY4CPP']  = task4kernel2gpu['DIRECTORY4CPP']  
     
            message2print = "{(Step 32B2/70) initialized" + "; directory2cpp4gpu-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['DIRECTORY4CPP'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # folder2lib4gpu
            code2ack  = code2ack + '32B2-'
        except: 
            message2print = "{(Step 32Z2/70) yielded" + "; directory2cpp4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '32Z1-' 
                         
    if 'DIRECTORY4C' not in keys2task:
        try:             
            # ---------------------------------------------------------------------------------------------
            # (32A0) DIRECTORY4C - C                 
            # ---------------------------------------------------------------------------------------------             
            dir2c4gpu = folder2lib4gpu                                                                                                                                                            
            message2print = "{(Step 32A0/70) set" + "; dir2c4gpu-set-by-create2kernel4gpu to: " + str(dir2c4gpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print)  
            # folder2lib4gpu 
            code2ack  = code2ack + '32A0-' 
        
            task4kernel2gpu ['DIRECTORY4C'] = dir2c4gpu
        except: 
            message2print = "{(Step 32Z0/70) yielded" + "; directory4py-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '32Z0-'
                                        
    if 'DIRECTORY4C' in keys2task:
        message2print = "{(Step 32A3/70) try" + "; directory2c4gpu-try-by-create2kernel4gpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (32) DIRECTORY4C - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['DIRECTORY4C']  = task4kernel2gpu['DIRECTORY4C']  
     
            message2print = "{(Step 32B3/70) initialized" + "; directory2c4gpu-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['DIRECTORY4C'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # folder2lib4gpu
            code2ack  = code2ack + '32B1-'
        except: 
            message2print = "{(Step 32Z3/70) yielded" + "; directory2c4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '32Z1-' 
             
    if 'DIRECTORY4CL' not in keys2task:
        try:             
            # ---------------------------------------------------------------------------------------------
            # (32A0) DIRECTORY4CL - C                 
            # ---------------------------------------------------------------------------------------------             
            dir2cl4gpu = folder2lib4gpu                                                                                                                                                            
            message2print = "{(Step 32A0/70) set" + "; dir2cl4gpu-set-by-create2kernel4gpu to: " + str(dir2cl4gpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print)  
            # folder2lib4gpu 
            code2ack  = code2ack + '32A0-' 
        
            task4kernel2gpu ['DIRECTORY4CL'] = dir2cl4gpu
        except: 
            message2print = "{(Step 32Z0/70) yielded" + "; directory4py-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '32Z0-'
                                        
    if 'DIRECTORY4CL' in keys2task:
        message2print = "{(Step 32A4/70) try" + "; directory2cl4gpu-try-by-create2kernel4gpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (32) DIRECTORY4CL - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['DIRECTORY4CL']  = task4kernel2gpu['DIRECTORY4CL']  
     
            message2print = "{(Step 32B4/70) initialized" + "; directory2cl4gpu-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['DIRECTORY4CL'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # folder2lib4gpu
            code2ack  = code2ack + '32B1-'
        except: 
            message2print = "{(Step 32Z4/70) yielded" + "; directory2cl4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '32Z1-'
              
    if 'DIRECTORY4PYC' not in keys2task:
        try:             
            # ---------------------------------------------------------------------------------------------
            # (32A0) DIRECTORY4PYC - C                 
            # ---------------------------------------------------------------------------------------------             
            dir2pycache4gpu = folder2lib4gpu + '__pycache__//'                                                                                                                                                           
            message2print = "{(Step 32B2/70) set" + "; dir2pycache4gpu-set-by-create2kernel4gpu to: " + str(dir2pycache4gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # folder2lib4gpu + '__pycache__//'
            code2ack  = code2ack + '32B2-' 
        
            task4kernel2gpu ['DIRECTORY4PYC'] = dir2pycache4gpu
        except: 
            message2print = "{(Step 32Z0/70) yielded" + "; directory4py-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '32Z0-'                     
                                                    
    if 'DIRECTORY4PYC' in keys2task:
        message2print = "{(Step 32C/70) try" + "; directory2pyc4gpu-try-by-create2kernel4gpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (32) DIRECTORY4PYC - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['DIRECTORY4PYC']  = task4kernel2gpu['DIRECTORY4PYC']  
     
            message2print = "{(Step 32D/70) initialized" + "; directory2pyc4gpu-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['DIRECTORY4PYC'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # folder2lib4gpu + __pycache__//
            code2ack  = code2ack + '32D-'
        except: 
            message2print = "{(Step 32Z2/70) yielded" + "; directory2pyc4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '32Z2-' 
             
    if 'DIRECTORY4BIN' not in keys2task:
        try:             
            # ---------------------------------------------------------------------------------------------
            # (32A0) DIRECTORY4BIN - C                 
            # ---------------------------------------------------------------------------------------------             
            dir2bin4gpu = folder2lib4gpu + 'bin//'                                                                                                                                                           
            message2print = "{(Step 12A3/70) set" + "; dir2bin4gpu-set-by-create2kernel4gpu to: " + str(dir2bin4gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # folder2lib4gpu + 'bin//'
            code2ack  = code2ack + '12A3-' 
        
            task4kernel2gpu ['DIRECTORY4BIN'] = dir2bin4gpu
        except: 
            message2print = "{(Step 32Z0/70) yielded" + "; directory4bin-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '32Z0-' 
                                        
    if 'DIRECTORY4BIN' in keys2task:
        message2print = "{(Step 32E/70) try" + "; directory2bin4gpu-try-by-create2kernel4gpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (32) DIRECTORY4BIN - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['DIRECTORY4BIN']  = task4kernel2gpu['DIRECTORY4BIN']  
     
            message2print = "{(Step 32F/70) initialized" + "; directory2bin4gpu-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['DIRECTORY4BIN'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # folder2lib4gpu + bin//
            code2ack  = code2ack + '32F-'
        except: 
            message2print = "{(Step 32Z3/70) yielded" + "; directory2bin4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '32Z3-'  
    
                     
    if 'CODE4PY' in keys2task:            
        message2print = "{(Step 33A1/70) try" + "; code2py4gputask-try-by-create2kernel4gpu: " + str(task4kernel2gpu ['CODE4PY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        # {folder2lib4gpu + 'gpu2sampler4xi5test.py': {1: '... profile = "11" />\n', 2: ...
        
        gpu2kernel4xi['CODE4PY'] = {}
        try:            
            # ---------------------------------------------------------------------------------------------
            # (33) CODE4PY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['CODE4PY'] = task4kernel2gpu ['CODE4PY']
    
     
            message2print = "{(Step 33B1/70) created" + "; code2py4gpu-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['CODE4PY'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4gpu + 'gpu2sampler4xi5test.py': {1: '... profile = "11" />\n', 2:
             
            code2ack  = code2ack + '33B1-'
        except: 
            message2print = "{(Step 33Z1/70) yielded" + "; code2py4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi['CODE4PY']) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2py4gpu2nack  = code2py4gpu2nack + '33Z1-'     
                     
    if 'CODE4CPP' in keys2task:            
        message2print = "{(Step 33A2/70) try" + "; code2cpp4gputask-try-by-create2kernel4gpu: " + str(len(task4kernel2gpu ['CODE4CPP']))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        # {folder2lib4gpu +  gpu2dummy4xi5test.cpp
        
        gpu2kernel4xi['CODE4CPP'] = {}
        try:            
            # ---------------------------------------------------------------------------------------------
            # (33) CODE4CPP - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['CODE4CPP'] = task4kernel2gpu ['CODE4CPP']
    
     
            message2print = "{(Step 33B2/70) created" + "; code2cpp4gpu-created-by-create2kernel4gpu: " + str(len(gpu2kernel4xi['CODE4CPP']))  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4gpu + gpu2dummy4xi5test.cpp
             
            code2ack  = code2ack + '33B1-'
        except: 
            message2print = "{(Step 33Z2/70) yielded" + "; code2cpp4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi['CODE4CPP']) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2cpp4gpu2nack  = code2cpp4gpu2nack + '33Z2-'     
                     
    if 'CODE4C' in keys2task:            
        message2print = "{(Step 33A3/70) try" + "; code2c4gputask-try-by-create2kernel4gpu: " + str(task4kernel2gpu ['CODE4C'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        # {folder2lib4gpu + 
        
        gpu2kernel4xi['CODE4C'] = {}
        try:            
            # ---------------------------------------------------------------------------------------------
            # (33) CODE4C - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['CODE4C'] = task4kernel2gpu ['CODE4C']
    
     
            message2print = "{(Step 33B3/70) created" + "; code2c4gpu-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['CODE4C'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4gpu + 
             
            code2ack  = code2ack + '33B1-'
        except: 
            message2print = "{(Step 33Z3/70) yielded" + "; code2c4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi['CODE4C']) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2c4gpu2nack  = code2c4gpu2nack + '33Z3-'                             
                     
    if 'CODE4CL' in keys2task:            
        message2print = "{(Step 33A4/70) try" + "; code2cl4gputask-try-by-create2kernel4gpu: " + str(task4kernel2gpu ['CODE4CL'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        # {folder2lib4gpu + 
        
        gpu2kernel4xi['CODE4CL'] = {}
        try:            
            # ---------------------------------------------------------------------------------------------
            # (33) CODE4CL - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['CODE4CL'] = task4kernel2gpu ['CODE4CL']
    
     
            message2print = "{(Step 33B4/70) created" + "; code2cl4gpu-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['CODE4CL'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4gpu + 
             
            code2ack  = code2ack + '33B1-'
        except: 
            message2print = "{(Step 33Z4/70) yielded" + "; code2cl4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi['CODE4CL']) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2cl4gpu2nack  = code2cl4gpu2nack + '33Z1-'     
                                        
    if 'CODE4PYC' in keys2task:            
        message2print = "{(Step 33C/70) try" + "; code2pyc4gpu-try-by-create2kernel4gpu: " + str(task4kernel2gpu ['CODE4PYC'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        # {folder2lib4gpu + 
        
        gpu2kernel4xi['CODE4PYC'] = {}
        try:            
            # ---------------------------------------------------------------------------------------------
            # (33) CODE4PYC - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['CODE4PYC'] = task4kernel2gpu ['CODE4PYC']
    
     
            message2print = "{(Step 33D/70) created" + "; code2pyc4gpu-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['CODE4PYC'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4gpu + 
             
            code2ack  = code2ack + '33D-'
        except: 
            message2print = "{(Step 33Z2/70) yielded" + "; code2pyc4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi['CODE4PYC']) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2pyc4gpu2nack  = code2pyc4gpu2nack + '33Z2-'     
                            
    if 'CODE4BIN' in keys2task:            
        message2print = "{(Step 33E/70) try" + "; code2bin4gpu2bin4gpu-try-by-create2kernel4gpu: " + str(task4kernel2gpu ['CODE4BIN'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        # {folder2lib4gpu + 
        
        gpu2kernel4xi['CODE4BIN'] = {}
        try:            
            # ---------------------------------------------------------------------------------------------
            # (33) CODE4BIN - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['CODE4BIN'] = task4kernel2gpu ['CODE4BIN']
    
     
            message2print = "{(Step 33F/70) created" + "; code2bin4gpu-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['CODE4BIN'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4gpu + 
             
            code2ack  = code2ack + '33F-'
        except: 
            message2print = "{(Step 33Z3/70) yielded" + "; code2bin4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi['CODE4BIN']) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2bin4gpu2nack  = code2bin4gpu2nack + '33Z3-'     
            
    if 'SIZE4PY' in keys2task:        
        message2print = "{(Step 34A1/70) try" + "; size2py4gpu-try-by-create2kernel4gpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (34B) SIZE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['SIZE4PY']  = task4kernel2gpu['SIZE4PY']  
     
            message2print = "{(Step 34B1/70) initialized" + "; size2py4gpu-initialized-by-create2kernel4gpu to: " + str(gpu2kernel4xi['SIZE4PY'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4gpu + 'gpu2sampler4xi5test.py': 1337}
            code2ack  = code2ack + '34B1-'
        except: 
            message2print = "{(Step 34Z1/70) yielded" + "; size2py4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '34Z1-'     
            
            
    if 'SIZE4CPP' in keys2task:        
        message2print = "{(Step 34A2/70) try" + "; size2cpp4gpu-try-by-create2kernel4gpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (34B) SIZE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['SIZE4CPP']  = task4kernel2gpu['SIZE4CPP']  
     
            message2print = "{(Step 34B2/70) initialized" + "; size2cpp4gpu-initialized-by-create2kernel4gpu to: " + str(gpu2kernel4xi['SIZE4CPP'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4gpu + gpu2dummy4xi5test.cpp': 333}
            code2ack  = code2ack + '34B1-'
        except: 
            message2print = "{(Step 34Z2/70) yielded" + "; size2cpp4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '34Z1-'     
            
    if 'SIZE4C' in keys2task:        
        message2print = "{(Step 34A3/70) try" + "; size2c4gpu-try-by-create2kernel4gpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (34B) SIZE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['SIZE4C']  = task4kernel2gpu['SIZE4C']  
     
            message2print = "{(Step 34B3/70) initialized" + "; size2c4gpu-initialized-by-create2kernel4gpu to: " + str(gpu2kernel4xi['SIZE4C'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4gpu + 
            code2ack  = code2ack + '34B1-'
        except: 
            message2print = "{(Step 34Z3/70) yielded" + "; size2c4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '34Z1-'     
            
    if 'SIZE4CL' in keys2task:        
        message2print = "{(Step 34A4/70) try" + "; size2cl4gpu-try-by-create2kernel4gpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (34B) SIZE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['SIZE4CL']  = task4kernel2gpu['SIZE4CL']  
     
            message2print = "{(Step 34B4/70) initialized" + "; size2cl4gpu-initialized-by-create2kernel4gpu to: " + str(gpu2kernel4xi['SIZE4CL'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4gpu + 
            code2ack  = code2ack + '34B1-'
        except: 
            message2print = "{(Step 34Z4/70) yielded" + "; size2cl4gpu-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '34Z1-'     
                                    
                        
    if 'SIZE4PYC' in keys2task:        
        message2print = "{(Step 34E/70) try" + "; size-try-by-create2kernel4gpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (34F) SIZE4PYC - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['SIZE4PYC']  = task4kernel2gpu['SIZE4PYC']  
     
            message2print = "{(Step 34F/70) initialized" + "; size-initialized-by-create2kernel4gpu to: " + str(gpu2kernel4xi['SIZE4PYC'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4gpu + 
            code2ack  = code2ack + '34F-'
        except: 
            message2print = "{(Step 34Z3/70) yielded" + "; size-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '34Z3-'     
            
    if 'SIZE4BIN' in keys2task:        
        message2print = "{(Step 34G/70) try" + "; size-try-by-create2kernel4gpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (34G) SIZE4BIN - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            gpu2kernel4xi['SIZE4BIN']  = task4kernel2gpu['SIZE4BIN']  
     
            message2print = "{(Step 34H/70) initialized" + "; size-initialized-by-create2kernel4gpu to: " + str(gpu2kernel4xi['SIZE4BIN'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {folder2lib4gpu + 
            code2ack  = code2ack + '34H-'
        except: 
            message2print = "{(Step 34Z4/70) yielded" + "; size-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '34Z4-'
                 
    if 'LANGUAGE' not in keys2task:
        try:             
            # ---------------------------------------------------------------------------------------------
            # (35A0) LANGUAGE - REQUIRED                 
            # ---------------------------------------------------------------------------------------------             
            task4kernel2gpu ['LANGUAGE'] = {}
            
            for key2code4gpu in keys2code4gpu:
                name2program4gpu=os.path.splitext(key2code4gpu)                                                     
                key2extension4gpu = name2program4gpu[1]                                                             
                if key2extension4gpu == '.py':                          
                    task4kernel2gpu ['LANGUAGE'] ['PY']  = "Python"  
                if key2extension4gpu == '.cpp':                          
                    task4kernel2gpu ['LANGUAGE'] ['CPP'] = "C++"
                if key2extension4gpu == '.c':                          
                    task4kernel2gpu ['LANGUAGE'] ['C']   = "C"  
                if key2extension4gpu == '.cl':                          
                    task4kernel2gpu ['LANGUAGE'] ['CL']  = "CL"                                     
                    
            message2print = "{(Step 15B/70) set" + "; language2task4gpu-set-by-create2kernel4gpu to: " + str(task4kernel2gpu ['LANGUAGE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CPP': 'C++', 'PY': 'Python'}
            code2ack  = code2ack + '35B-' 
        except: 
            message2print = "{(Step 35Z0/70) yielded" + "; language-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '35Z0-'
                                     
    if 'LANGUAGE' in keys2task:        
        message2print = "{(Step 35A/70) try" + "; language-try-by-create2kernel4gpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (35) LANGUAGE - PYTHON, C++, C, CL                              
            # ---------------------------------------------------------------------------------------------             
                            
            gpu2kernel4xi['LANGUAGE']  = task4kernel2gpu['LANGUAGE']  
     
            message2print = "{(Step 35B/70) initialized" + "; language-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['LANGUAGE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CPP': 'C++', 'PY': 'Python'}
            code2ack  = code2ack + '35B-'
        except: 
            message2print = "{(Step 35Z1/70) yielded" + "; language-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '35Z1-'  
                     
    if 'COUNT2REFERENCE' not in keys2task:
        try:             
            # ---------------------------------------------------------------------------------------------
            # (36A0) COUNT2REFERENCE - REQUIRED                 
            # ---------------------------------------------------------------------------------------------             
            count2reference2gpu  = 1
           
            message2print = "{(Step 36B/70) set" + "; count2reference2gpu-set-by-create2kernel4gpu to: " + str(count2reference2gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1 
            code2ack  = code2ack + '36B-' 
        
            task4kernel2gpu ['COUNT2REFERENCE'] = count2reference2gpu     
        except: 
            message2print = "{(Step 36Z0/70) yielded" + "; count2reference2gpu-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '36Z0-'             
                
    if 'COUNT2REFERENCE' in keys2task:                
        message2print = "{(Step 36A/70) try" + "; count2reference-try-by-create2kernel4gpu: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (36) COUNT2REFERENCE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
                            
            gpu2kernel4xi['COUNT2REFERENCE']  = task4kernel2gpu['COUNT2REFERENCE']  
     
            message2print = "{(Step 36B/70) initialized" + "; count2reference-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1
            code2ack  = code2ack + '36B-'
        except: 
            message2print = "{(Step 36Z1/70) yielded" + "; count2reference-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '36Z1-' 
                
    if 'PROGRAM' not in keys2task:
        message2print = "{(Step 37/70) warned" + "; keys2task4program-warned-by-create2kernel4gpu: " + " ** " + str(keys2task)  + " ** " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:             
            # ---------------------------------------------------------------------------------------------
            # (37A0) PROGRAM - REQUIRED                 
            # ---------------------------------------------------------------------------------------------             
            task4kernel2gpu ['PROGRAM'] = None
            message2print = "{(Step 37A/70) try" + "; program2gpu-try-by-create2kernel4gpu: " + str(task4kernel2gpu ['PROGRAM'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #             
            try:
                task4kernel2gpu ['PROGRAM'] = create2programwithsource4gpu()
                keys2program = task4kernel2gpu ['PROGRAM'].keys()
                
                message2print = "{(Step 37B/70) set" + "; program-set-by-create2kernel4gpu to: " + str(keys2program)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # ['CODE2ERROR' ... 
                #  'PROGRAM', 'NACK2RETURN', 'CONTEXT', 'COMMANDQUEUE', 'DEVICE', ... 'PIPE']

                code2ack  = code2ack + '37B-'
            except:
                message2print = "{(Step 37Z1/70) failed" + "; task4kernel2gpu-failed-by-create2kernel4gpu: " + str(task4kernel2gpu ['PROGRAM']) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                #             
                code2nack  = code2nack + '37Z-'    
        except: 
            message2print = "{(Step 37Z0/70) yielded" + "; program-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '37Z0-' 
                            
    if 'PROGRAM' in keys2task:
        message2print = "{(Step 37A/70) try" + "; program-try-by-create2kernel4gpu: {}" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        try:            
            # ---------------------------------------------------------------------------------------------
            # (37) PROGRAM - REQUIRED                          
            # ---------------------------------------------------------------------------------------------             
                            
            gpu2kernel4xi['PROGRAM']  = task4kernel2gpu['PROGRAM']  
     
            message2print = "{(Step 37B/70) initialized" + "; program-created-by-create2kernel4gpu: " + str(len(gpu2kernel4xi['PROGRAM']))  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0 ... 
            
            code2ack  = code2ack + '37B-'
        except: 
            message2print = "{(Step 37Z1/70) yielded" + "; program-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '37Z1-' 
                       
    if 'FUNCTION' not in keys2task:
        try:             
            # ---------------------------------------------------------------------------------------------
            # (38) FUNCTION - REQUIRED                
            # --------------------------------------------------------------------------------------------- 
            task4kernel2gpu ['FUNCTION'] = {}
            message2print = "{(Step 38A0/70) set" + "; function4gpu-set-by-create2kernel4gpu to: " + str(task4kernel2gpu ['FUNCTION'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
             
            task4kernel2gpu ['FUNCTION'] ['.py'] = {}
            message2print = "{(Step 38A1/70) set" + "; pythonfunction4gpu-set-by-create2kernel4gpu to: " + str(task4kernel2gpu ['FUNCTION'] ['.py'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
                        
            tag2search4gpu  = kb2kernel4gpu ['.py'] ['TAG']
            message2print = "{(Step 38A2/70) using" + "; tag2search4gpu-using-by-create2kernel4gpu: " + str(tag2search4gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # ... function =
            
            value2search4gpu    = kb2kernel4gpu ['.py'] ['VALUE']
            message2print = "{(Step 38A3/70) using" + "; value2search4gpu-using-by-create2kernel4gpu: " + str(value2search4gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # gpu2sampler4xi5test
            
            attribute2search4gpu= kb2kernel4gpu ['.py'] ['ATTRIBUTE'] 
            message2print = "{(Step 38A4/70) using" + "; attribute2search4gpu-using-by-create2kernel4gpu: " + str(attribute2search4gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # usage =
            try:
                keys2code4gpu = task4kernel2gpu ['PROGRAM'] ['CODE4SOURCE'].keys()
                for key2code4gpu in keys2code4gpu:
                    # ---------------------------------------------------------------------------------------------
                    # SOURCE FILE
                    # ---------------------------------------------------------------------------------------------
                    code2examine4gpu = task4kernel2gpu ['PROGRAM'] ['CODE4SOURCE'] [key2code4gpu]
                    keys2line4gpu = code2examine4gpu.keys()
                    for key2line4gpu in keys2line4gpu:
                        # -----------------------------------------------------------------------------------------
                        # SOURCE LINE
                        # -----------------------------------------------------------------------------------------
                        line2examine4gpu = task4kernel2gpu ['PROGRAM'] ['CODE4SOURCE'] [key2code4gpu] [key2line4gpu]
                        # message2print = "{(Step 38A5/70) examining" + "; line2examine4gpu-examining-by-create2kernel4gpu: " + str(line2examine4gpu)  + "; time: " + time4xi() + "}"
                        # print4xi(class2print, method2print, message2print) 
                        #                             
                        
                        index2pattern = str.find(line2examine4gpu,tag2search4gpu)  
                        if index2pattern == -1:
                            pass
                        else:
                            # -------------------------------------------------------------------------------------
                            # FUNCTION TAG
                            # -------------------------------------------------------------------------------------
                            message2print = "{(Step 38B/70) examining" + "; line2function4gpu-examining-by-create2kernel4gpu: " + str(line2examine4gpu)  + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print) 
                            # ... function = "set2trace4sampler" ... # function = "gpu2sampler4xi5test" usage = "
                             
                            index2function = str.find(line2examine4gpu,value2search4gpu)
                            if index2function == -1:
                                message2print = "{(Step 38C/70) skipped" + "; line2function4gpu-skipped-by-create2kernel4gpu: " + str(line2examine4gpu)  + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print) 
                                # function = "set2trace4sampler" ... # function = "gpu2sampler4xi5test" usage = " ...function = "main" usage = "
                                
                            else:
                                # ----------------------------------------------------------------------------------
                                # KERNEL FUNCTION  
                                # ----------------------------------------------------------------------------------
                                index2usage = str.find(line2examine4gpu,attribute2search4gpu)             # ....function = "gpu2sampler4xi5test" usage = "
                                if index2usage  == -1:                                                
                                    # ------------------------------------------------------------------------------
                                    # NOT OK KERNEL FUNCTION TAG LINE - USAGE ATTRIBUTE MISSING
                                    # ------------------------------------------------------------------------------
                                    message2print = "{(Step 38D/70) warning" + "; line2function4gpu-warning-by-create2kernel4gpu: " + str(line2examine4gpu)  + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print) 
                                    # 
                                    
                                else:
                                    # ------------------------------------------------------------------------------
                                    # OK KERNEL FUNCTION TAG LINE
                                    # ------------------------------------------------------------------------------    
                                    message2print = "{(Step 38E/70) accepted" + "; line2examine4gpu-accepted-by-create2kernel4gpu: " + str(line2examine4gpu)  + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print) 
                                    # ... function = "gpu2sampler4xi5test" usage = "
                                    # 0123456789012345678901234567890123456789012345678901234
                                    
                                    key2pythonfunction4gpu = line2examine4gpu[index2function:index2usage-2]
                                    
                                    task4kernel2gpu ['FUNCTION'] ['.py'] [key2pythonfunction4gpu] = {}                                  

                                    message2print = "{(Step 38F/70) used" + "; key2pythonfunction4gpu-used-by-create2kernel4gpu: " + str(key2pythonfunction4gpu)  + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print) 
                                    # gpu2sampler4xi5test
                                    
                                    code2ack  = code2ack + '38F-'
                            # -------------------------------------------------------------------------------------
                            # 
                            # -------------------------------------------------------------------------------------                                        
                             
            except:
                message2print = "{(Step 38Z1/70) failed" + "; task4kernel2gpu-failed-by-create2kernel4gpu: " + str(task4kernel2gpu ['FUNCTION']) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                #             
                code2nack  = code2nack + '38Z-'            
                                    
            # ---------------------------------------------------------------------------------------------
            # (39) FUNCTION - KERNEL FUNCTIONS FROM FECTHED C++ FILE(S) - NOT FROM PROGRAM OBJECT                
            # --------------------------------------------------------------------------------------------- 
            task4kernel2gpu ['FUNCTION'] ['.cpp'] = {}            
            message2print = "{(Step 39A0F/70) set" + "; cplusfunction4gpu-set-by-create2kernel4gpu to: " + str(task4kernel2gpu ['FUNCTION'] ['.cpp'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}   
            
            tag2search4gpu      = kb2kernel4gpu ['.cpp'] ['TAG']
            message2print = "{(Step 39A2/70) using" + "; tag2search4gpu-using-by-create2kernel4gpu: " + str(tag2search4gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # function =
            
            attribute2search4gpu= kb2kernel4gpu ['.cpp'] ['ATTRIBUTE'] 
            message2print = "{(Step 39A4/70) using" + "; attribute2search4gpu-using-by-create2kernel4gpu: " + str(attribute2search4gpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # usage = kernel
            

            try:
                keys2code4gpu = task4kernel2gpu ['PROGRAM'] ['CODE4SOURCE'].keys()
                for key2code4gpu in keys2code4gpu:
                    # ---------------------------------------------------------------------------------------------
                    # SOURCE FILE
                    # ---------------------------------------------------------------------------------------------
                    code2examine4gpu = task4kernel2gpu ['PROGRAM'] ['CODE4SOURCE'] [key2code4gpu]
                    message2print = "{(Step 39A5/70) using" + "; key2code4gpu-using-by-create2kernel4gpu: " + str(key2code4gpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # gpu2dummy4xi5test.cpp
                       
                    keys2line4gpu = code2examine4gpu.keys()
                    # message2print = "{(Step 39A6/70) using" + "; keys2line4gpu-using-by-create2kernel4gpu: " + str(keys2line4gpu)  + "; time: " + time4xi() + "}"
                    # print4xi(class2print, method2print, message2print) 
                    # 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 ...                    
                    for key2line4gpu in keys2line4gpu:
                        # -----------------------------------------------------------------------------------------
                        # SOURCE LINE
                        # -----------------------------------------------------------------------------------------
                        line2examine4gpu = task4kernel2gpu ['PROGRAM'] ['CODE4SOURCE'] [key2code4gpu] [key2line4gpu]
                        # message2print = "{(Step 39A7/70) examining" + "; line2examine4gpu-examining-by-create2kernel4gpu: " + str(line2examine4gpu)  + "; time: " + time4xi() + "}"
                        # print4xi(class2print, method2print, message2print) 
                        # function = "nbody_kern" usage = "kernel example 1"*/                        
                        
                        index2pattern = str.find(line2examine4gpu,tag2search4gpu)  
                        if index2pattern == -1:
                            pass
                        else:
                            # -------------------------------------------------------------------------------------
                            # FUNCTION TAG 
                            # -------------------------------------------------------------------------------------
                            message2print = "{(Step 39B1/70) examining" + "; line2function4gpu-examining-by-create2kernel4gpu: " + str(line2examine4gpu)  + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # function = "OCLError" usage = "error code checking"*/
                            # function = "nbody_kern" usage = "kernel example 1"*/
                            
                            message2print = "{(Step 39B2/70) using" + "; attribute2search4gpu-using-by-create2kernel4gpu: " + str(attribute2search4gpu)  + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print) 
                            #                                                         
                            index2usage = str.find(line2examine4gpu,attribute2search4gpu)
                            if index2usage == -1:
                                # ----------------------------------------------------------------------------------
                                # NON KERNEL FUNCTION  
                                # ----------------------------------------------------------------------------------                            
                                message2print = "{(Step 39C/70) skipped" + "; line2function4gpu-skipped-by-create2kernel4gpu: " + str(line2examine4gpu)  + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print) 
                                # function = "OCLError" usage = "error code checking"*/
                                
                            else:
                                # ----------------------------------------------------------------------------------
                                # KERNEL FUNCTION  
                                # ----------------------------------------------------------------------------------
                                message2print = "{(Step 39E/70) accepted" + "; line2examine4gpu-accepted-by-create2kernel4gpu: " + str(line2examine4gpu)  + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print) 
                                # function = "nbody_kern" usage = "kernel example 1"*/; 
                                # function = "nbody2_kern" usage = "kernel example 2"*/ 
                                # 012345678901234567890123456789012345678901234567890
                                 
                                key2kernelfunction4gpu = line2examine4gpu[27:index2usage-2]  
                                task4kernel2gpu ['FUNCTION'] ['.cpp'] [key2kernelfunction4gpu] =  {}                                       

                                message2print = "{(Step 39F/70) used" + "; key2kernelfunction4gpu-used-by-create2kernel4gpu: " + str(key2kernelfunction4gpu)  + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print) 
                                # nbody_kern; nbody2_kern 
                                    
                                code2ack  = code2ack + '39F-'
                                    
            except: 
                message2print = "{(Step 37Z1/70) yielded" + "; program-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                code2nack  = code2nack + '37Z1-'
        except: 
            message2print = "{(Step 37Z0/70) yielded" + "; function-gpu2kernel4xi-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '37Z0-'                               
                
    if 'FUNCTION' in keys2task:    
        message2print = "{(Step 38A/70) trying" + "; function-trying-by-create2kernel4gpu: " + str(task4kernel2gpu['FUNCTION'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'.py': {'gpu2sampler4xi5test': {}}, 
        # '.cpp': {'nbody_kern': {}, 'nbody2_kern': {}}}    
        try:            
            # ---------------------------------------------------------------------------------------------
            # (38) FUNCTION - REQUIRED                          
            # ---------------------------------------------------------------------------------------------             
                            
            gpu2kernel4xi['FUNCTION']  = task4kernel2gpu['FUNCTION']   
     
            message2print = "{(Step 38B/70) created" + "; function-created-by-create2kernel4gpu: " + str(gpu2kernel4xi['FUNCTION'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'.py': {'gpu2sampler4xi5test': {}}, '.cpp': {'nbody_kern': {}, 'nbody2_kern': {}}} 
            
            code2ack  = code2ack + '38B-'
        except: 
            message2print = "{(Step 38Z1/70) yielded" + "; function-yielded-by-create2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '38Z1-'            
                                       
                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)     
    # -----------------------------------------------------------------------------------------
    gpu2kernel4xi ['ACK2RETURN']  = code2ack
    gpu2kernel4xi ['NACK2RETURN'] = code2nack 
     
    keys2kernel4gpu = gpu2kernel4xi.keys()
    message2print = "{(Step 70A/70) returning" + "; keys2kernel4gpu-returning-by-create2kernel4gpu: " + str(keys2kernel4gpu) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)
    #1 (no parameters given by caller): 
    # ['DIRECTORY4CL',
    #  'DIRECTORY4PY',
    #  'DIRECTORY4C',
    #  'DIRECTORY4CPP',
    #  'DIRECTORY4PYC',    
    #  'DIRECTORY4BIN',    
    #  'CODE4PY',
    #  'CODE4PYC',
    #  'CODE4CPP',
    #  'CODE4BIN',
    #  'CODE4C',
    #  'CODE4CL',
    #  'COUNT4C',                
    #  'COUNT4CL',    
    #  'COUNT4CPP',
    #  'COUNT4PYC',
    #  'COUNT4PY',
    #  'COUNT4BIN',
    #  'SIZE4PY',    
    #  'SIZE4CPP', <-- OK: C and CL MISSING
    #  'SIZE4PYC',
    #  'SIZE4BIN',
                        
    #  'QUEUE',
    #  'PROGRAM',
    #  'CONTEXT',
    #  'PIPE',
    #  'FUNCTION2CALLBACK',
    #  'COMMANDQUEUE',
    #  'PLATFORM',
    #  'FUNCTION',
    #  'CODE2ERROR',
    #  'ACK2RETURN',
    #  'DATA2CALLBACK',
    #  'BUFFER',
    #  'COUNT2REFERENCE',
    #  'DEVICE',
    #  'LANGUAGE',
    #  'NACK2RETURN',
    #  'EVENT']



    
    #2 (function and program parameters given by user):
    # ['FUNCTION',
    #  'ACK2RETURN',
    #  'PROGRAM',
    #  'NACK2RETURN']

    
    keys2programkernel4gpu = gpu2kernel4xi['PROGRAM'].keys()
    message2print = "{(Step 70B/70) returning" + "; keys2programkernel4gpu-returning-by-create2kernel4gpu: " + str(keys2programkernel4gpu) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)
    #1 and 2:
    # ['CODE2ERROR', 'LANGUAGE4SOURCE', 'COUNT4SOURCE' ...
    
    try:
        message2print = "{(Step 70C1/70) returning" + "; count2py4gpu-returning-by-create2kernel4gpu: " + str(gpu2kernel4xi ['COUNT4PY']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 1
    except:
        pass    
    
    try:
        message2print = "{(Step 70C2/70) returning" + "; count2cpp4gpu-returning-by-create2kernel4gpu: " + str(gpu2kernel4xi ['COUNT4CPP']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 1
    except:
        pass    
    
    try:
        message2print = "{(Step 70C3/70) returning" + "; count2c4gpu-returning-by-create2kernel4gpu: " + str(gpu2kernel4xi ['COUNT4C']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 0
    except:
        pass    
    
    try:
        message2print = "{(Step 70C4/70) returning" + "; count2cl4gpu-returning-by-create2kernel4gpu: " + str(gpu2kernel4xi ['COUNT4CL']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 0
    except:
        pass    
            
    try:        
        message2print = "{(Step 70S1/70) returning" + "; size2py4gpu-returning-by-create2kernel4gpu: " + str(gpu2kernel4xi['SIZE4PY']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # pu2sampler4xi5test.py': 1337
    except:
        pass    
            
    try:        
        message2print = "{(Step 70S2/70) returning" + "; size2cpp4gpu-returning-by-create2kernel4gpu: " + str(gpu2kernel4xi['SIZE4CPP']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # gpu2dummy4xi5test.cpp': 331
    except:
        pass    
            
    try:        
        message2print = "{(Step 70S3/70) returning" + "; size2c4gpu-returning-by-create2kernel4gpu: " + str(gpu2kernel4xi['SIZE4C']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 
    except:
        pass    
            
    try:        
        message2print = "{(Step 70S4/70) returning" + "; size2cl4gpu-returning-by-create2kernel4gpu: " + str(gpu2kernel4xi['SIZE4CL']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 
    except:
        pass    


    message2print = "{(Step 70T/70) returning" + "; function2kernel4gpu-returning-by-create2kernel4gpu: " + str(gpu2kernel4xi['FUNCTION']) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)
    # {'.py':  {'gpu2sampler4xi5test': {}},
    #  '.cpp': {'nbody_kern': {}, 'nbody2_kern': {}}
    # }  
        
    
                         
    return gpu2kernel4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "add2kernel4gpu" usage = "
#     
#   print('(Step 3A2/700) OF GPU2KERNEL4XI5TEST - TRYING TO ADD KERNEL INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())        
#   try:
#       task4add2kernel = {}
#       task4add2kernel ['CODE4KERNEL'] = {}
#       task4add2kernel ['CODE4KERNEL'] [folder2lib4gpu + 'gpu2dummy4xi.py'] = {}
#       task4add2kernel ['CODE4KERNEL'] [folder2lib4gpu + 'gpu2dummy4xi.py'] [1] = '... profile = "11" />'
#       task4add2kernel ['SIZE4KERNEL'] = {}
#       task4add2kernel ['SIZE4KERNEL'] [folder2lib4gpu + 'gpu2dummy4xi.py'] = len(task4add2kernel ['CODE4KERNEL'] [folder2lib4gpu + 'gpu2dummy4xi.py'])
#       
#       response4add2kernel = add2kernel4gpu(task4add2kernel)
#       print('(Step 3A3/700) OF GPU2KERNEL4XI5TEST - ADDED KERNEL INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())
#   except:
#       print('(Step 3Z2/700) OF GPU2KERNEL4XI5TEST *** FAILED TO PRINT response4add2kernel: ', response4add2kernel)
#          
#">

# -------------------------------------------------------------------------------------------------   
def add2kernel4gpu(task4kernel2gpu):

    '''
    adds to kernel for GPU
    '''
    global gpu2kernel4xi                                                                                            # Kernel of GPU    
    global folder2lib4gpu  
      
    
    # -----------------------------------------------------------------------------------------
    # (Step 10/70)      
    # -----------------------------------------------------------------------------------------          
    class2print = "add2kernel4gpu" 
    method2print = "add2kernel4gpu"

    message2print = "{(Step 10A/70) started" + "; add2kernel4gpu-started: " + str(add2kernel4gpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # 
  
    message2print = "{(Step 10B/70) passed" + "; task4kernel2gpu-to-add2kernel4gpu: " + str(task4kernel2gpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # 
    code2ack  = ' '
    code2nack = ' ' 
    try:
        keys2kernel4task = task4kernel2gpu ['CODE4KERNEL'].keys()
        message2print = "{(Step 10C/70) using" + "; keys2kernel4task-using-by-add2kernel4gpu: " + str(keys2kernel4task)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #         
    except: 
        message2print = "{(Step 10Z1/70) failed" + "; keys2kernel4task-failed-by-add2kernel4gpu: " + str(task4kernel2gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
    try:
        keys2exist4task =  gpu2kernel4xi ['CODE4KERNEL'].keys() 
        message2print = "{(Step 10D/70) using" + "; keys2exist4task-using-by-add2kernel4gpu: " + str(keys2exist4task)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    except:
        try:
            response2createwb4gpu = create2kernelsinkernel4gpu()
            keys2createwb4gpu = response2createwb4gpu.keys()
            for key2createwb4gpu in keys2createwb4gpu:
                if key2createwb4gpu == 'CODE4KERNEL':
                    message2print = "{(Step 10E/70) created" + "; key2createwb4gpu-created-by-add2kernel4gpu: " + str(key2createwb4gpu) + "; time: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print)
                    # 
            
            keys2exist4task =  gpu2kernel4xi ['CODE4KERNEL'].keys() 
            message2print = "{(Step 10F/70) using" + "; keys2exist4task-using-by-add2kernel4gpu: " + str(keys2exist4task)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        
        except: 
            message2print = "{(Step 10Z2/70) failed" + "; gpu2kernel4xi-failed-by-add2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
    try:            
        # ---------------------------------------------------------------------------------------------
        # (33) CODE - EXPECTED - SEVERAL KERNEL CODE CAN BE ADDED INTO ONE PROGRAM OBJECT                              
        # ---------------------------------------------------------------------------------------------             
        for key2kernel4task in keys2kernel4task:
            try:
                message2print = "{(Step 33A/70) using" + "; key2kernel4task-using-by-add2kernel4gpu: " + str(key2kernel4task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                            
                if key2kernel4task in keys2exist4task:
                    message2print = "{(Step 33Z0/70) yielded" + "; key2kernel4task-yielded-by-add2kernel4gpu to: " + str(key2kernel4task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # folder2lib4gpu +  
                     
                    code2nack  = code2nack + '33Z0-'
                else: 
                    try:
                        gpu2kernel4xi ['CODE4KERNEL'] [key2kernel4task] = task4kernel2gpu ['CODE4KERNEL'] [key2kernel4task]
                    except:
                        message2print = "{(Step 33Z1/70) failed" + "; task4kernel2gpu-failed-by-add2kernel4gpu: " + str(task4kernel2gpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        code2nack  = code2nack + '33Z1-' 
                    try:             
                        message2print = "{(Step 33B/70) added" + "; key2kernel4task-added-by-add2kernel4gpu to: " + str(key2kernel4task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        #folder2lib4gpu +  
                    except:
                        message2print = "{(Step 33Z2/70) failed" + "; task4kernel2gpu-failed-by-add2kernel4gpu: " + str(task4kernel2gpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        code2nack  = code2nack + '33Z2-' 
                    try:             
                        message2print = "{(Step 33C/70) added" + "; code-added-by-add2kernel4gpu to: " + str(gpu2kernel4xi ['CODE4KERNEL'] [key2kernel4task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                    except:
                        message2print = "{(Step 33Z3/70) failed" + "; task4kernel2gpu-failed-by-add2kernel4gpu: " + str(task4kernel2gpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        code2nack  = code2nack + '33Z3-'                 
                    try:
                        gpu2kernel4xi ['COUNT4KERNEL'] = gpu2kernel4xi ['COUNT4KERNEL'] + 1
                        message2print = "{(Step 33D/70) added" + "; count4kernel-added-by-add2kernel4gpu to: " + str(gpu2kernel4xi ['COUNT4KERNEL'])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                    except:
                        message2print = "{(Step 33Z4/70) failed" + "; task4kernel2gpu-failed-by-add2kernel4gpu: " + str(task4kernel2gpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        code2nack  = code2nack + '33Z4-' 
                    try:                            
                        gpu2kernel4xi ['SIZE4KERNEL'] [key2kernel4task]  = task4kernel2gpu['SIZE4KERNEL'] [key2kernel4task]  
                        message2print = "{(Step 33E/70) set" + "; size4kernel-set-by-add2kernel4gpu to: " + str(gpu2kernel4xi ['SIZE4KERNEL'] [key2kernel4task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                        code2ack  = code2ack + '33D-'
                    except:
                        message2print = "{(Step 33Z5/70) failed" + "; task4kernel2gpu-failed-by-add2kernel4gpu: " + str(task4kernel2gpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        code2nack  = code2nack + '33Z5-'    
            except:
                message2print = "{(Step 33Z6/70) failed" + "; task4kernel2gpu-failed-by-add2kernel4gpu: " + str(task4kernel2gpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                code2nack  = code2nack + '33Z6-'                     
    except: 
        message2print = "{(Step 33Z7/70) yielded" + "; kernel-yielded-by-add2kernel4gpu: " + str(gpu2kernel4xi ['CODE4KERNEL']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '33Z7-'       
          
                                                                                                                   
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    gpu2kernel4xi ['ACK2RETURN']  = code2ack
    gpu2kernel4xi ['NACK2RETURN'] = code2nack  
    
    keys2kernelkernels = gpu2kernel4xi ['CODE4KERNEL'].keys()
    for key2kernelkernel in sorted(keys2kernelkernels):
        message2print = "{(Step 70A/70) got" + "; key2kernelkernel-got-by-add2kernel4gpu: " + str(key2kernelkernel) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)                                                              
        #folder2lib4gpu +  


    keys2kernel4gpu = gpu2kernel4xi.keys()
    for key2kernel4gpu in keys2kernel4gpu:
        message2print = "{(Step 70B/70) returning" + "; key2kernel4gpu-returning-by-add2kernel4gpu: " + str(key2kernel4gpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 
        
    return gpu2kernel4xi
            
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "execute2nativekernel4gpu" usage = "
#     
#   print('(Step 2A2/700) OF GPU2KERNEL4XI5TEST - TRYING TO EXECUTE NATIVE KERNEL INTO PROGRAM OBJECT WITH FUNCTION ', time4xi())           
#   try:
#       response4execute = execute2nativekernel4gpu()
#   except:
#       print('(Step 2Z1/700) OF GPU2KERNEL4XI5TEST *** FAILED TO EXECUTE NATIVE KERNEL INTO PROGRAM OBJECT WITH FUNCTION *** ', time4xi())                                                                      
#          
#">

# -------------------------------------------------------------------------------------------------   
def execute2nativekernel4gpu(task4program2gpu=None):

    '''
    executes native kernel of GPU
    '''
    global gpu2kernel4xi                                                                                            # Kernel of GPU    
    global import2use4gpu                                                                                           # Imports of GPU
    global target2function4gpu                                                                                      # Target function of Process4Xi
    global folder2lib4gpu  
            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "execute2nativekernel4gpu" 
    method2print = "execute2nativekernel4gpu"

    message2print = "{(Step 1A/70) started" + "; execute2nativekernel4gpu-started: " + str(execute2nativekernel4gpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # 
  
             
    message2print = "{(Step 1B/70) passed" + "; task4program2gpu-to-execute2nativekernel4gpu: " + str(task4program2gpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 

    code2ack  = ' '
    code2nack = ' '    
    
    kernel2program4gpu = {}                                                                                       # INITIALIZES TO EMPTY DICTIONARY
    message2print = "{(Step 1C/70) initialized" + "; kernel2program4gpu-initialized-by-execute2nativekernel4gpu to: " + str(kernel2program4gpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    #                
    try:             
        if task4program2gpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER
            # -----------------------------------------------------------------------------------------    
            pass                                                                       
        else:
            # -----------------------------------------------------------------------------------------
            # PARAMETERS GIVEN ARE OMITTED 
            # -----------------------------------------------------------------------------------------         
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4program2gpu-yielded-by-execute2nativekernel4gpu: " + str(task4program2gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
       
        code2nack  = code2nack + '1Z1-' 

    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - EXECUTE NATIVE KERNELS INSIDE PROGRAM OBJECT FOR GPU ! 
    # ---------------------------------------------------------------------------------------------    
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) - SOURCE CODE WITHOUT EOL CHARACTERS                            
        # ---------------------------------------------------------------------------------------------             
        keys2kernel4gpu = gpu2kernel4xi['CODE4KERNEL'].keys()
        message2print = "{(Step 30A/70) using" + "; keys2kernel4gpu-using-by-execute2nativekernel4gpu: " + str(keys2kernel4gpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        unit4processing = "gpu"
        
        for key2kernel4gpu in keys2kernel4gpu:
            index2search=str.find(key2kernel4gpu,unit4processing) 
            if index2search == -1:
                pass
            else:
                target2execute = key2kernel4gpu[index2search:]                                                          # gpu2sampler4xi5test.py
                index2search=str.find(target2execute,".py")
                if index2search == -1:
                    pass
                else:
                    target2execute4gpu = target2execute[:index2search]                                                  
                    message2print = "{(Step 32A/70) set" + "; target2execute4gpu-set-by-execute2nativekernel4gpu to: " + str(target2execute4gpu)  + "; time: " + time4xi() + "}"  
                    print4xi(class2print, method2print, message2print)
                    # 

                    p2kernel4gpu = None    
                    try:
                        # ---------------------------------------------------------------------------------------------    
                        # IMPORT TARGET FUNCTION FROM ITS MODULE AS TARGET2FUNCTION4GPU 
                        # ---------------------------------------------------------------------------------------------    
             
                        try:
                            import2use4gpu.get(target2execute4gpu, handle2error4gpu) ()
                            
                            message2print = "{(Step 32B/70) imported" + "; target2execute4gpu-imported-by-execute2nativekernel4gpu: " + str(target2execute4gpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # 
                            code2ack  = code2ack + '32B-'
                        except:
                            message2print = "{(Step 32Z1/70) failed" + "; import2use4gpu-failed-by-execute2nativekernel4gpu: " + str(import2use4gpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)                            
                            # 
                            message2print = "{(Step 32Z2/70) used" + "; target2execute4gpu-used-by-execute2nativekernel4gpu: " + str(target2execute4gpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # 
                            code2nack  = code2nack + '32Z2-'
                                    
                        # ---------------------------------------------------------------------------------------------    
                        # CREATE PROCESS WITH TARGET2FUNCTION4GPU
                        # ---------------------------------------------------------------------------------------------                          
                        p2gpu = Process4Xi(target=target2function4gpu, args=())                                
                        message2print = "{(Step 33A/70) created" + "; p2gpu-created-by-execute2nativekernel4gpu: " + str(p2gpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # <Process4Xi(Process4Xi-1, initial)>
                        
                        code2ack  = code2ack + '33A-'
                        try:
                            # ---------------------------------------------------------------------------------------------    
                            # START PROCESS
                            # --------------------------------------------------------------------------------------------- 
                            p2gpu.start()
                            message2print = "{(Step 34A/70) started" + "; p2gpu-started-by-execute2nativekernel4gpu: " + str(p2gpu.name) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # Process4Xi-1
                            
                            try:
                                # ---------------------------------------------------------------------------------------------    
                                # STATUS OF PROCESS
                                # ---------------------------------------------------------------------------------------------                             
                                status2process4gpu = p2gpu.is_alive()
                                message2print = "{(Step 35A/70) got" + "; status-got-by-execute2nativekernel4gpu: " + str(status2process4gpu) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                # True
                                
                                try:
                                    # ---------------------------------------------------------------------------------------------    
                                    # RUN PROCESS
                                    # ---------------------------------------------------------------------------------------------                                  
                                    p2gpu.run()
                                    message2print = "{(Step 36A/70) run" + "; p2gpu-run-by-execute2nativekernel4gpu: " + str(p2gpu) + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print)
                                    # <Process4Xi(Process4Xi-1, stopped[1])>
                                except:
                                    message2print = "{(Step 36Z/70) failed" + "; run-failed-by-execute2nativekernel4gpu: " + str(p2gpu) + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print)  
                                    code2nack  = code2nack + '36Z-'            
                                    # 
                            except:
                                message2print = "{(Step 35Z/70) failed" + "; status-failed-by-execute2nativekernel4gpu: " + str(p2gpu) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)  
                                # 
                                code2nack  = code2nack + '35Z-'                                   
                                
                        except:
                            message2print = "{(Step 33Z/70) failed" + "; start-failed-by-execute2nativekernel4gpu: " + str(p2gpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)  
                            code2nack  = code2nack + '33Z-'     
                        
                    except:
                        message2print = "{(Step 32Z/70) failed" + "; p2kernel4gpu-failed-by-execute2nativekernel4gpu: " + str(p2kernel4gpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        code2nack  = code2nack + '32Z-' 
                                                                                    
            # ---------------------------------------------------------------------------------------------
            # SAVE SOURCE WITHOUT END OF LINES
            # ---------------------------------------------------------------------------------------------        
            keys2lines = gpu2kernel4xi['CODE4KERNEL'][key2kernel4gpu].keys()     
            kernel2program4gpu[key2kernel4gpu] = ""
            for key2line in keys2lines:
                code2add4gpu = gpu2kernel4xi ['CODE4KERNEL'] [key2kernel4gpu] [key2line]
                kernel2program4gpu[key2kernel4gpu] = kernel2program4gpu[key2kernel4gpu] + code2add4gpu.replace("\n","")
        
            message2print = "{(Step 34A/70) saved" + "; lenght-kernel4gpu-saved-by-execute2nativekernel4gpu to: " + str(len(kernel2program4gpu[key2kernel4gpu]))  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #     
                
 
        message2print = "{(Step 38A/70) set" + "; lenght-kernel2program4gpu-set-by-execute2nativekernel4gpu to: " + str(len(kernel2program4gpu))  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
         
        code2ack  = code2ack + '38A-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; kernel-yielded-by-execute2nativekernel4gpu: " + str(gpu2kernel4xi['CODE4KERNEL']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '30Z1-'     

                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    gpu2kernel4xi ['ACK2RETURN']  = code2ack
    gpu2kernel4xi ['NACK2RETURN'] = code2nack  
    
    keys2kernel4gpu = kernel2program4gpu.keys()
    for key2kernel4gpu in keys2kernel4gpu:
        message2print = "{(Step 70/70) returning" + "; key2kernel4gpu-returning-by-execute2nativekernel4gpu: " + str(key2kernel4gpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)                                                              
        # 
    
    return kernel2program4gpu
            
#</function4xi> 
 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "delete2kernel4gpu" usage = "
#     
#   print('(Step 3A3/700) OF GPU2KERNEL4XI5TEST - TRYING TO DELETE KERNEL INSIDE PROGRAM OBJECT WITH FUNCTION ', time4xi())        
#   try:
#       task4delete2kernel = {}
#       task4delete2kernel ['CODE4KERNEL'] = {}
#       task4delete2kernel ['CODE4KERNEL'] [folder2lib4gpu + 'gpu2program4xi.py'] = 'Yes'
#       task4delete2kernel ['SIZE4KERNEL'] = {}
#       task4delete2kernel ['SIZE4KERNEL'] [folder2lib4gpu + 'gpu2program4xi.py'] = 'Yes'
#       
#       response4delete2kernel = delete2kernel4gpu(task4delete2kernel) 
#       print('(Step 3A3/700) OF GPU2KERNEL4XI5TEST - DELETED KERNEL INSIDE PROGRAM OBJECT WITH FUNCTION ', time4xi())
#   except:
#       print('(Step 3Z3/700) OF GPU2KERNEL4XI5TEST *** FAILED TO DELETE KERNEL INSIDE PROGRAM OBJECT WITH FUNCTION *** ', time4xi()) 
#          
#">

# -------------------------------------------------------------------------------------------------   
def delete2kernel4gpu(task4kernel2gpu):

    '''
    deletes kernel for GPU
    '''
    global gpu2kernel4xi                                                                                     
    global folder2lib4gpu  
      
    
    # -----------------------------------------------------------------------------------------
    # (Step 10/70)      
    # -----------------------------------------------------------------------------------------          
    class2print = "delete2kernel4gpu" 
    method2print = "delete2kernel4gpu"

    message2print = "{(Step 10A/70) started" + "; delete2kernel4gpu-started: " + str(delete2kernel4gpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function delete2kernel4gpu at 0x000000000405CEC8>
  
    message2print = "{(Step 10B/70) passed" + "; task4kernel2gpu-to-delete2kernel4gpu: " + str(task4kernel2gpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 


    code2ack  = ' '
    code2nack = ' '    
    #
    ack4key2kernelkernel = False
    try:
        # ---------------------------------------------------------------------------------------------
        # TASK MUST HAVE BEEN GIVEN BY USER - CONTAINS KERNEL KEYS    
        # ---------------------------------------------------------------------------------------------
        keys2delete4kernel = task4kernel2gpu ['CODE4KERNEL'].keys()        
        message2print = "{(Step 19A/70) using" + "; keys2delete4kernel-using-by-delete2kernel4gpu: " + str(sorted(keys2delete4kernel))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # 
        
        # ---------------------------------------------------------------------------------------------
        # (Step 20A/70) - DELETE KERNEL INSIDE PROGRAM OBJECT FOR GPU ! 
        # ---------------------------------------------------------------------------------------------    
        message2print = "{(Step 20A0/70) try" + "; key-try-by-delete2kernel4gpu: None" + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)      

        for key2delete4kernel in keys2delete4kernel:
            try:            
                del gpu2kernel4xi ['CODE4KERNEL'] [key2delete4kernel]        
               
                message2print = "{(Step 20C/70) deleted" + "; code4kernel-deleted-by-delete2kernel4gpu: " + str(key2delete4kernel) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                # 
                
                code2ack  = code2ack + '20C-' 
                
                del gpu2kernel4xi ['SIZE4KERNEL'] [key2delete4kernel]        
               
                message2print = "{(Step 20D/70) deleted" + "; size4kernel-deleted-by-delete2kernel4gpu: " + str(key2delete4kernel) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                # 
                
                code2ack  = code2ack + '20D-' 
                
                gpu2kernel4xi ['COUNT4KERNEL'] = gpu2kernel4xi ['COUNT4KERNEL'] - 1
                message2print = "{(Step 20E/70) upgraded" + "; count4kernel-upgraded-by-delete2kernel4gpu: " + str(gpu2kernel4xi ['COUNT4KERNEL'] ) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                    
                #                                
                
            except:
                message2print = "{(Step 20Z1/70) yielded" + "; key2delete4kernel-yielded-by-delete2kernel4gpu: " + str(key2delete4kernel) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                
                code2nack  = code2nack + '20Z1-' 
    except: 
        message2print = "{(Step 20Z0/70) failed" + "; task4kernel2gpu-gpu2kernel4xi-failed-by-delete2kernel4gpu: " + str(task4kernel2gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '20Z0-'    
                                                                                            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    gpu2kernel4xi ['ACK2RETURN']  = code2ack
    gpu2kernel4xi ['NACK2RETURN'] = code2nack  
    try:
        keys2kernelkernels = gpu2kernel4xi ['CODE4KERNEL'] .keys()
        for key2kernelkernels in keys2kernelkernels:
            message2print = "{(Step 70A/70) got" + "; key2kernelkernels-got-by-delete2kernel4gpu: " + str(key2kernelkernels) + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print)                                                              
            #  
    except:
        pass 
               
     
    keys2kernel4gpu = gpu2kernel4xi.keys()
    for key2kernel4gpu in keys2kernel4gpu:
        message2print = "{(Step 70B/70) returning" + "; key2kernel4gpu-returning-by-delete2kernel4gpu: " + str(key2kernel4gpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 


    return gpu2kernel4xi
            
#</function4xi> 
 



# -------------------------------------------------------------------------------------------------
#<function4xi function = "create2kernelsinprogram4gpu" usage = "
#     
#   print('(Step 3A1/700) OF GPU2KERNEL4XI5TEST - TRYING TO CREATE KERNELS IN PROGRAM FOR GPU ', time4xi())          
#   try:
#       response4create2kernelsinprogram = create2kernelsinprogram4gpu()
#       print('(Step 3A2/700) OF GPU2KERNEL4XI5TEST - CREATED KERNELS IN PROGRAM FOR GPU ', time4xi())
#   except:
#       print('(Step 3Z1/700) OF GPU2KERNEL4XI5TEST *** FAILED TO CREATE KERNELS IN PROGRAM FOR GPU *** ', time4xi())
#
#">

# -------------------------------------------------------------------------------------------------   
def create2kernelsinprogram4gpu(task4kernel2gpu=None):

    '''
    creates kernels in program for GPU
    '''
    global gpu2kernel4xi                                                                                            # Kernel of GPU    
    global folder2lib4gpu  
      
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - START            
    # -----------------------------------------------------------------------------------------          
    class2print = "create2kernelsinprogram4gpu" 
    method2print = "create2kernelsinprogram4gpu"

    message2print = "{(Step 1A/70) started" + "; create2kernelsinprogram4gpu-started: " + str(create2kernelsinprogram4gpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # 
  
             
    message2print = "{(Step 1B/70) passed" + "; task4kernel2gpu-to-create2kernelsinprogram4gpu: " + str(task4kernel2gpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 

    code2ack  = ' '
    code2nack = ' '    

    
    keys2kernel4gpu = gpu2kernel4xi.keys()
    for key2kernel4gpu in keys2kernel4gpu:
        message2print = "{(Step 70/70) returning" + "; key2kernel4gpu-returning-by-create2kernelsinprogram4gpu: " + str(key2kernel4gpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 
    
    return gpu2kernel4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "fetch2kernel4gpu" usage = "
#   fetch2kernel4gpu(file2walk)
#"> 
# -------------------------------------------------------------------------------------------------
def fetch2kernel4gpu(program2explore4gpu):
    global fetch2code4gpu                                                                                           # dictionary to receive files 
        
    global kb2fetch4gpu                                                                                             # knowledgebase for fetching
    global profile2fetch                                                                                            # profiles
    global keys2profile4gpu                                                                                         # file extensions 
    
    global folder2lib4gpu  

    class2print  = "fetch2kernel4gpu"
    method2print = "fetch2kernel4gpu" 
     
    message2print = "{(Step 1A/33) passed" + "; program2explore4gpu-passed-to-fetch2kernel4gpu: " + str(program2explore4gpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # folder2lib4gpu + gpu2buffer4xi.py          ... gpu2sampler4xi5test.py ... gpu2dummy4xi5test.cpp ...           # Python and C++ programs
    # __pycache__\gpu2buffer4xi.cpython-33.pyc   ... gpu2sampler4xi5test.cpython-33.pyc ...                         # Cache of Python binaries
    # //bin\gpu2dummy4xi5test.exe                                                                                   # Built C++ exe files
                  
    # ---------------------------------------------------------------------------------------------
    # SPLIT PROGRAM NAME WITH PATH AND EXTENSION TO TWO PIECES
    # ---------------------------------------------------------------------------------------------    
    name2program4gpu=os.path.splitext(program2explore4gpu)                                                          # gpu2sampler4xi5test + py

    key2extension4gpu = name2program4gpu[1]                                                                         # py,pyc,cpp,exe 
    message2print = "{(Step 1B/33) set" + "; key2extension4gpu-set-by-fetch2kernel4gpu to: " + str(key2extension4gpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)          
    # .py; .pyc
    message2print = "{(Step 1C/33) using" + "; profile2fetch-using-by-fetch2kernel4gpu: " + str(profile2fetch)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)          
    # {'.pyc': 'NACK', '.py': 'ACK', '.cpp': 'ACK', '.exe': 'NACK'}  
        
    if profile2fetch [key2extension4gpu] == 'ACK':                                                                  # profile2fetch ['py']  = 'ACK'       
        # -----------------------------------------------------------------------------------------
        # (Step 1/33) OF FETCH2SCOPE4XI - EXTENSION IN PROFILE 
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 1A/33) analyzing" + "; program2explore4gpu-analyzing-by-fetch2kernel4gpu: " + str(program2explore4gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        # folder2lib4gpu +  gpu2buffer4xi.py ; gpu2sampler4xi5test.py ...  ... __pycache__\gpu2sampler4xi5test.cpython-33.pyc
        # gpu2dummy4xi5test.cpp
             
        path2input4gpu = program2explore4gpu
        message2print = "{(Step 1B/33) set" + "; path2input4gpu-set-by-fetch2kernel4gpu to: " + str(path2input4gpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        # folder2lib4gpu +  gpu2buffer4xi.py ; __pycache__\gpu2sampler4xi5test.cpython-33.pyc; gpu2dummy4xi5test.cpp

        # -----------------------------------------------------------------------------------------
        # (Step 2/33) OF FETCH2SCOPE4XI - CHECKING WHETHER PROGRAM MUST BE FETCHED 
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A1/33) analyzing" + "; path2input4gpu-analyzing-by-fetch2kernel4gpu: " + str(path2input4gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #folder2lib4gpu +  gpu2buffer4xi.py ; gpu2dummy4xi5test.cpp 
            
        keys2fetch4gpu = kb2fetch4gpu.keys()
        message2print = "{(Step 2A2/33) using" + "; keys2fetch4gpu-using-by-fetch2kernel4gpu: " + str(keys2fetch4gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #        
        ack2match4file = False
        for key2fecth4gpu in keys2fetch4gpu:                                                                        # 1,2 ...
            # -----------------------------------------------------------------------------------------
            # 
            # -----------------------------------------------------------------------------------------
            index2search=str.find(path2input4gpu,kb2fetch4gpu[key2fecth4gpu])                                       
            if index2search == -1:                                         
                # -----------------------------------------------------------------------------------------
                # VALID FILE TYPE BUT NOT SELECTED TO BE FETCHED - OK ONE MAY ALSO PASS HERE!
                # -----------------------------------------------------------------------------------------               
                message2print = "{(Step 2C1/33) skipped" + "; path2input4gpu-skipped-by-fetch2kernel4gpu: " + str(path2input4gpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                      
                # folder2lib4gpu +  gpu2buffer4xi.py 
                
            else:
                ack2match4file = True
                
                message2print = "{(Step 2D/33) decided" + "; program-to-read-decided-by-fetch2kernel4gpu: " + str(path2input4gpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)        
                #folder2lib4gpu +  gpu2sampler4xi5test.py; __pycache__//gpu2sampler4xi.cpython-33.pyc; 
                #               +  gpu2dummy4xi5test.cpp

                # -----------------------------------------------------------------------------------------
                # (Step 3/33) OF FETCH2SCOPE4XI - OPEN INPUT FILE - READ MODE SET ACCORDING TO FILE TYPE
                # -----------------------------------------------------------------------------------------
                if ((key2extension4gpu == '.py') or (key2extension4gpu == '.cpp') or (key2extension4gpu == '.c') or (key2extension4gpu == '.cl')):
                    file2input4gpu = open(path2input4gpu,'r')
                    message2print = "{(Step 3A/33) opened" + "; file2input4gpu-opened-by-fetch2kernel4gpu: " + str(file2input4gpu) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)     
                    # <_io.TextIOWrapper name=folder2lib4gpu + 'gpu2sampler4xi5test.py' mode='r' encoding='cp1252'> 
                     
                if ((key2extension4gpu == '.pyc') or (key2extension4gpu == '.exe')):
                    file2input4gpu = open(path2input4gpu,'rb')
                    message2print = "{(Step 3B/33) opened" + "; file2input4gpu-opened-by-fetch2kernel4gpu: " + str(file2input4gpu) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)     
                    #  __pycache__//gpu2sampler4xi.cpython-33.pyc               
                    
                # -----------------------------------------------------------------------------------------
                # (Step 4/33) OF FETCH2SCOPE4XI - OUTPUT 
                # -----------------------------------------------------------------------------------------
                key2save4gpu = path2input4gpu
                fetch2code4gpu[key2save4gpu] = {}                                                                   # fetch2code4gpu['gpu2sampler4xi5test.py'] = {}
                number2line4gpu = 0  
                message2print = "{(Step 3B/33) saving" + "; key2fecth4gpu-saving-by-fetch2kernel4gpu: " + str(key2fecth4gpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)              
                # 
         
                while 1:
                    # -------------------------------------------------------------------------------------
                    # (Step 5/33) OF FETCH2SCOPE4XI - PROCESS FILE
                    # -------------------------------------------------------------------------------------             
                    line2process4gpu = file2input4gpu.readline()  
                    if not line2process4gpu:
                        # ---------------------------------------------------------------------------------
                        # (Step 30/33) OF FETCH2SCOPE4XI - ALL LINES READ, FINISH
                        # ---------------------------------------------------------------------------------             
                        break                                               
                    else:
                        # ---------------------------------------------------------------------------------
                        # (Step 7/33) OF FETCH2SCOPE4XI - SAVE LINE
                        # ---------------------------------------------------------------------------------
                        # message2print = "{(Step 7A/33) read" + "; line2process4gpu-read-by-fetch2kernel4gpu: " + str(line2process4gpu) + "; time: " + time4xi() + "}"
                        # print4xi(class2print, method2print, message2print)              
                        # ... profile = "11" />
                        # ...
                        # # end of gpu2sampler4xi5test.py 
                        # #/> 
                        try:
                            number2line4gpu = number2line4gpu + 1
                            fetch2code4gpu[key2save4gpu][number2line4gpu] = line2process4gpu
                        except:
                            message2print = "{(Step 7Z1/33) failed" + "; key2fecth4gpu-failed-by-fetch2kernel4gpu: " + str(key2fecth4gpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            
                            message2print = "{(Step 7Z2/33) failed" + "; line2process4gpu-failed-by-fetch2kernel4gpu: " + str(line2process4gpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            
                            message2print = "{(Step 7Z3/33) failed" + "; number2line4gpu-failed-by-fetch2kernel4gpu: " + str(number2line4gpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)                            
                                                             
                # -----------------------------------------------------------------------------------------
                # (Step 31/33) OF FETCH2SCOPE4XI - CLOSE INPUT
                # -----------------------------------------------------------------------------------------            
                file2input4gpu.close()
                message2print = "{(Step 31A/33) closed" + "; file2input4gpu-closed-by-fetch2kernel4gpu: " + str(file2input4gpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)           
                # <_io.TextIOWrapper name=folder2lib4gpu + 'gpu2sampler4xi5test.py' mode='r' encoding='cp1252'>     # Python source files
                # {folder2lib4gpu + 'gpu2sampler4xi5test.py': {1: '... profile = "11" />\n', 2: '# -- 
                # gpu2dummy4xi5test.cpp                                                                             # C++ source files
                # __pycache__//gpu2sampler4xi.cpython-33.pyc                                                        # Cached binary files of Python
                # gpu2dummy4xi5test.exe                                                                             # C++ exe files
        
                message2print = "{(Step 31B1/33) saved" + "; fetch2code4gpu-saved-by-fetch2kernel4gpu: " + str(fetch2code4gpu[key2save4gpu] [1]) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # ... profile = "11" />
                
                message2print = "{(Step 31B2/33) saved" + "; fetch2code4gpu-saved-by-fetch2kernel4gpu: " + str(fetch2code4gpu[key2save4gpu] [2]) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # gpu2dummy4xi5test.cpp
                
                message2print = "{(Step 31B3/33) saved" + "; fetch2code4gpu-saved-by-fetch2kernel4gpu: " + str(fetch2code4gpu[key2save4gpu] [3]) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # ... program = "gpu2sampler4xi5test.py">
                
                message2print = "{(Step 31B4/33) saved" + "; fetch2code4gpu-saved-by-fetch2kernel4gpu: " + str(fetch2code4gpu[key2save4gpu] [4]) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # ...  abstract = "tests gpu2sampler4xi.py"/>
                                                                 
                # ---------------------------------------------------------------------------------------------
                #
                # ---------------------------------------------------------------------------------------------                                          


    else:
        # -----------------------------------------------------------------------------------------
        # FILE TYPE - NOT IN PROFILE
        # -----------------------------------------------------------------------------------------       
        message2print = "{(Step 1F/33) skipped" + "; name2program4gpu-skipped-by-fetch2kernel4gpu: " + str(name2program4gpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      
        # folder2lib4gpu + '__pycache__//gpu2buffer4xi.cpython-32', '.pyc')
        
        extension2skip = name2program4gpu[1]
        message2print = "{(Step 1G/33) skipped" + "; extension-skipped-by-fetch2kernel4gpu: " + str(extension2skip[1:])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # pyc

        message2print = "{(Step 1H/33) returning" + "; fetch2kernel4gpu-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
 
    message2print = "{(Step 1H/33) returning" + "; fetch2kernel4gpu-returning: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)            
    #   
                                            
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2kernel4gpu" usage = "
#
#   ... prepare task4set2kernelws ...    
#
#   response4set2kernel = {} 
#   print('(Step 17F/700) OF GPU2KERNEL4XI5TEST - TRYING TO SET KERNEL: ', set2kernel4gpu)   
#   # 
#   try:                               
#       response4set2kernel = set2kernel4gpu(task4set2kernelws)
#       print('(Step 17G/700) OF GPU2KERNEL4XI5TEST - SET KERNEL: ', response4set2kernel)                                                                                                           
#   except:
#       print('(Step 17Z/700) OF GPU2KERNEL4XI5TEST *** FAILED TO SET KERNEL: ', set2kernel4gpu)  
#
#">

# -------------------------------------------------------------------------------------------------   
def set2kernel4gpu(task4kernelset2gpu=None):

    '''
    sets kernels for GPU
    '''
    global gpu2kernel4xi  
    global folder2lib4gpu  
        
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set2kernel4gpu"
    method2print = "set2kernel4gpu" 

    message2print = "{(Step 1A/33) started" + "; set2kernel4gpu-started: " + str(set2kernel4gpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set2kernel4gpu at 0x0000000003837948> 
    
    keys2task = task4kernelset2gpu.keys()
    for key2task in sorted(keys2task):
        message2print = "{(Step 1B/33) passed" + "; key2task-passed-to-set2kernel4gpu: " + str(key2task) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
        # 
        
    code2ack  = ' '
    code2nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if gpu2kernel4xi is None:
            gpu2kernel4xi = create2kernel4gpu()                                                                              # INITIALIZE PROGRAM FOR GPU 
            message2print = "{(Step 1C/70) set" + "; gpu2kernel4xi-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2kernel4gpu-skipped-by-set2kernel4gpu: " + str(create2kernel4gpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                 
    except:
        try:
            gpu2kernel4xi = create2kernel4gpu()
            message2print = "{(Step 1E/70) set" + "; gpu2kernel4xi-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            gpu2kernel4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; gpu2kernel4xi-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET       
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data2callback-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        gpu2kernel4xi['DATA2CALLBACK'] = task4kernelset2gpu['DATA2CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data2callback-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-gpu2kernel4xi-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        gpu2kernel4xi['FUNCTION2CALLBACK'] = task4kernelset2gpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function2callback-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-gpu2kernel4xi-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        gpu2kernel4xi['PLATFORM'] = task4kernelset2gpu['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #   
        code2ack  = code2ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-gpu2kernel4xi-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2gpu-try-by-set2kernel4gpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        gpu2kernel4xi['CONTEXT'] = task4kernelset2gpu['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-gpu2kernel4xi-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set2kernel4gpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        gpu2kernel4xi['DEVICE'] = task4kernelset2gpu['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-gpu2kernel4xi-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        gpu2kernel4xi['CODE2ERROR'] = task4kernelset2gpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code2error-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-gpu2kernel4xi-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        gpu2kernel4xi['QUEUE'] = task4kernelset2gpu['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
         
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-gpu2kernel4xi-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        gpu2kernel4xi['PIPE'] = task4kernelset2gpu['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-gpu2kernel4xi-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set2kernel4gpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        gpu2kernel4xi['COMMANDQUEUE']  = task4kernelset2gpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set2kernel4gpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
        gpu2kernel4xi['BUFFER']  = task4kernelset2gpu['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set2kernel4gpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        gpu2kernel4xi['EVENT']  = task4kernelset2gpu['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '30Z1-'
                    
              
    message2print = "{(Step 31A/70) try" + "; count2py4gpu-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31)  - COUNT4PY                           
        # ---------------------------------------------------------------------------------------------             
        gpu2kernel4xi['COUNT4PY']  = task4kernelset2gpu['COUNT4PY']  
 
        message2print = "{(Step 31B/70) set" + "; count2py4gpu-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['COUNT4PY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 1
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; count2py4gpu-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '31Z1-'  
                
    message2print = "{(Step 31C/70) try" + "; count2pyc4gpu-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31C)  - COUNT4PYC                           
        # ---------------------------------------------------------------------------------------------             
        gpu2kernel4xi['COUNT4PYC']  = task4kernelset2gpu['COUNT4PYC']  
 
        message2print = "{(Step 31D/70) set" + "; count2pyc4gpu-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['COUNT4PYC'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 1
        code2ack  = code2ack + '31D-'
    except: 
        message2print = "{(Step 31Z2/70) yielded" + "; count2pyc4gpu-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '31Z2-'  
                
    message2print = "{(Step 31E/70) try" + "; count2c4gpu-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31E)  - COUNT4C                           
        # ---------------------------------------------------------------------------------------------             
        gpu2kernel4xi['COUNT4C']  = task4kernelset2gpu['COUNT4C']  
 
        message2print = "{(Step 31F/70) set" + "; count2c4gpu-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['COUNT4C'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 1
        code2ack  = code2ack + '31F-'
    except: 
        message2print = "{(Step 31Z3/70) yielded" + "; count2c4gpu-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '31Z3-'                  
                
    message2print = "{(Step 31G/70) try" + "; count2bin4gpu-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31G)  - COUNT4BIN                           
        # ---------------------------------------------------------------------------------------------             
        gpu2kernel4xi['COUNT4BIN']  = task4kernelset2gpu['COUNT4BIN']  
 
        message2print = "{(Step 31H/70) set" + "; count2bin4gpu-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['COUNT4BIN'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 1
        code2ack  = code2ack + '31H-'
    except: 
        message2print = "{(Step 31Z4/70) yielded" + "; count2bin4gpu-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '31Z4-'  
        
    message2print = "{(Step 32A/70) try" + "; directory-try-by-set2kernel4gpu: " + str(task4kernelset2gpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print) 
    # 
    
    try:            
        # ---------------------------------------------------------------------------------------------
        # (32B) DIRECTORY4PY - REQUIRED                     
        # ---------------------------------------------------------------------------------------------                                    
        gpu2kernel4xi['DIRECTORY4PY']  = task4kernelset2gpu['DIRECTORY4PY'] 
        
        message2print = "{(Step 32B/70) set" + "; directory2py4gpu-set-by-set2kernelwithdirectory2py4gpu4gpu to: " + str(gpu2kernel4xi['DIRECTORY4PY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '32B-'
    except: 
        message2print = "{(Step 32Z1/70) yielded" + "; directory2py4gpu-yielded-by-set2kernelwithdirectory2py4gpu4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '32Z1-' 

    try:            
        # ---------------------------------------------------------------------------------------------
        # (32C) DIRECTORY4PYC - REQUIRED                     
        # ---------------------------------------------------------------------------------------------                                    
        gpu2kernel4xi['DIRECTORY4PYC']  = task4kernelset2gpu['DIRECTORY4PYC'] 
        
        message2print = "{(Step 32C/70) set" + "; directory2pyc4gpu-set-by-set2kernelwithdirectory2pyc4gpu4gpu to: " + str(gpu2kernel4xi['DIRECTORY4PYC'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '32C-'
    except: 
        message2print = "{(Step 32Z2/70) yielded" + "; directory2pyc4gpu-yielded-by-set2kernelwithdirectory2pyc4gpu4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '32Z2-'

    try:            
        # ---------------------------------------------------------------------------------------------
        # (32D) DIRECTORY4C - REQUIRED                     
        # ---------------------------------------------------------------------------------------------                                    
        gpu2kernel4xi['DIRECTORY4CPP']  = task4kernelset2gpu['DIRECTORY4CPP'] 
        
        message2print = "{(Step 32D/70) set" + "; directory2c4gpu-set-by-set2kernelwithdirectory2c4gpu4gpu to: " + str(gpu2kernel4xi['DIRECTORY4CPP'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '32D-'
    except: 
        message2print = "{(Step 32Z3/70) yielded" + "; directory2c4gpu-yielded-by-set2kernelwithdirectory2c4gpu4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '32Z3-' 

    try:            
        # ---------------------------------------------------------------------------------------------
        # (32E) DIRECTORY4BIN - REQUIRED                     
        # ---------------------------------------------------------------------------------------------                                    
        gpu2kernel4xi['DIRECTORY4BIN']  = task4kernelset2gpu['DIRECTORY4BIN'] 
        
        message2print = "{(Step 32E/70) set" + "; directory2bin4gpu-set-by-set2kernelwithdirectory2bin4gpu4gpu to: " + str(gpu2kernel4xi['DIRECTORY4BIN'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '32E-'
    except: 
        message2print = "{(Step 32Z4/70) yielded" + "; directory2bin4gpu-yielded-by-set2kernelwithdirectory2bin4gpu4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '32Z34-'         
                         
    message2print = "{(Step 33A/70) try" + "; code2py4gpu-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (33B) CODE4PY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        gpu2kernel4xi['CODE4PY']  = task4kernelset2gpu['CODE4PY']  
 
        message2print = "{(Step 33B/70) set" + "; code2py4gpu-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['CODE4PY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {folder2lib4gpu + 'gpu2image4xi5test.py': 
        #  {1: '... profile = "11" />\n', 
        #   2: '... kernel = "gpu2image4xi5test.py">\n'}
 
        code2ack  = code2ack + '33B-'
    except: 
        message2print = "{(Step 33Z1/70) yielded" + "; code2py4gpu-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2py4gpu2nack  = code2py4gpu2nack + '33Z1-'        
                         
    message2print = "{(Step 33C/70) try" + "; code2c4gpu-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (33C) CODE4C - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        gpu2kernel4xi['CODE4CPP']  = task4kernelset2gpu['CODE4CPP']  
 
        message2print = "{(Step 33D/70) set" + "; code2c4gpu-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['CODE4CPP'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {folder2lib4gpu + 
 
        code2c4gpu2ack  = code2c4gpu2ack + '33D-'
    except: 
        message2print = "{(Step 33Z2/70) yielded" + "; code2c4gpu-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2c4gpu2nack  = code2c4gpu2nack + '33Z2-'                     
                         
    message2print = "{(Step 33D/70) try" + "; code2pyc4gpu-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (33D) CODE4PYC - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        gpu2kernel4xi['CODE4PYC']  = task4kernelset2gpu['CODE4PYC']  
 
        message2print = "{(Step 33E/70) set" + "; code2pyc4gpu-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['CODE4PYC'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {folder2lib4gpu + 
 
        code2pyc4gpu2ack  = code2pyc4gpu2ack + '33E-'
    except: 
        message2print = "{(Step 33Z3/70) yielded" + "; code2pyc4gpu-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2pyc4gpu2nack  = code2pyc4gpu2nack + '33Z3-'        
                         
    message2print = "{(Step 33F/70) try" + "; code2bin4gpu-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (33D) CODE4BIN - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        gpu2kernel4xi['CODE4BIN']  = task4kernelset2gpu['CODE4BIN']  
 
        message2print = "{(Step 33G/70) set" + "; code2bin4gpu-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['CODE4BIN'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {folder2lib4gpu + 
 
        code2bin4gpu2ack  = code2bin4gpu2ack + '33G-'
    except: 
        message2print = "{(Step 33Z4/70) yielded" + "; code2bin4gpu-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2bin4gpu2nack  = code2bin4gpu2nack + '33Z4-'                
                
    message2print = "{(Step 34A/70) try" + "; size2py4gpu-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34B) SIZE4PY - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
                        
        gpu2kernel4xi['SIZE4PY']  = task4kernelset2gpu['SIZE4PY']  
 
        message2print = "{(Step 34B/70) set" + "; size2py4gpu-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['SIZE4PY'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '34B-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; size2py4gpu-yielded-by-set2kernel4gpu: " + str(task4kernelset2gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '34Z1-'    
                
    message2print = "{(Step 34C/70) try" + "; size2c4gpu-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34C) SIZE4C - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
                        
        gpu2kernel4xi['SIZE4C']  = task4kernelset2gpu['SIZE4C']  
 
        message2print = "{(Step 34D/70) set" + "; size2c4gpu-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['SIZE4C'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '34D-'
    except: 
        message2print = "{(Step 34Z2/70) yielded" + "; size2c4gpu-yielded-by-set2kernel4gpu: " + str(task4kernelset2gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '34Z2-'
                
    message2print = "{(Step 34E/70) try" + "; size2pyc4gpu-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34E) SIZE4PYC - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
                        
        gpu2kernel4xi['SIZE4PYC']  = task4kernelset2gpu['SIZE4PYC']  
 
        message2print = "{(Step 34F/70) set" + "; size2pyc4gpu-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['SIZE4PYC'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '34F-'
    except: 
        message2print = "{(Step 34Z3/70) yielded" + "; size2pyc4gpu-yielded-by-set2kernel4gpu: " + str(task4kernelset2gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '34Z3-'          
                
    message2print = "{(Step 34G/70) try" + "; size2bin4gpu-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34G) SIZE4BIN - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
                        
        gpu2kernel4xi['SIZE4BIN']  = task4kernelset2gpu['SIZE4BIN']  
 
        message2print = "{(Step 34H/70) set" + "; size2bin4gpu-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['SIZE4BIN'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z4/70) yielded" + "; size2bin4gpu-yielded-by-set2kernel4gpu: " + str(task4kernelset2gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '34Z4-'  
        
    
                                  
    message2print = "{(Step 35A/70) try" + "; language-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) LANGUAGE (DICTIONARY) - REQUIRED                     
        # ---------------------------------------------------------------------------------------------             
                        
        gpu2kernel4xi['LANGUAGE']  = task4kernelset2gpu['LANGUAGE']  
 
        message2print = "{(Step 35B/70) set" + "; language-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['LANGUAGE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '35B-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; language-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '35Z1-'          
                        
    message2print = "{(Step 36A/70) try" + "; count2reference-try-by-set2kernel4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) COUNT2REFERENCE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        gpu2kernel4xi['COUNT2REFERENCE']  = task4kernelset2gpu['COUNT2REFERENCE']  
 
        message2print = "{(Step 36B/70) set" + "; count2reference-set-by-set2kernel4gpu to: " + str(gpu2kernel4xi['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; count2reference-yielded-by-set2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '36Z1-'                           

    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    gpu2kernel4xi ['ACK2RETURN']  = code2ack
    gpu2kernel4xi ['NACK2RETURN'] = code2nack 
    
     
    keys2kernel4gpu = gpu2kernel4xi.keys()
    for key2kernel4gpu in keys2kernel4gpu:
        message2print = "{(Step 70/70) returning" + "; key2kernel4gpu-returning-by-set2kernel4gpu: " + str(key2kernel4gpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 

    return gpu2kernel4xi

           
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2kernelarg4gpu" usage = "
#
#   ... prepare task4set2kernelargws ...    
#
#   response4set2kernelarg = {} 
#   print('(Step 17F/700) OF GPU2KERNEL4XI5TEST - TRYING TO SET KERNEL ARGUMENTS : ', set2kernelarg4gpu)   
#   # <function set2kernelarg4gpu at 0x0000000003837948>
#   try:                               
#       response4set2kernel = set2kernelarg4gpu(task4set2kernelargws)
#       print('(Step 17G/700) OF GPU2KERNEL4XI5TEST - SET KERNEL ARGUMENTS: ', set2kernelarg4gpu)                                                                                                           
#   except:
#       print('(Step 17Z/700) OF GPU2KERNEL4XI5TEST *** FAILED TO SET KERNEL ARGUMENTS: ', set2kernelarg4gpu)  
#
#">

# -------------------------------------------------------------------------------------------------   
def set2kernelarg4gpu(task4kernelargset2gpu=None):

    '''
    sets kernel arguments for GPU
    '''
    global gpu2kernel4xi  
    global folder2lib4gpu  
        
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set2kernelarg4gpu"
    method2print = "set2kernelarg4gpu" 

    message2print = "{(Step 1A/33) started" + "; set2kernelarg4gpu-started: " + str(set2kernelarg4gpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set2kernelarg4gpu at 0x0000000003837948> 
    
    keys2task = task4kernelargset2gpu.keys()
    for key2task in sorted(keys2task):
        message2print = "{(Step 1B/33) passed" + "; key2task-passed-to-set2kernelarg4gpu: " + str(key2task) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
        # 
          
    code2ack  = ' '
    code2nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if gpu2kernel4xi is None:
            gpu2kernel4xi = create2kernel4gpu()                                                                              # INITIALIZE PROGRAM FOR GPU 
            message2print = "{(Step 1C/70) set" + "; gpu2kernel4xi-set-by-set2kernelarg4gpu to: " + str(gpu2kernel4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2kernel4gpu-skipped-by-set2kernelarg4gpu: " + str(create2kernel4gpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                 
    except:
        try:
            gpu2kernel4xi = create2kernel4gpu()
            message2print = "{(Step 1E/70) set" + "; gpu2kernel4xi-set-by-set2kernelarg4gpu to: " + str(gpu2kernel4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            gpu2kernel4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; gpu2kernel4xi-yielded-by-set2kernelarg4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET       
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data2callback-try-by-set2kernelarg4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        gpu2kernel4xi['DATA2CALLBACK'] = task4kernelargset2gpu['DATA2CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data2callback-set-by-set2kernelarg4gpu to: " + str(gpu2kernel4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-gpu2kernel4xi-yielded-by-set2kernelarg4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-set2kernelarg4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        gpu2kernel4xi['FUNCTION2CALLBACK'] = task4kernelargset2gpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function2callback-set-by-set2kernelarg4gpu to: " + str(gpu2kernel4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-gpu2kernel4xi-yielded-by-set2kernelarg4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set2kernelarg4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        gpu2kernel4xi['PLATFORM'] = task4kernelargset2gpu['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set2kernelarg4gpu to: " + str(gpu2kernel4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #   
        code2ack  = code2ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-gpu2kernel4xi-yielded-by-set2kernelarg4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2gpu-try-by-set2kernelarg4gpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        gpu2kernel4xi['CONTEXT'] = task4kernelargset2gpu['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set2kernelarg4gpu to: " + str(gpu2kernel4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-gpu2kernel4xi-yielded-by-set2kernelarg4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set2kernelarg4gpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        gpu2kernel4xi['DEVICE'] = task4kernelargset2gpu['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set2kernelarg4gpu to: " + str(gpu2kernel4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-gpu2kernel4xi-yielded-by-set2kernelarg4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-set2kernelarg4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        gpu2kernel4xi['CODE2ERROR'] = task4kernelargset2gpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code2error-set-by-set2kernelarg4gpu to: " + str(gpu2kernel4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-gpu2kernel4xi-yielded-by-set2kernelarg4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set2kernelarg4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        gpu2kernel4xi['QUEUE'] = task4kernelargset2gpu['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set2kernelarg4gpu to: " + str(gpu2kernel4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
         
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-gpu2kernel4xi-yielded-by-set2kernelarg4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set2kernelarg4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        gpu2kernel4xi['PIPE'] = task4kernelargset2gpu['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set2kernelarg4gpu to: " + str(gpu2kernel4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-gpu2kernel4xi-yielded-by-set2kernelarg4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set2kernelarg4gpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        gpu2kernel4xi['COMMANDQUEUE']  = task4kernelargset2gpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set2kernelarg4gpu to: " + str(gpu2kernel4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set2kernelarg4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set2kernelarg4gpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
        gpu2kernel4xi['BUFFER']  = task4kernelargset2gpu['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set2kernelarg4gpu to: " + str(gpu2kernel4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set2kernelarg4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set2kernelarg4gpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        gpu2kernel4xi['EVENT']  = task4kernelargset2gpu['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set2kernelarg4gpu to: " + str(gpu2kernel4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set2kernelarg4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '30Z1-'            
                
    message2print = "{(Step 31A/70) try" + "; count4source-try-by-set2kernelarg4gpu: None" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31)  - COUNT4SOURCE                           
        # ---------------------------------------------------------------------------------------------             
        gpu2kernel4xi['COUNT4SOURCE']  = task4kernelargset2gpu['COUNT4SOURCE']  
 
        message2print = "{(Step 31B/70) set" + "; count4source-set-by-set2kernelarg4gpu to: " + str(gpu2kernel4xi['COUNT4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 1
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; count4source-yielded-by-set2kernelarg4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '31Z1-'  

    message2print = "{(Step 32A/70) try" + "; directory-try-by-set2kernelarg4gpu: " + str(task4kernelargset2gpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print) 
    # {'CODE2ERROR': 0 ... 'DIRECTORY4SOURCE': '...//a' ... 

    try:            
        # ---------------------------------------------------------------------------------------------
        # (32) DIRECTORY - REQUIRED                     
        # ---------------------------------------------------------------------------------------------                                    
        gpu2kernel4xi['DIRECTORY4SOURCE']  = task4kernelargset2gpu['DIRECTORY4SOURCE'] 
        
        message2print = "{(Step 32B/70) set" + "; directory-set-by-set2kernelargwithdirectory4gpu to: " + str(gpu2kernel4xi['DIRECTORY4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # OK
        code2ack  = code2ack + '32B-'
    except: 
        message2print = "{(Step 32Z1/70) yielded" + "; directory-yielded-by-set2kernelargwithdirectory4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '32Z1-' 
        
    message2print = "{(Step 33A/70) try" + "; code-try-by-set2kernelarg4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (33) CODE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        gpu2kernel4xi['CODE4SOURCE']  = task4kernelargset2gpu['CODE4SOURCE']  
 
        message2print = "{(Step 33B/70) set" + "; code-set-by-set2kernelarg4gpu to: " + str(gpu2kernel4xi['CODE4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {folder2lib4gpu + 'gpu2image4xi5test.py': 
        #  {1: '... profile = "11" />\n', 
        #   2: '... kernelarg = "gpu2image4xi5test.py">\n'}
 
        code2ack  = code2ack + '33B-'
    except: 
        message2print = "{(Step 33Z1/70) yielded" + "; code-yielded-by-set2kernelarg4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '33Z1-'             
        
    message2print = "{(Step 34A/70) try" + "; size-try-by-set2kernelarg4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) SIZE - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
                        
        gpu2kernel4xi['SIZE4SOURCE']  = task4kernelargset2gpu['SIZE4SOURCE']  
 
        message2print = "{(Step 34B/70) set" + "; size-set-by-set2kernelarg4gpu to: " + str(gpu2kernel4xi['SIZE4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '34B-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; size-yielded-by-set2kernelarg4gpu: " + str(task4kernelargset2gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '34Z1-'    
                
    message2print = "{(Step 35A/70) try" + "; language-try-by-set2kernelarg4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) LANGUAGE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        gpu2kernel4xi['LANGUAGE4SOURCE']  = task4kernelargset2gpu['LANGUAGE4SOURCE']  
 
        message2print = "{(Step 35B/70) set" + "; language-set-by-set2kernelarg4gpu to: " + str(gpu2kernel4xi['LANGUAGE4SOURCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '35B-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; language-yielded-by-set2kernelarg4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '35Z1-'          
                        
    message2print = "{(Step 36A/70) try" + "; count2reference-try-by-set2kernelarg4gpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) COUNT2REFERENCE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        gpu2kernel4xi['COUNT2REFERENCE']  = task4kernelargset2gpu['COUNT2REFERENCE']  
 
        message2print = "{(Step 36B/70) set" + "; count2reference-set-by-set2kernelarg4gpu to: " + str(gpu2kernel4xi['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; count2reference-yielded-by-set2kernelarg4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '36Z1-'                           

    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    gpu2kernel4xi ['ACK2RETURN']  = code2ack
    gpu2kernel4xi ['NACK2RETURN'] = code2nack 
    
     
    keys2kernelarg4gpu = gpu2kernel4xi.keys()
    for key2kernelarg4gpu in keys2kernelarg4gpu:
        message2print = "{(Step 70/70) returning" + "; key2kernelarg4gpu-returning-by-set2kernelarg4gpu: " + str(key2kernelarg4gpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 

    return gpu2kernel4xi

           
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "release2kernel4gpu" usage = "
#
#   task4release2kernel  = {} 
#   task4release2kernel  ['COUNT2REFERENCE'] = 1 
#   response4release2kernel = {}
#   
#   print('(Step 18A/700) OF GPU2KERNEL4XI5TEST - TRYING TO RELEASE KERNEL OF GPU DEVICE ', time4xi())    
#   try:                               
#       response4release2kernel = release2kernel4gpu(task4release2kernel)
#       print('(Step 18B/700) OF GPU2KERNEL4XI5TEST - RELEASED KERNEL OF GPU DEVICE ', time4xi()) 
#   except:
#       print('(Step 18Z1/700) OF GPU2KERNEL4XI5TEST *** FAILED TO RELEASE KERNEL OF GPU DEVICE ', time4xi())                                                                                                  
#   
#">

# -------------------------------------------------------------------------------------------------   
def release2kernel4gpu(task4release2gpu=None):

    '''
    releases kernel for GPU
    '''
    global gpu2kernel4xi  
    global folder2lib4gpu  
        
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "release2kernel4gpu"
    method2print = "release2kernel4gpu" 

    message2print = "{(Step 1A/33) started" + "; release2kernel4gpu-started: " + str(release2kernel4gpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    
    message2print = "{(Step 1B/33) passed" + "; task4release2gpu-to-release2kernel4gpu: " + str(task4release2gpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    

    code2ack  = ' '
    code2nack = ' '

    try:
        if gpu2kernel4xi is None:
            gpu2kernel4xi = create2kernel4gpu()                                                                       # INITIALIZE KERNEL                 
    except:
        try:
            gpu2kernel4xi = create2kernel4gpu()
        except:
            gpu2kernel4xi = {}
            message2print = "{(Step 2Z0/70) yielded" + "; gpu2kernel4xi-yielded-by-release2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '2Z0-' 
                
    message2print = "{(Step 1A/70) try" + "; release-try-by-release2kernel4gpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    if task4release2gpu is None:
        task4release2gpu = {}
        task4release2gpu['COUNT2REFERENCE'] = 1
           
    try:
        # -----------------------------------------------------------------------------------------
        # RELEASE
        # -----------------------------------------------------------------------------------------               
        gpu2kernel4xi['COUNT2REFERENCE'] = gpu2kernel4xi['COUNT2REFERENCE'] - task4release2gpu['COUNT2REFERENCE']
        
        message2print = "{(Step 1A/70) calculated" + "; count2reference-calculated-by-release2kernel4gpu: " + str(gpu2kernel4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
         
        if gpu2kernel4xi['COUNT2REFERENCE'] < 0:
            gpu2kernel4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 1B/70) warning" + "; count2reference-warning-by-release2kernel4gpu: " + " ** " + str(gpu2kernel4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            # 
             
            code2nack  = code2nack + '1B-'
        else:
            code2ack  = code2ack + '1B-'
    except:
        try:
            gpu2kernel4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 8Z1/70) warning" + "; count2reference-warning-by-release2kernel4gpu: " + " ** " + str(gpu2kernel4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            code2nack  = code2nack + '8Z1-'
            
        except:
            message2print = "{(Step 8Z2/70) yielded" + "; gpu2kernel4xi-yielded-by-release2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '8Z2-'
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    gpu2kernel4xi ['ACK2RETURN']  = code2ack
    gpu2kernel4xi ['NACK2RETURN'] = code2nack  
    
    keys2program4gpu = gpu2kernel4xi.keys()
    for key2program4gpu in keys2program4gpu:
        message2print = "{(Step 70/70) returning" + "; key2program4gpu-returning-by-release2kernel4gpu: " + str(key2program4gpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 
    
    return gpu2kernel4xi

#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "retain2kernel4gpu" usage = "
#
#   task4retain2kernel  = {} 
#   task4retain2kernel  ['COUNT2REFERENCE'] = 1 
#   response4retain2kernel = {}
#   
#   print('(Step 19A/700) OF GPU2KERNEL4XI5TEST - TRYING TO RETAIN PROGRAM OF GPU DEVICE ', time4xi())    
#   # 
#   try:                               
#       response4retain2kernel = retain2kernel4gpu(task4retain2kernel)
#       print('(Step 19B/700) OF GPU2KERNEL4XI5TEST - RETAINED PROGRAM OF GPU DEVICE ', time4xi()) 
#   except:
#       print('(Step 19Z1/700) OF GPU2KERNEL4XI5TEST *** FAILED TO RETAIN PROGRAM OF GPU DEVICE ', time4xi())                                                                                                  
#
#">

# -------------------------------------------------------------------------------------------------   
def retain2kernel4gpu(task4retain2gpu=None):

    '''
    retains kernel of GPU
    '''
    global gpu2kernel4xi 
    global folder2lib4gpu  
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "retain2kernel4gpu"
    method2print = "retain2kernel4gpu"

    message2print = "{(Step 1A/33) started" + "; retain2kernel4gpu-started: " + str(retain2kernel4gpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    
    message2print = "{(Step 1B/33) passed" + "; task4retain2gpu-to-retain2kernel4gpu: " + str(task4retain2gpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # (Step 2/70) - PROVIDE TASKS, IF NECESSARY AND CHECK GIVEN TASKS 
    # -----------------------------------------------------------------------------------------    
    if task4retain2gpu is None:
        task4retain2gpu = {}
        task4retain2gpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 2A/33) set" + "; task4retain2gpu-set-by-retain2kernel4gpu to: " + str(task4retain2gpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    else:
        pass

    
    # -----------------------------------------------------------------------------------------
    # (Step 3/70) - CHECK COUNT2REFERENCE
    # -----------------------------------------------------------------------------------------         
    if 'COUNT2REFERENCE' in task4retain2gpu.keys():
        pass
    else:
        task4retain2gpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3A/33) set" + "; count2reference-set-by-retain2kernel4gpu to: " + str(task4retain2gpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                                   
    if task4retain2gpu['COUNT2REFERENCE'] < 1:
        task4retain2gpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3B/33) set" + "; count2reference-set-by-retain2kernel4gpu to: " + str(task4retain2gpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
        
    # -----------------------------------------------------------------------------------------
    # (Step 4/70) - INITIALIZE PROGRAM IF NECESSARY
    # -----------------------------------------------------------------------------------------
    try:
        if gpu2kernel4xi is None:
            gpu2kernel4xi = create2kernel4gpu()
            message2print = "{(Step 4A/70) created" + "; gpu2kernel4xi-created-by-retain2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                                                                          
    except:
        try:
            gpu2kernel4xi = create2kernel4gpu()
            message2print = "{(Step 4B/70) created" + "; gpu2kernel4xi-created-by-retain2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
        except:
            gpu2kernel4xi = {}
            message2print = "{(Step 4Z/70) yielded" + "; gpu2kernel4xi-yielded-by-retain2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '4Z-' 


    # -----------------------------------------------------------------------------------------
    # (Step 5/70) - RETAIN
    # -----------------------------------------------------------------------------------------             
    try:
        gpu2kernel4xi['COUNT2REFERENCE'] = gpu2kernel4xi['COUNT2REFERENCE'] + task4retain2gpu['COUNT2REFERENCE']
        message2print = "{(Step 6A/70) calculated" + "; count2reference-calculated-by-retain2kernel4gpu: " + str(gpu2kernel4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 1            
        if gpu2kernel4xi['COUNT2REFERENCE'] <= 0:
            gpu2kernel4xi['COUNT2REFERENCE'] = 1
            message2print = "{(Step 6Z1/70) warning" + "; count2reference-warning-by-retain2kernel4gpu: " + " ** " + str(gpu2kernel4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            code2nack  = code2nack + '6Z1-'
        else:
            code2ack  = code2ack + '6C-'

    except:
        try:
            gpu2kernel4xi['COUNT2REFERENCE'] = 1
        
            message2print = "{(Step 7Z1/70) warning" + "; count2reference-warning-by-retain2kernel4gpu: " + " ** " + str(gpu2kernel4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            code2nack  = code2nack + '7Z1-'
        
        except:
            message2print = "{(Step 7Z2/70) yielded" + "; gpu2kernel4xi-yielded-by-retain2kernel4gpu: " + str(gpu2kernel4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '7Z2-'
        
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70) - RETURN           
    # -----------------------------------------------------------------------------------------
    
    gpu2kernel4xi ['ACK2RETURN']  = code2ack
    gpu2kernel4xi ['NACK2RETURN'] = code2nack     
     
    keys2program4gpu = gpu2kernel4xi.keys()
    for key2program4gpu in keys2program4gpu:
        message2print = "{(Step 70/70) returning" + "; key2program4gpu-returning-by-retain2kernel4gpu: " + str(key2program4gpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 
    
    return gpu2kernel4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2kernelinfo4gpu" usage = "
#
#   print('(Step 3B1/700) OF GPU2KERNEL4XI5TEST - TRYING TO GET KERNEL INFORMATION FOR GPU ', time4xi())     
#   try:
#       response4kernelinfo2gpu = get2kernelinfo4gpu() 
#       print('(Step 3B2/700) OF GPU2KERNEL4XI5TEST - GOT KERNEL INFORMATION FOR GPU ', time4xi())
#   except:
#       print('(Step 3Z3/700) OF GPU2KERNEL4XI5TEST *** FAILED TO GET KERNEL INFORMATION FOR GPU', time4xi())                  
#   
#">

# -------------------------------------------------------------------------------------------------   
def get2kernelinfo4gpu(task4getinfo2kernel=None):

    '''
    gets kernel information of GPU
    '''
    
    global gpu2kernel4xi
    global folder2lib4gpu  
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2kernelinfo4gpu"
    method2print = "get2kernelinfo4gpu"

    message2print = "{(Step 0A/33) started" + "; get2kernelinfo4gpu-started: " + str(get2kernelinfo4gpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
                                                                
    
    message2print = "{(Step 0B/33) passed" + "; task4getinfo2kernel-passed-to-get2kernelinfo4gpu: " + str(task4getinfo2kernel) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # 
    
    
    gpu2kernel = {}
    try: 
        if gpu2kernel4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE SOURCE DATA FOR PROGRAM FOR GPU           
            # -------------------------------------------------------------------------------------           
            gpu2kernel = create2kernel4gpu()
            message2print = "{(Step 1B2/33) initialized" + "; gpu2kernel-initialized-by-get2kernelinfo4gpu to: " + str(gpu2kernel)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        else:
            message2print = "{(Step 1B3/33) using" + "; gpu2kernel4xi-using-by-get2kernelinfo4gpu: " + str(gpu2kernel4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF PROGRAM INFO OF GPU          
        # -------------------------------------------------------------------------------------     
        if task4getinfo2kernel is None:
            task4getinfo2kernel = {}
            gpu2kernel = gpu2kernel4xi
            message2print = "{(Step 1C/33) using" + "; gpu2kernel-got-by-get2kernelinfo4gpu: " + str(gpu2kernel)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        else:
            pass    

        keys2task4kernelinfo = task4getinfo2kernel.keys()
        keys2task = task4getinfo2kernel.keys()
            
        # -------------------------------------------------------------------------------------
        # SOURCE OR BINARY PROGRAM INFO OF GPU          
        # -------------------------------------------------------------------------------------         
        for key2task in keys2task:
            try:
                message2print = "{(Step 2D2/33) using" + "; key2task-using-by-get2kernel4gpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                 
                gpu2kernel [key2task] = gpu2kernel4xi[key2task]
                
                message2print = "{(Step 2D3/33) set" + "; info-of-gpu2kernel-set-by-get2kernelinfo4gpu to: " + str(gpu2kernel [key2task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key2task-yielded-by-get2kernel4gpu: " + str(key2task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # 
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo2kernel-using-by-get2kernel4gpu: " + str(task4getinfo2kernel)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; gpu2kernel4xi-using-by-get2kernel4gpu: " + str(gpu2kernel4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 

    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE PROGRAM WITH SOURCE DATA           
            # -------------------------------------------------------------------------------------           
            gpu2kernel = create2kernel4gpu()
            
            message2print = "{(Step 1F/33) initialized" + "; gpu2kernel-initialized-by-get2kernelinfo4gpu to: " + str(gpu2kernel)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            if task4getinfo2kernel is None:
                gpu2kernel = gpu2kernel4xi
                message2print = "{(Step 1G/33) set" + "; gpu2kernel-set-by-get2kernelinfo4gpu to: " + str(gpu2kernel)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
            else:
                keys2task = task4getinfo2kernel.keys()
                for key2task in keys2task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key2task-using-by-get2kernel4gpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        gpu2kernel [key2task] = gpu2kernel4xi[key2task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-gpu2kernel -set-by-get2kernelinfo4gpu to: " + str(gpu2kernel  [key2task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key2task-yielded-by-get2kernel4gpu: " + str(key2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                                                                            
        except:
            gpu2kernel = {}
            message2print = "{(Step 1Z3/70) yielded" + "; gpu2kernel-yielded-by-get2kernel4gpu: " + str(gpu2kernel) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
     
    keys2kernel4gpu = gpu2kernel4xi.keys()
    for key2kernel4gpu in keys2kernel4gpu:
        message2print = "{(Step 70/70) returning" + "; key2kernel4gpu-returning-by-get2kernelinfo4gpu: " + str(key2kernel4gpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # CODE2ERROR ...
        
    return gpu2kernel
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2kernelarginfo4gpu" usage = "
#
#   print('(Step 3B1/700) OF GPU2KERNEL4XI5TEST - TRYING TO GET KERNEL ARGUMENTGS INFO FOR GPU ', time4xi())     
#   try:
#       response4kernelinfo2gpu = get2kernelarginfo4gpu(task4getinfo2kernel) 
#       print('(Step 3B2/700) OF GPU2KERNEL4XI5TEST - GOT KERNEL ARGUMENTGS INFO FOR GPU ', time4xi())
#   except:
#       print('(Step 3Z3/700) OF GPU2KERNEL4XI5TEST *** FAILED TO GET KERNEL ARGUMENTGS INFO FOR GPU ', time4xi())                  
#   
#">

# -------------------------------------------------------------------------------------------------   
def get2kernelarginfo4gpu(task4getinfo2kernel=None):

    '''
    gets kernel argument information of GPU
    '''
    
    global gpu2kernel4xi
    global folder2lib4gpu  
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2kernelarginfo4gpu"
    method2print = "get2kernelarginfo4gpu"

    message2print = "{(Step 0A/33) started" + "; get2kernelarginfo4gpu-started: " + str(get2kernelarginfo4gpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
                                                                
    
    message2print = "{(Step 0B/33) passed" + "; task4getinfo2kernel-passed-to-get2kernelarginfo4gpu: " + str(task4getinfo2kernel) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # 
    
    
    gpu2kernel = {}
    try: 
        if gpu2kernel4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE SOURCE DATA FOR PROGRAM FOR GPU           
            # -------------------------------------------------------------------------------------           
            gpu2kernel = create2kernel4gpu()
            message2print = "{(Step 1B2/33) initialized" + "; gpu2kernel-initialized-by-get2kernelarginfo4gpu to: " + str(gpu2kernel)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        else:
            message2print = "{(Step 1B3/33) using" + "; gpu2kernel4xi-using-by-get2kernelarginfo4gpu: " + str(gpu2kernel4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF PROGRAM INFO OF GPU          
        # -------------------------------------------------------------------------------------     
        if task4getinfo2kernel is None:
            task4getinfo2kernel = {}
            gpu2kernel = gpu2kernel4xi
            message2print = "{(Step 1C/33) using" + "; gpu2kernel-got-by-get2kernelarginfo4gpu: " + str(gpu2kernel)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        else:
            pass    

        keys2task4kernelinfo = task4getinfo2kernel.keys()
        keys2task = task4getinfo2kernel.keys()
            
        # -------------------------------------------------------------------------------------
        # SOURCE OR BINARY PROGRAM INFO OF GPU          
        # -------------------------------------------------------------------------------------         
        for key2task in keys2task:
            try:
                message2print = "{(Step 2D2/33) using" + "; key2task-using-by-get2kernel4gpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                 
                gpu2kernel [key2task] = gpu2kernel4xi[key2task]
                
                message2print = "{(Step 2D3/33) set" + "; info-of-gpu2kernel-set-by-get2kernelarginfo4gpu to: " + str(gpu2kernel [key2task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key2task-yielded-by-get2kernel4gpu: " + str(key2task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # 
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo2kernel-using-by-get2kernel4gpu: " + str(task4getinfo2kernel)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; gpu2kernel4xi-using-by-get2kernel4gpu: " + str(gpu2kernel4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 

    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE PROGRAM WITH SOURCE DATA           
            # -------------------------------------------------------------------------------------           
            gpu2kernel = create2kernel4gpu()
            
            message2print = "{(Step 1F/33) initialized" + "; gpu2kernel-initialized-by-get2kernelarginfo4gpu to: " + str(gpu2kernel)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            if task4getinfo2kernel is None:
                gpu2kernel = gpu2kernel4xi
                message2print = "{(Step 1G/33) set" + "; gpu2kernel-set-by-get2kernelarginfo4gpu to: " + str(gpu2kernel)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
            else:
                keys2task = task4getinfo2kernel.keys()
                for key2task in keys2task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key2task-using-by-get2kernel4gpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        gpu2kernel [key2task] = gpu2kernel4xi[key2task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-gpu2kernel -set-by-get2kernelarginfo4gpu to: " + str(gpu2kernel  [key2task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key2task-yielded-by-get2kernel4gpu: " + str(key2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                                                                            
        except:
            gpu2kernel = {}
            message2print = "{(Step 1Z3/70) yielded" + "; gpu2kernel-yielded-by-get2kernel4gpu: " + str(gpu2kernel) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
     
    keys2kernel4gpu = gpu2kernel4xi.keys()
    for key2kernel4gpu in keys2kernel4gpu:
        message2print = "{(Step 70/70) returning" + "; key2kernel4gpu-returning-by-get2kernelarginfo4gpu: " + str(key2kernel4gpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)
        # 
        
    return gpu2kernel
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2buildinfo4gpu" usage = "
#
#   response4get2buildinfo = get2buildinfo4gpu()
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2buildinfo4gpu(task4getinfo2build=None):

    '''
    gets gpu2build4xi and gpu2build4xi
    '''
    
    global gpu2build4xi
    global folder2lib4gpu  
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2buildinfo4gpu"
    method2print = "get2buildinfo4gpu"

    message2print = "{(Step 0A/33) started" + "; get2buildinfo4gpu-started: " + str(get2buildinfo4gpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
                                                                
    
    message2print = "{(Step 0B/33) passed" + "; task4getinfo2build-passed-to-get2buildinfo4gpu: " + str(task4getinfo2build) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # 
    
    
    gpu2build = {}
    try: 
        if gpu2build4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE SOURCE DATA FOR PROGRAM FOR GPU           
            # -------------------------------------------------------------------------------------           
            gpu2build = {}
            message2print = "{(Step 1B2/33) initialized" + "; gpu2build-initialized-by-get2buildinfo4gpu to: " + str(gpu2build)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        else:
            message2print = "{(Step 1B3/33) using" + "; gpu2build4xi-using-by-get2buildinfo4gpu: " + str(gpu2build4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF PROGRAM INFO OF GPU          
        # -------------------------------------------------------------------------------------     
        if task4getinfo2build is None:
            task4getinfo2build = {}
            gpu2build = gpu2build4xi
            message2print = "{(Step 1C/33) using" + "; gpu2build-got-by-get2buildinfo4gpu: " + str(gpu2build)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        else:
            pass    

        keys2task4buildinfo = task4getinfo2build.keys()
        keys2task = task4getinfo2build.keys()
            
        # -------------------------------------------------------------------------------------
        # BUILD INFO OF GPU          
        # -------------------------------------------------------------------------------------         
        for key2task in keys2task:
            try:
                message2print = "{(Step 2D2/33) using" + "; key2task-using-by-get2build4gpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                 
                gpu2build [key2task] = gpu2build4xi[key2task]
                
                message2print = "{(Step 2D3/33) set" + "; info-of-gpu2build-set-by-get2buildinfo4gpu to: " + str(gpu2build [key2task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key2task-yielded-by-get2build4gpu: " + str(key2task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # 
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo2build-using-by-get2build4gpu: " + str(task4getinfo2build)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; gpu2build4xi-using-by-get2build4gpu: " + str(gpu2build4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 

    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE BUILD           
            # -------------------------------------------------------------------------------------           
            gpu2build = {}
            message2print = "{(Step 1F/33) initialized" + "; gpu2build-initialized-by-get2buildinfo4gpu to: " + str(gpu2build)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            if task4getinfo2build is None:
                gpu2build = gpu2build4xi
                message2print = "{(Step 1G/33) set" + "; gpu2build-set-by-get2buildinfo4gpu to: " + str(gpu2build)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
            else:
                keys2task = task4getinfo2build.keys()
                for key2task in keys2task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key2task-using-by-get2build4gpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        gpu2build [key2task] = gpu2build4xi[key2task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-gpu2build -set-by-get2buildinfo4gpu to: " + str(gpu2build  [key2task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key2task-yielded-by-get2build4gpu: " + str(key2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                                                                            
        except:
            gpu2build = {}
            message2print = "{(Step 1Z3/70) yielded" + "; gpu2build-yielded-by-get2build4gpu: " + str(gpu2build) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; gpu2build-returning-by-get2buildinfo4gpu: " + str(gpu2build) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)
    # 
    
    return gpu2build
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2kernelworkgroupinfo4gpu" usage = "
#     
#   print('(Step 2AB2/700) OF GPU2KERNEL4XI5TEST - TRYING TO GET KERNEL WORK GROUP INFORMATION ', time4xi())           
#   try:
#       response4link = get2kernelworkgroupinfo4gpu()
#       print('(Step 2BB2/700) OF GPU2KERNEL4XI5TEST - GOT KERNEL WORK GROUP INFORMATION ', time4xi()) 
#   except:
#       print('(Step 2Z2/700) OF GPU2KERNEL4XI5TEST *** FAILED TO KERNEL WORK GROUP INFORMATION ', time4xi())                                                                     
#          
#">

# -------------------------------------------------------------------------------------------------   
def get2kernelworkgroupinfo4gpu(task4kernelworkgroupinfo2gpu=None):

    '''
    gets kernel workgroup info of GPU
    '''
    global gpu2kernel4xi                                                                                            # Kernel of GPU    
    global import2use4gpu                                                                                           # Imports of GPU
    global target2function4gpu                                                                                      # Target function of Process4Xi
    global folder2lib4gpu  
            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2kernelworkgroupinfo4gpu" 
    method2print = "get2kernelworkgroupinfo4gpu"

    message2print = "{(Step 1A/70) started" + "; get2kernelworkgroupinfo4gpu-started: " + str(get2kernelworkgroupinfo4gpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # 
  
             
    message2print = "{(Step 1B/70) passed" + "; task4kernelworkgroupinfo2gpu-to-get2kernelworkgroupinfo4gpu: " + str(task4kernelworkgroupinfo2gpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 

    code2ack  = ' '
    code2nack = ' '    
    
    process2kernelworkgroupinfo4gpu = {}                                                                                       # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4kernelworkgroupinfo2gpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER
            # -----------------------------------------------------------------------------------------    
            pass                                                                       
        else:
            # -----------------------------------------------------------------------------------------
            # PARAMETERS GIVEN ARE OMITTED 
            # -----------------------------------------------------------------------------------------         
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4kernelworkgroupinfo2gpu-yielded-by-get2kernelworkgroupinfo4gpu: " + str(task4kernelworkgroupinfo2gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
       
        code2nack  = code2nack + '1Z1-' 

    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - LINK SOURCES INSIDE PROGRAM OBJECT FOR GPU ! 
    # ---------------------------------------------------------------------------------------------    
    process2kernelworkgroupinfo4gpu = {}                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; process2kernelworkgroupinfo4gpu-initialized-by-get2kernelworkgroupinfo4gpu to: " + str(process2kernelworkgroupinfo4gpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {}

    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) - SOURCE CODE WITHOUT EOL CHARACTERS                  
        # ---------------------------------------------------------------------------------------------             
        keys2code4gpu = gpu2kernel4xi['CODE4SOURCE'].keys()
        message2print = "{(Step 30A/70) using" + "; keys2code4gpu-using-by-get2kernelworkgroupinfo4gpu: " + str(keys2code4gpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # dict_keys([folder2lib4gpu + 'gpu2sampler4xi5test.py'])
        
        unit4processing = "gpu"
        
        for key2code4gpu in keys2code4gpu:
            index2search=str.find(key2code4gpu,unit4processing) 
            if index2search == -1:
                pass
            else:
                target2link = key2code4gpu[index2search:]                                                          # gpu2sampler4xi5test.py
                index2search=str.find(target2link,".py")
                if index2search == -1:
                    pass
                else:
                    target2link4gpu = target2link[:index2search]                                                  
                    message2print = "{(Step 32A/70) set" + "; target2link4gpu-set-by-get2kernelworkgroupinfo4gpu to: " + str(target2link4gpu)  + "; time: " + time4xi() + "}"  
                    print4xi(class2print, method2print, message2print)
                    # gpu2sampler4xi5test

                    p2code4gpu = None    
                    try:
                        # ---------------------------------------------------------------------------------------------    
                        # IMPORT TARGET FUNCTION FROM ITS MODULE AS TARGET2FUNCTION4GPU 
                        # ---------------------------------------------------------------------------------------------    
      
                        try:
                            import2use4gpu.get(target2link4gpu, handle2error4gpu) ()
                            
                            message2print = "{(Step 32B/70) imported" + "; target2link4gpu-imported-by-get2kernelworkgroupinfo4gpu: " + str(target2link4gpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # gpu2sampler4xi5test
                            code2ack  = code2ack + '32B-'
                        except:
                            message2print = "{(Step 32Z1/70) failed" + "; import2use4gpu-failed-by-get2kernelworkgroupinfo4gpu: " + str(import2use4gpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)                            
                            # 
                            message2print = "{(Step 32Z2/70) used" + "; target2link4gpu-used-by-get2kernelworkgroupinfo4gpu: " + str(target2link4gpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # 
                            code2nack  = code2nack + '32Z2-'
                                    
                        # ---------------------------------------------------------------------------------------------    
                        # CREATE PROCESS WITH TARGET2FUNCTION4GPU
                        # ---------------------------------------------------------------------------------------------                          
                        p2gpu = Process4Xi(target=target2function4gpu, args=())                                
                        message2print = "{(Step 33A/70) created" + "; p2gpu-created-by-get2kernelworkgroupinfo4gpu: " + str(p2gpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # <Process4Xi(Process4Xi-2, initial)>
                        
                        process2kernelworkgroupinfo4gpu[key2code4gpu] = p2gpu
                        
                        code2ack  = code2ack + '33A-'
                        try:
                            # ---------------------------------------------------------------------------------------------    
                            # START PROCESS
                            # --------------------------------------------------------------------------------------------- 
                            p2gpu.start()
                            message2print = "{(Step 34A/70) started" + "; p2gpu-started-by-get2kernelworkgroupinfo4gpu: " + str(p2gpu.name) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # Process4Xi-2
                            
                            try:
                                # ---------------------------------------------------------------------------------------------    
                                # STATUS OF PROCESS
                                # ---------------------------------------------------------------------------------------------                             
                                status2process4gpu = p2gpu.is_alive()
                                message2print = "{(Step 35A/70) got" + "; status-got-by-get2kernelworkgroupinfo4gpu: " + str(status2process4gpu) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                # True
                                
                                try:
                                    # ---------------------------------------------------------------------------------------------    
                                    # RUN PROCESS
                                    # ---------------------------------------------------------------------------------------------                                  
                                    p2gpu.run()
                                    message2print = "{(Step 36A/70) run" + "; p2gpu-run-by-get2kernelworkgroupinfo4gpu: " + str(p2gpu) + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print)
                                    # <Process4Xi(Process4Xi-1, stopped[1])>
                                except:
                                    message2print = "{(Step 36Z/70) failed" + "; run-failed-by-get2kernelworkgroupinfo4gpu: " + str(p2gpu) + "; time: " + time4xi() + "}"
                                    print4xi(class2print, method2print, message2print)  
                                    code2nack  = code2nack + '36Z-'            
                                    # 
                            except:
                                message2print = "{(Step 35Z/70) failed" + "; status-failed-by-get2kernelworkgroupinfo4gpu: " + str(p2gpu) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)  
                                # 
                                code2nack  = code2nack + '35Z-'                                   
                                
                        except:
                            message2print = "{(Step 33Z/70) failed" + "; start-failed-by-get2kernelworkgroupinfo4gpu: " + str(p2gpu) + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)  
                            code2nack  = code2nack + '33Z-'     
                        
                    except:
                        message2print = "{(Step 32Z/70) failed" + "; p2code4gpu-failed-by-get2kernelworkgroupinfo4gpu: " + str(p2code4gpu) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        code2nack  = code2nack + '32Z-' 
                                                                                    

 
        message2print = "{(Step 38A/70) set" + "; process2kernelworkgroupinfo4gpu-set-by-get2kernelworkgroupinfo4gpu to: " + str(process2kernelworkgroupinfo4gpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
         
        code2ack  = code2ack + '38A-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; code-yielded-by-get2kernelworkgroupinfo4gpu: " + str(gpu2kernel4xi['CODE4SOURCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '30Z1-'     

                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    gpu2kernel4xi ['ACK2RETURN']  = code2ack
    gpu2kernel4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; process2kernelworkgroupinfo4gpu-returning-by-get2kernelworkgroupinfo4gpu: " + str(process2kernelworkgroupinfo4gpu) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                              
    # 
    return process2kernelworkgroupinfo4gpu
            
#</function4xi> 
 
                                        
# =================================================================================================
#<class4xi class = "GPU2Kernel4Xi" base = "object" child = "None"
#   synopsis = "offers  methods for GPU kernels" usage = "Kernel4Xi of GPU"
#>

# =================================================================================================
class GPU2Kernel4Xi(object): 
    ''' 
    GPU2Kernel4Xi offers  methods for GPU kernels
    '''            

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    classvariable2gpu = cpu_corecount4xi()
    #"
    #/>
       
       
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = (
            'add2kernel',                      # ACK              

            'execute2kernel',                  # ACK 5.8    p169                                             
            'execute2task',                    # ACK 5.8    p172   
            'execute2nativekernel',            # ACK 5.8    p174     
            
            'create2kernel',                   # ACK 5.7.1  p157   
            'create2kernelsinprogram',         # ACK 5.7.1  p158  
                       
            'delete2kernel',                   # ACK

            'fetch2kernel',                    # ACK 
            
            'get2kernelinfo',                  # ACK 5.7.3  p162  
            'get2kernelworkgroupinfo',         # ACK 5.7.3  p164  
            'get2kernelarginfo',               # ACK 5.7.3  p166                                  
            
            'release2kernel',                  # ACK 5.7.1  p159                                         
            'retain2kernel',                   # ACK 5.7.1  p159   
 
            'set2kernel',                      # ACK 
            'set2kernelcallback',              # ACK 
            'set2kernelarg',                   # ACK 5.7.2  p160 
                                          
            'trace2kernel',                    # ACK 
            
            'sense',                           # ACK
            'reason',                          # ACK
            'ace')                             # ACK   
    #"
    #/>
 
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-GPU2Kernel4Xi" usage = "
    # 
    #   print('(Step 110A/700) OF GPU2KERNEL4XI5TEST - CREATING KERNEL OBJECT OF GPU ', time4xi())        
    #   try:
    #       # ---------------------------------------------------------------------------------------------
    #       # CREATE WITH OBJECT                  
    #       # ---------------------------------------------------------------------------------------------        
    #       kernelobject2gpu = GPU2Kernel4Xi()
    #       print('(Step 110B/700) OF GPU2KERNEL4XI5TEST - CREATED KERNEL OBJECT OF GPU: ', kernelobject2gpu)                                  
    #       # 
    #   except:
    #       print('(Step 110Z/700) OF GPU2KERNEL4XI5TEST *** FAILED TO CREATE KERNEL OBJECT OF GPU') 
    # 
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def __init__(self, task4kernel2gpu = None):
        '''
        _init__ method initializes GPU2Kernel4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------
                    
        class2print = "GPU2Kernel4Xi"
        method2print = "__init__"
                     
        message2print = "{(Step 1A/33) started" + "; init-of-GPU2Kernel4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 

        message2print = "{(Step 1B/33) borning" + "; gpu2kernel2xi-of-GPU2Kernel4Xi-borning: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
                
        message2print = "{(Step 1C/33) inheriting" + "; object-inheriting-by-gpu2kernel2xi-of-GPU2Kernel4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'>
        
        message2print = "{(Step 1D/33) sharing" + "; classvariable2gpu-sharing-by-gpu2kernel2xi-of-GPU2Kernel4Xi: " + str(GPU2Kernel4Xi.classvariable2gpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4        
                                
        message2print = "{(Step 1E/33) passed" + "; task4kernel2gpu-passed-to-init-of-GPU2Kernel4Xi: " + str(task4kernel2gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None                      
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)             
        # -----------------------------------------------------------------------------------------
    
        message2print = "{(Step 2A/33) passing" + "; task4kernel2gpu-passing-to-create2kernel4gpu-by-init-of-GPU2Kernel4Xi: " + str(task4kernel2gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None  
                
        gpu2kernel2xi = create2kernel4gpu(task4kernel2gpu)
        
        message2print = "{(Step 2B/33) performed" + "; create2kernel4gpu-performed-by-init-of-GPU2Kernel4Xi: " + str(create2kernel4gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <function create2kernel4gpu at 0x00000000041E54C8>
       
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)             
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3A/33) activated" + "; gpu2kernel2xi-of-GPU2Kernel4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
                
        message2print = "{(Step 3B/33) inherited" + "; object-inherited-by-gpu2kernel2xi-of-GPU2Kernel4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        message2print = "{(Step 3C/33) shared" + "; classvariable2gpu-shared-by-gpu2kernel2xi-of-GPU2Kernel4Xi: " + str(GPU2Kernel4Xi.classvariable2gpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        #     
       
        message2print = "{(Step 3D/33) returning" + "; init-of-GPU2Kernel4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        

    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "add2kernel-of-GPU2Kernel4Xi" usage = "
    #  
    #   print('(Step 132A/700) OF GPU2KERNEL4XI5TEST - TRYING TO ADD KERNEL TO GPU  WITH METHOD', time4xi())         
    #   try:
    #       task4add2kernel ['CODE4KERNEL'] [folder2lib4gpu + 'gpu2dummy4xi.py'] = {}
    #       task4add2kernel ['CODE4KERNEL'] [folder2lib4gpu + 'gpu2dummy4xi.py'] [1] = '... profile = "11" />'
    #
    #       response4add2kernel = programobject2gpu.add2kernel(task4add2kernel) 
    #       print('(Step 132B/700) OF GPU2KERNEL4XI5TEST - ADDED KERNEL TO GPU  WITH METHOD', time4xi())
    #   except:
    #       print('(Step 132Z1/700) OF GPU2KERNEL4XI5TEST *** FAILED TO ADD KERNEL TO GPU WITH METHOD *** ', time4xi())
    #  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def add2kernel(self, task4add2kernel = None):
        '''
        adds kernel to GPU
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "GPU2Kernel4Xi"
        method2print = "add2kernel"
                     
        message2print = "{(Step 1A/33) started" + "; add2kernel-of-GPU2Kernel4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4add2kernel-passed-to-add2kernel-of-GPU2Kernel4Xi: " + str(task4add2kernel) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4add2kernel-passing-to-add2kernel4gpu-by-add2kernel-of-GPU2Kernel4Xi: " + str(task4add2kernel) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        response4add2kernel = add2kernel4gpu(task4add2kernel)

        message2print = "{(Step 2B/33) performed" + "; add2kernel4gpu-performed-by-add2kernel-of-GPU2Kernel4Xi: " + str(add2kernel4gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4add2kernel-returning-by-add2kernel-of-GPU2Kernel4Xi: " + str(response4add2kernel) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
                
        return response4add2kernel
                                
                            
    #</methodize4xi>    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "execute2task-of-GPU2Kernel4Xi" usage = "
    # 
    #   print('(Step 240A/700) OF GPU2KERNEL4XI5TEST - TRYING TO EXECUTE TASK OF GPU WITH METHOD ', time4xi())    
    #   try:                               
    #       response4execute2task = programobject2gpu.execute2task() 
    #       print('(Step 240B/700) OF GPU2KERNEL4XI5TEST - EXECUTED TASK OF GPU WITH METHOD ', time4xi())
    #   except:
    #       print('(Step 240Z1/700) OF GPU2KERNEL4XI5TEST *** FAILED TO EXECUTE TASK OF GPU WITH METHOD ', time4xi())
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def execute2task(self, task4execute = None):
        '''
        executes task of kernel of GPU
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "GPU2Kernel4Xi"
        method2print = "execute2task"
                     
        message2print = "{(Step 1A/33) started" + "; execute2task-of-GPU2Kernel4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4execute-passed-to-create-of-GPU2Kernel4Xi: " + str(task4execute) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4execute-passing-to-execute2task4gpu-by-execute2task-of-GPU2Kernel4Xi: " + str(task4execute) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #    
  
        gpu2program2xi = execute2task4gpu(task4execute)

        message2print = "{(Step 2B/33) performed" + "; execute2task4gpu-performed-by-execute2task-of-GPU2Kernel4Xi: " + str(execute2task4gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; gpu2program2xi-returning-by-execute2task-of-GPU2Kernel4Xi: " + str(gpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # 

        return gpu2program2xi
                     
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "execute2nativekernel-of-GPU2Kernel4Xi" usage = "
    # 
    #   print('(Step 121A1/700) OF GPU2KERNEL4XI5TEST - TRYING TO EXECUTE KERNEL OF GPU ', time4xi())      
    #   try:
    #       # ---------------------------------------------------------------------------------------------
    #       # COMPILE                        
    #       # ---------------------------------------------------------------------------------------------        
    #       response4compile = executeobject2gpu.execute2nativekernel()
    #       print('(Step 121A2/700) OF GPU2KERNEL4XI5TEST - EXECUTED KERNEL OF GPU', time4xi())                                   
    #   except:
    #       print('(Step 121AZ1/700) OF GPU2KERNEL4XI5TEST *** FAILED TO EXECUTE KERNEL OF GPU', time4xi())   
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def execute2nativekernel(self, task4execute2kernel = None):
        '''
        executes native kernel of GPU
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "GPU2Kernel4Xi"
        method2print = "execute2nativekernel"
                     
        message2print = "{(Step 1A/33) started" + "; execute2nativekernel-of-GPU2Kernel4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4execute2kernel-passed-to-create-of-GPU2Kernel4Xi: " + str(task4execute2kernel) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4execute2kernel-passing-to-execute2nativekernel4gpu-by-execute2nativekernel-of-GPU2Kernel4Xi: " + str(task4execute2kernel) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
  
        gpu2execute2xi = execute2nativekernel4gpu(task4execute2kernel)

        message2print = "{(Step 2B/33) performed" + "; execute2nativekernel4gpu-performed-by-execute2nativekernel-of-GPU2Kernel4Xi: " + str(execute2nativekernel4gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; gpu2execute2xi-returning-by-execute2nativekernel-of-GPU2Kernel4Xi: " + str(gpu2execute2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        return gpu2execute2xi
                                
    #                        
    #</methodize4xi>
        

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2kernelworkgroupinfo-of-GPU2Kernel4Xi" usage = "
    # 
    #   print('(Step 121C1/700) OF GPU2KERNEL4XI5TEST - TRYING TO GET WORK GROUP INFOR OF GPU WITH METHOD ', time4xi())      
    #   try:
    #      
    #       response4getworkgroupinfo = programobject2gpu.get2kernelworkgroupinfo()
    #       print('(Step 121C2/700) OF GPU2KERNEL4XI5TEST - GOT WORK GROUP INFOR OF GPU WITH METHOD ', time4xi())                                   
    #   except:
    #       print('(Step 121CZ1/700) OF GPU2KERNEL4XI5TEST *** FAILED TO GET WORK GROUP INFOR OF GPU WITH METHOD ', time4xi())  
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def get2kernelworkgroupinfo(self, task4getworkgroupinfo = None):
        '''
        get kernel work group info OF GPU
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "GPU2Kernel4Xi"
        method2print = "get2kernelworkgroupinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2kernelworkgroupinfo-of-GPU2Kernel4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4getworkgroupinfo-passed-to-create-of-GPU2Kernel4Xi: " + str(task4getworkgroupinfo) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4getworkgroupinfo-passing-to-get2kernelworkgroupinfo4gpu-by-get2kernelworkgroupinfo-of-GPU2Kernel4Xi: " + str(task4getworkgroupinfo) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
  
        gpu2program2xi = get2kernelworkgroupinfo4gpu(task4getworkgroupinfo)

        message2print = "{(Step 2B/33) performed" + "; get2kernelworkgroupinfo4gpu-performed-by-get2kernelworkgroupinfo-of-GPU2Kernel4Xi: " + str(get2kernelworkgroupinfo4gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; gpu2program2xi-returning-by-get2kernelworkgroupinfo-of-GPU2Kernel4Xi: " + str(gpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        return gpu2program2xi
                                
    #                        
    #</methodize4xi>
        

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create2kernel-of-GPU2Kernel4Xi" usage = "
    # 
    #   print('(Step 190A/700) OF GPU2KERNEL4XI5TEST - TRYING TO CREATE KERNEL FOR GPU ', time4xi()) 
    #   try:
    #       kernelobject2gpu = GPU2Kernel4Xi()                            
    #       response4create2kernel = kernelobject2gpu.create2kernel()
    #       print('(Step 190B/700) OF GPU2KERNEL4XI5TEST - CREATED KERNEL FOR GPU ', time4xi())
    #   except:
    #       print('(Step 190Z1/700) OF GPU2KERNEL4XI5TEST *** FAILED TO CREATE KERNEL FOR GPU ', time4xi())                                    
    #           
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def create2kernel(self, task4kernel2create = None):
        '''
        create2kernel method creates kernel objects of GPU2Kernel4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "GPU2Kernel4Xi"
        method2print = "create2kernel"
                     
        message2print = "{(Step 1A/33) started" + "; create2kernel-of-GPU2Kernel4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #  
            
        message2print = "{(Step 1B/33) passed" + "; task4kernel2create-passed-to-create-of-GPU2Kernel4Xi: " + str(task4kernel2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4kernel2create-passing-to-create2kernel4gpu-by-create2kernel-of-GPU2Kernel4Xi: " + str(task4kernel2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #  
  
        gpu2kernel2xi = create2kernel4gpu(task4kernel2create)

        message2print = "{(Step 2B/33) performed" + "; create2kernel4gpu-performed-by-create2kernel-of-GPU2Kernel4Xi: " + str(create2kernel4gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function create2kernel4gpu at 0x0000000003B0C5C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        keys2response4create = gpu2kernel2xi.keys() 
        message2print = "{(Step 33A/33) returning" + "; keys2response4create-returning-by-create2kernel-of-GPU2Kernel4Xi: " + str(keys2response4create) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                         
        # ['FUNCTION',
        #  'CODE2ERROR',
        #  'COUNT2REFERENCE',
        #  'DATA2CALLBACK',
        #  'BUFFER',
        #  'FUNCTION2CALLBACK',
        #  'ACK2RETURN',
        #  'QUEUE',
        #  'PLATFORM',
        #  'PROGRAM',
        #  'NACK2RETURN',
        #  'CONTEXT',
        #  'COMMANDQUEUE',
        #  'DEVICE',
        #  'EVENT',
        #  'PIPE']
        message2print = "{(Step 33B/33) returning" + "; function-returning-by-create2kernel-of-GPU2Kernel4Xi: " + str(gpu2kernel2xi['FUNCTION']) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
        # gpu2sampler4xi5test 
        
        return gpu2kernel2xi
                                
    #                        
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create2kernelsinprogram-of-GPU2Kernel4Xi" usage = "
    # 
    #   print('(Step 130A/700) OF GPU2KERNEL4XI5TEST - CREATING KERNELS IN PROGRM FOR GPU ', time4xi())      
    #   try:
    #       response4create2kernelsinprogram = kernelobject2gpu.create2kernelsinprogram()
    #       print('(Step 130A/700) OF GPU2KERNEL4XI5TEST - CREATED KERNELS IN PROGRM FOR GPU ', time4xi())
    #   except:
    #       print('(Step 130Z1/700) OF GPU2KERNEL4XI5TEST *** FAILED TO KERNELS IN PROGRM FOR GPU', time4xi())
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def create2kernelsinprogram(self, task4kernel2create = None):
        '''
        creates kernels in program of GPU
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "GPU2Kernel4Xi"
        method2print = "create2kernelsinprogram"
                     
        message2print = "{(Step 1A/33) started" + "; create2kernelsinprogram-of-GPU2Kernel4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4kernel2create-passed-to-create-of-GPU2Kernel4Xi: " + str(task4kernel2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4kernel2create-passing-to-create2kernelsinprogram4gpu-by-create2kernelsinprogram-of-GPU2Kernel4Xi: " + str(task4kernel2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
  
        gpu2kernel2xi = create2kernelsinprogram4gpu(task4kernel2create)

        message2print = "{(Step 2B/33) performed" + "; create2kernelsinprogram4gpu-performed-by-create2kernelsinprogram-of-GPU2Kernel4Xi: " + str(create2kernelsinprogram4gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function create2kernelsinprogram4gpu at 0x00000000040248C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; gpu2kernel2xi-returning-by-create2kernelsinprogram-of-GPU2Kernel4Xi: " + str(gpu2kernel2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # 

        return gpu2kernel2xi
                                
    #                        
    #</methodize4xi>
        

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "delete2kernel-of-GPU2Kernel4Xi" usage = "
    #  
    #   print('(Step 134A/700) OF GPU2KERNEL4XI5TEST - TRYING TO DELETE KERNEL IN PROGRAM OF GPU WITH METHOD ', time4xi())        
    #   try:
    #       task4delete2kernel = {}
    #       task4delete2kernel ['CODE4KERNEL'] = {}
    #       task4delete2kernel ['CODE4KERNEL'] [folder2lib4gpu + 'dummy3image4xi..py'] = 'Yes'
    #       task4delete2kernel ['SIZE4KERNEL'] = {}
    #       task4delete2kernel ['SIZE4KERNEL'] [folder2lib4gpu + 'dummy3image4xi..py'] = 'Yes'
    #       
    #       response4delete2kernel = programobject2gpu.delete2kernel(task4delete2kernel)
    #       print('(Step 134B/700) OF GPU2KERNEL4XI5TEST - DELETED DELETE KERNEL IN PROGRAM of GPUWITH METHOD ', time4xi())                                                                  
    #   except:
    #       print('(Step 134Z1/700) OF GPU2KERNEL4XI5TEST *** FAILED TO DELETE DELETE KERNEL IN PROGRAM OF GPU WITH METHOD *** ', time4xi())
    #                  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def delete2kernel(self, task4delete2kernel = None):
        '''
        deletes kernel in program of GPU
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "GPU2Kernel4Xi"
        method2print = "delete2kernel"
                     
        message2print = "{(Step 1A/33) started" + "; delete2kernel-of-GPU2Kernel4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4delete2kernel-passed-to-delete2kernel-of-GPU2Kernel4Xi: " + str(task4delete2kernel) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4delete2kernel-passing-to-delete2kernel4gpu-by-delete2kernel-of-GPU2Kernel4Xi: " + str(task4delete2kernel) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        response4delete2kernel = delete2kernel4gpu(task4delete2kernel)

        message2print = "{(Step 2B/33) performed" + "; delete2kernel4gpu-performed-by-delete2kernel-of-GPU2Kernel4Xi: " + str(delete2kernel4gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4delete2kernel-returning-by-delete2kernel-of-GPU2Kernel4Xi: " + str(response4delete2kernel) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
                
        return response4delete2kernel
                                
                            
    #</methodize4xi>    


        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2kernel-of-GPU2Kernel4Xi" usage = "
    # 
    #   ... prepare task4set2kernel ...
    #   print('(Step 158A/700) OF GPU2KERNEL4XI5TEST - TRYING TO SET KERNEL OF GPU: ', time4xi()) 
    #   try: 
    #       response4set2kernel = kernelobject2gpu.set2kernel(task4set2kernel)
    #       print('(Step 158B/700) OF GPU2KERNEL4XI5TEST - SET KERNEL OF GPU: ', time4xi())    
    #   except:
    #       print('(Step 158Z1/700) OF GPU2KERNEL4XI5TEST *** FAILED TO SET KERNEL OF GPU: ', time4xi())
    #       
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2kernel(self, task4set2kernel = None):
        '''
        sets kernel GPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "GPU2Kernel4Xi"
        method2print = "set2kernel"
                     
        message2print = "{(Step 1A/33) started" + "; set2kernel-of-GPU2Kernel4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4set2kernel-passed-to-set2kernel-of-GPU2Kernel4Xi: " + str(task4set2kernel) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2kernel-passing-to-set2kernel4gpu-by-set2kernel-of-GPU2Kernel4Xi: " + str(task4set2kernel) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        gpu2kernel2xi = set2kernel4gpu(task4set2kernel)

        message2print = "{(Step 2B/33) performed" + "; set2kernel4gpu-performed-by-set2kernel-of-GPU2Kernel4Xi: " + str(set2kernel4gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; gpu2kernel2xi-returning-by-set2kernel-of-GPU2Kernel4Xi: " + str(gpu2kernel2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
                
        return gpu2kernel2xi
                             
                            
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2kernelarg-of-GPU2Kernel4Xi" usage = "
    # 
    #   ... prepare task4set2kernelarg ...
    #   print('(Step 158A/700) OF GPU2KERNEL4XI5TEST - TRYING TO SET KERNEL ARGUMENTS FOR GPU: ', time4xi())
    #   try: 
    #       response4set2kernelarg = programobject2gpu.set2kernelarg(task4set2kernelarg)
    #       print('(Step 158B/700) OF GPU2KERNEL4XI5TEST - SET KERNEL ARGUMENTS FOR GPU ', time4xi())    
    #   except:
    #       print('(Step 158Z1/700) OF GPU2KERNEL4XI5TEST *** FAILED TO SET KERNEL ARGUMENTS FOR GPU ', time4xi())
    #       
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2kernelarg(self, task4set2kernelarg = None):
        '''
        sets kernel arguments for GPU
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "GPU2Kernel4Xi"
        method2print = "set2kernelarg"
                     
        message2print = "{(Step 1A/33) started" + "; set2kernelarg-of-GPU2Kernel4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4set2kernelarg-passed-to-set2kernelarg-of-GPU2Kernel4Xi: " + str(task4set2kernelarg) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2kernelarg-passing-to-set2kernelarg4gpu-by-set2kernelarg-of-GPU2Kernel4Xi: " + str(task4set2kernelarg) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        gpu2program2xi = set2kernelarg4gpu(task4set2kernelarg)

        message2print = "{(Step 2B/33) performed" + "; set2kernelarg4gpu-performed-by-set2kernelarg-of-GPU2Kernel4Xi: " + str(set2kernelarg4gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; gpu2program2xi-returning-by-set2kernelarg-of-GPU2Kernel4Xi: " + str(gpu2program2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
                
        return gpu2program2xi
                             
                            
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2kernelinfo-of-GPU2Kernel4Xi" usage = "
    #
    #   print('(Step 106A/700) OF GPU2KERNEL4XI5TEST - TRYING TO GET KERNEL INFO ', time4xi())    
    #   try:                               
    #       response4get2kernelinfo = kernelobject2gpu.get2kernelinfo()                                 
    #       print('(Step 106B/700) OF GPU2KERNEL4XI5TEST - GOT GET KERNEL INFO ', time4xi())                             
    #   except:
    #       print('(Step 106Z/700) OF GPU2KERNEL4XI5TEST *** FAILED TO GET KERNEL INFO ', time4xi()) 
    #   
    # 
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2kernelinfo(self, task4getinfo2kernel=None):
        '''
        gets kernel info of GPU 
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "GPU2Kernel4Xi"
        method2print = "get2kernelinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2kernelinfo-of-GPU2Kernel4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        gpu2kernel2xi = get2kernelinfo4gpu(task4getinfo2kernel)
        
        message2print = "{(Step 2A/33) returning" + "; gpu2kernel2xi-returning-by-get2kernelinfo-of-GPU2Kernel4Xi: " + str(gpu2kernel2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 

        return gpu2kernel2xi
                            
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2kernelarginfo-of-GPU2Kernel4Xi" usage = "
    #
    #   print('(Step 106A/700) OF GPU2KERNEL4XI5TEST - TRYING TO GET KERNEL ARGUMENT INFO OF GPU ', time4xi())    
    #   try:                               
    #       response4get2kernelarginfo = kernelobject2gpu.get2kernelarginfo()
    #       print('(Step 106B/700) OF GPU2KERNEL4XI5TEST - GOT KERNEL ARGUMENT INFO OF GPU ', time4xi())                                   
    #   except:
    #       print('(Step 106Z1/700) OF GPU2KERNEL4XI5TEST *** FAILED TO GET KERNEL ARGUMENT INFO OF GPU ', time4xi()) 
    # 
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2kernelarginfo(self, task4getinfo2kernel=None):
        '''
        gets kernel argument info of GPU
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "GPU2Kernel4Xi"
        method2print = "get2kernelarginfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2kernelarginfo-of-GPU2Kernel4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 05:01:22 11/17/11 Pacific Standard Time

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        gpu2kernel2xi = get2kernelarginfo4gpu(task4getinfo2kernel)
        
        message2print = "{(Step 2A/33) returning" + "; gpu2kernel2xi-returning-by-get2kernelarginfo-of-GPU2Kernel4Xi: " + str(gpu2kernel2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 

        return gpu2kernel2xi
                            
    #</methodize4xi>
   
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2buildinfo-of-GPU2Kernel4Xi" usage = "
    #
    #   print('(Step 106A1/700) OF GPU2BUILD4XI5TEST - TRYING TO GET BUILD INFO ', time4xi())    
    #   try:                               
    #                               
    #       response4get2buildinfo = programobject2gpu.get2buildinfo()
    #       print('(Step 106A2/700) OF GPU2BUILD4XI5TEST - GOT BUILD INFO ', time4xi())    
    #   except:
    #       print('(Step 106Z1/700) OF GPU2BUILD4XI5TEST *** FAILED TO GET BUILD INFO ', time4xi()) 
    # 
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2buildinfo(self, task4getinfo2build=None):
        '''
        get2buildinfo method gets objects of GPU2Kernel4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "GPU2Kernel4Xi"
        method2print = "get2buildinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2buildinfo-of-GPU2Kernel4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        gpu2build2xi = get2buildinfo4gpu(task4getinfo2build)
        
        message2print = "{(Step 2A/33) returning" + "; gpu2build2xi-returning-by-get2buildinfo-of-GPU2Kernel4Xi: " + str(gpu2build2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
        
        return gpu2build2xi
                            
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "release2kernel-of-GPU2Kernel4Xi" usage = "
    #
    #   print('(Step 170A/700) OF GPU2KERNEL4XI5TEST - RELEASING KERNEL ', time4xi())  
    #   task4release2kernel  = {}
    #   task4release2kernel  ['COUNT2REFERENCE'] = 1
    #   try:
    #       response4release2kernel = kernelobject2gpu.release2kernel(task4release2kernel)
    #       print('(Step 170B/700) OF GPU2KERNEL4XI5TEST - RELEASED KERNEL ', time4xi())
    #   except:
    #       print('(Step 170Z1/700) OF GPU2KERNEL4XI5TEST *** FAILED TO RELEASE KERNEL ', time4xi())                        
    #       keys2response = response4release2kernel.keys()
    #  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def release2kernel(self, task4release2kernel = None):
        '''
        releases GPU2Kernel4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "GPU2Kernel4Xi"
        method2print = "release2kernel"
                     
        message2print = "{(Step 1A/33) started" + "; release2kernel-of-GPU2Kernel4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4release2kernel-passed-to-release2kernel-of-GPU2Kernel4Xi: " + str(task4release2kernel) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4release2kernel-passing-to-release2kernel4gpu-by-release2kernel-of-GPU2Kernel4Xi: " + str(task4release2kernel) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        gpu2kernel2xi = release2kernel4gpu(task4release2kernel)

        message2print = "{(Step 2B/33) performed" + "; release2kernel4gpu-performed-by-release2kernel-of-GPU2Kernel4Xi: " + str(release2kernel4gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; gpu2kernel2xi-returning-by-release2kernel-of-GPU2Kernel4Xi: " + str(gpu2kernel2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #               
                
        return gpu2kernel2xi
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "retain2kernel-of-GPU2Kernel4Xi" usage = "
    #
    #   print('(Step 180A/700) OF GPU2KERNEL4XI5TEST - RETAINING KERNEL ', time4xi())
    #   task4retain2kernel  = {}
    #   task4retain2kernel  ['COUNT2REFERENCE'] = 1
    #   try:
    #       response4retain2kernel = kernelobject2gpu.retain2kernel(task4retain2kernel)
    #       print('(Step 180B/700) OF GPU2KERNEL4XI5TEST - RETAINED KERNEL ', time4xi())
    #   except:
    #       print('(Step 180Z1/700) OF GPU2KERNEL4XI5TEST *** FAILED TO RETAIN KERNEL ', time4xi())                    
    # 
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def retain2kernel(self, task4retain2kernel = None):
        '''
        retains GPU2Kernel4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "GPU2Kernel4Xi"
        method2print = "retain2kernel"
                     
        message2print = "{(Step 1A/33) started" + "; retain2kernel-of-GPU2Kernel4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
            
        message2print = "{(Step 1B/33) passed" + "; task4retain-passed-to-retain2kernel-of-GPU2Kernel4Xi: " + str(task4retain2kernel) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4retain-passing-to-retain2kernel4gpu-by-retain-of-GPU2Kernel4Xi: " + str(task4retain2kernel) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
  
        gpu2kernel2xi = retain2kernel4gpu(task4retain2kernel)

        message2print = "{(Step 2B/33) performed" + "; retain2kernel4gpu-performed-by-retain2kernel-of-GPU2Kernel4Xi: " + str(retain2kernel4gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; gpu2kernel2xi-returning-by-retain2kernel-of-GPU2Kernel4Xi: " + str(gpu2kernel2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
            
                
        return gpu2kernel2xi
                                                           
    #</methodize4xi>   
    
 
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "trace2kernel-of-GPU2Kernel4Xi" usage = "
    #     
    #   print('(Step 210A/25) OF GPU2KERNEL4XI5TEST - TRYING TO TRACE GPU: ', time4xi())      
    #   trace2gpu = None
    #   try:
    #       trace2gpu = programobject2gpu.trace2kernel() 
    #               print('(Step 210B/25) OF GPU2KERNEL4XI5TEST - TRACE2GPU: ', trace2gpu)  
    #       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2kernel4gpu-failed {}; time: 04:40:24 11/16/11 Pacific Standard Time}', 'CLASS': 'trace2kernel4gpu', 'METHOD': 'trace2kernel4gpu'}
    #        except:
    #           print('(Step 210Z1/25) OF GPU2KERNEL4XI5TEST *** FAILED TO TRACE GPU: ', trace2gpu) 
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def trace2kernel(self, task4trace2kernel = None):
        '''
        traces GPU2Kernel4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "GPU2Kernel4Xi"
        method2print = "trace2kernel"
                     
        message2print = "{(Step 1A/33) started" + "; trace2kernel-of-GPU2Kernel4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
         
            
        message2print = "{(Step 1B/33) passed" + "; task4trace2kernel-passed-to-trace-of-GPU2Kernel4Xi: " + str(task4trace2kernel) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
         
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4trace2kernel-passing-to-trace2kernel4gpu-by-trace2kernel-of-GPU2Kernel4Xi: " + str(task4trace2kernel) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
         
  
        trace2xi = trace2kernel4gpu(task4trace2kernel)

        message2print = "{(Step 2B/33) performed" + "; trace2kernel4gpu-performed-by-trace2kernel-of-GPU2Kernel4Xi: " + str(trace2kernel4gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace2xi-returning-by-trace2kernel-of-GPU2Kernel4Xi: " + str(trace2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #       
                
        return trace2xi
                                                           
    #</methodize4xi>       

                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "execute2kernel-of-GPU2Kernel4Xi" usage = "
    #        
    #   print('(Step 220A/25) OF GPU2KERNEL4XI5TEST - TRYING TO UNLOAD COMPILER OF GPU: ', time4xi())      
    #   try:
    #       response4execute2kernel = programobject2gpu.execute2kernel() 
    #       print('(Step 220B/25) OF GPU2KERNEL4XI5TEST - UNLOADED COMPILER OF GPU: ', response4execute2kernel)  
    #       # 
    #   except:
    #       print('(Step 220Z1/25) OF GPU2KERNEL4XI5TEST *** FAILED TO TO UNLOAD COMPILER OF GPU: ', time4xi()) 
    #    
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def execute2kernel(self, task4execute2kernel = None):
        '''
        executes kernel
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "GPU2Kernel4Xi"
        method2print = "execute2kernel"
                     
        message2print = "{(Step 1A/33) started" + "; execute2kernel-of-GPU2Kernel4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
         
            
        message2print = "{(Step 1B/33) passed" + "; task4execute2kernel-passed-to-trace-of-GPU2Kernel4Xi: " + str(task4execute2kernel) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
         
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4execute2kernel-passing-to-execute2kernel4gpu-by-execute2kernel-of-GPU2Kernel4Xi: " + str(task4execute2kernel) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
         
  
        trace2xi = execute2kernel4gpu(task4execute2kernel)

        message2print = "{(Step 2B/33) performed" + "; execute2kernel4gpu-performed-by-execute2kernel-of-GPU2Kernel4Xi: " + str(execute2kernel4gpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace2xi-returning-by-execute2kernel-of-GPU2Kernel4Xi: " + str(trace2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #       
                
        return trace2xi
                                                           
    #</methodize4xi>       

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2kernelcallback-of-GPU2Kernel4Xi" usage = "
    #
    #   print('(Step 200A/700) OF GPU2KERNEL4XI5TEST - TRYING TO SET KERNELCALLBACK WITH METHOD OF OBJECT XI2GPU: ', programobject2gpu) 
    #   try:
    #       callback2gpu = programobject2gpu.set2kernelcallback()                                  
    #       print('(Step 200B/700) OF GPU2KERNEL4XI5TEST - SET KERNELWITH CALLBACK TO: ', callback2gpu)
    #       # 
    #   except:
    #       print('(Step 200Z/700) OF GPU2KERNEL4XI5TEST *** FAILED TO SET KERNELCALLBACK WITH METHOD ***', time4xi())
    #
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2kernelcallback(self, task4set2kernelcallback = None): 
        '''
        set2kernelcallback of GPU2Kernel4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "GPU2Kernel4Xi"
        method2print = "set2kernelcallback"
                     
        message2print = "{(Step 1A/33) started" + "; set2kernelcallback-of-GPU2Kernel4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
            
        message2print = "{(Step 1B/33) passed" + "; task4set2kernelcallback-passed-to-set2kernelcallback-of-GPU2Kernel4Xi: " + str(task4set2kernelcallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
           
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2kernelcallback-passing-to-set2kernelcallback-by-set2kernelcallback-of-GPU2Kernel4Xi: " + str(task4set2kernelcallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        
        callback2gpu4program = {} 
        try:
            callback2gpu4program = set2kernelcallback4gpu (task4set2kernelcallback)

            
        except:
            message2print = "{(Step 4A/33) failed" + "; callback2gpu4program-failed-by-set2kernelcallback-of-GPU2Kernel4Xi: " + str(callback2gpu4program) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                            
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; callback2gpu4program-returning-by-set2kernelcallback-of-GPU2Kernel4Xi: " + str(callback2gpu4program) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # <function trace2kernel4gpu at 0x00000000038CF5C8> 
        
                                             
        return callback2gpu4program
                                                           
    #</methodize4xi>     
    
                                                              
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-GPU2Kernel4Xi" usage = "
    #
    #   print('(Step 310A/700) OF GPU2KERNEL4XI5TEST - SENSING ', time4xi())  
    #   try:
    #       print(programobject2gpu.sense())
    #       print('(Step 310B/700) OF GPU2KERNEL4XI5TEST - SENSED ', time4xi()) 
    #       # 
    #   except:
    #       print('(Step 310Z/700) OF GPU2KERNEL4XI5TEST *** FAILED TO SENSE ', time4xi())     
    #
    # ">
    
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses GPU2Kernel4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'GPU2Kernel4Xi'
        method2print = 'sense'
        
                     
        message2print = "{(Step 1A/33) started" + "; sense-of-GPU2Kernel4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        
        message2print = "{(Step 2A/33) sensing" + "; GPU2Kernel4Xi-sensing-by-sense-of-GPU2Kernel4Xi: " + str(GPU2Kernel4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # 
            
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3A/33) got" + "; kb2xi-got-by-sense-of-GPU2Kernel4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
         
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['classvariable2gpu-of-GPU2Kernel4Xi'] = GPU2Kernel4Xi.classvariable2gpu
        except:
            message2print = "{(Step 4A/33) yielded" + "; kb2xi-yielded-by-sense-of-GPU2Kernel4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
      
        message2print = "{(Step 5A/33) sensed" + "; GPU2Kernel4Xi-sensed-by-sense-of-GPU2Kernel4Xi: " + str(GPU2Kernel4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # 
        
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        
        message2print = "{(Step 6A/33) returning" + "; kb2xi-returning-by-sense-of-GPU2Kernel4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
                   
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-GPU2Kernel4Xi" usage = "
    # 
    #   print('(Step 320A/700) OF GPU2KERNEL4XI5TEST - REASONING ', time4xi())
    #   try:
    #       print(programobject2gpu.reason())
    #       print('(Step 320B/700) OF GPU2KERNEL4XI5TEST - REASONED ', time4xi())
    #       # 
    #   except:
    #       print('(Step 320Z/700) OF GPU2KERNEL4XI5TEST *** FAILED TO REASON ', time4xi()) 
    #
    #">
    
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons GPU2Kernel4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'GPU2Kernel4Xi'
        method2print = 'reason'

                     
        message2print = "{(Step 1A/33) started" + "; reason-of-GPU2Kernel4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) reasoning" + "; GPU2Kernel4Xi-reasoning-by-reason-of-GPU2Kernel4Xi: " + str(GPU2Kernel4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{(Step 3A/33) got" + "; kb4xi-got-by-reason-of-GPU2Kernel4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
             
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        try:
            kb4xi['time-reason-of-GPU2Kernel4Xi'] = time4xi() #adding time
        except:
            message2print = "{(Step 4Z/33) yielded" + "; kb4xi-yielded-by-reason-of-GPU2Kernel4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
          
        
        message2print = "{(Step 5A/33) reasoned" + "; GPU2Kernel4Xi-reasoned-by-reason-of-GPU2Kernel4Xi: " + str(GPU2Kernel4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # 
        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        
        message2print = "{(Step 6A/33) returning" + "; kb4xi-returning-by-reason-of-GPU2Kernel4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        #           
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-GPU2Kernel4Xi" usage = "
    #
    #   print('(Step 330A/700) OF GPU2KERNEL4XI5TEST - ACING ', time4xi())   
    #   try:
    #       print(programobject2gpu.ace())
    #       print('(Step 330B/700) OF GPU2KERNEL4XI5TEST - ACED ', time4xi())                                                 
    #       # 
    #   except:
    #       print('(Step 330Z/700) OF GPU2KERNEL4XI5TEST *** FAILED TO ACE ', time4xi())                                 
    #    
    #">
    
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces GPU2Kernel4Xi class objects 
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------       
        
        
        class2print = 'GPU2Kernel4Xi'
        method2print = 'ace'
        
                     
        message2print = "{(Step 1A/33) started" + "; ace-of-GPU2Kernel4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) acing" + "; GPU2Kernel4Xi-acing-by-ace-of-GPU2Kernel4Xi: " + str(GPU2Kernel4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #      
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3A/33) got" + "; kb8xi-got-by-ace-of-GPU2Kernel4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        try:
             kb8xi['cores-of-GPU2Kernel4Xi'] = GPU2Kernel4Xi.classvariable2gpu
        except:
             message2print = "{(Step 4A/33) yielded" + "; kb8xi-yielded-by-ace-of-GPU2Kernel4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
             print4xi(class2print, method2print, message2print) 
  
        message2print = "{(Step 5A/33) aced" + "; GPU2Kernel4Xi-aced-by-ace-of-GPU2Kernel4Xi: " + str(GPU2Kernel4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'gpu2codeprogram4xi.GPU2Kernel4Xi'> 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 6A/33) returning" + "; kb8xi-returning-by-ace-of-GPU2Kernel4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
#</class4xi>

# =================================================================================================
#<class4xi class = "DirectoryWalker" base = "None" child = "None" synopsis = "a forward iterator that traverses a directory tree" usage = "
#   to review all tests"
#>
# =================================================================================================
class DirectoryWalker:

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init" usage = "initializes DirectoryWalker"> 
    # ---------------------------------------------------------------------------------------------
    def __init__(self, directory):
        self.stack = [directory]
        self.files = []
        self.index = 0
        
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "getitem" usage = "getting item"> 
    # ---------------------------------------------------------------------------------------------
    def __getitem__(self, index):
        while 1:
            # ------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------          
            try:
                # --------------------------------------------------------------------------------- 
                #
                # ---------------------------------------------------------------------------------             
                file = self.files[self.index]
                self.index = self.index + 1
            except IndexError:
                # --------------------------------------------------------------------------------- 
                # pop next directory from stack
                # ---------------------------------------------------------------------------------             
                self.directory = self.stack.pop()
                self.files = os.listdir(self.directory)
                self.index = 0
            else:
                # --------------------------------------------------------------------------------- 
                # got a filename
                # ---------------------------------------------------------------------------------             
                fullname = os.path.join(self.directory, file)
                if os.path.isdir(fullname) and not os.path.islink(fullname):
                    self.stack.append(fullname)
                return fullname
    #</methodize4xi>
#</class4xi>

#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   gpu2codeprogram4xi5test.txt
#   sense-gpu2codeprogram4xi5test.xml
#   reason-gpu2codeprogram4xi5test.xml
#   ace-gpu2codeprogram4xi5test.xml
#"
#>

#<acknowledge4xi acknowledgement = "33" tester = "Small & Smart"/>

# -------------------------------------------------------------------------------------------------
# to-be-done                              
# -------------------------------------------------------------------------------------------------    
#<do4xi do = "
# nothing
#"
#/>

#<note4xi note = "
# parse4xi.py - to parse this module
# "/>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of gpu2kernel4xi.py                              
# ------------------------------------------------------------------------------------------------- 
#"
#/>
